@@ -79,13 +79,7 @@ if (3, 0) <= sys.version_info[:2] <= (3, 5):
 
 warnings.filterwarnings('ignore', module='cryptography|OpenSSL', message=(
     'Python 2 is no longer supported by the Python core team. '
-    'Support for it is now deprecated in cryptography, '
-    'and will be removed in a future release.'))
-warnings.filterwarnings('ignore', module='cryptography|OpenSSL', message=(
-    'Python 2 is no longer supported by the Python core team. '
-    'Support for it is now deprecated in cryptography, '
-    'and will be removed in the next release.'))
+    'Support for it is now deprecated in cryptography'))
 warnings.filterwarnings('ignore', message=(
     'Python 3.6 is no longer supported by the Python core team. '
-    'Therefore, support for it is deprecated in cryptography '
-    'and will be removed in a future release.'))
+    'Therefore, support for it is deprecated in cryptography'))
