@@ -15,7 +15,6 @@ import itertools
 import json
 import os
 import sqlite3
-from hashlib import md5
 from collections import defaultdict
 
 from six.moves import urllib
@@ -32,6 +31,7 @@ from swift.obj.diskfile import get_data_dir, read_metadata, DATADIR_BASE, \
     extract_policy
 from swift.common.storage_policy import POLICIES
 from swift.common.middleware.crypto.crypto_utils import load_crypto_meta
+from swift.common.utils import md5
 
 
 class InfoSystemExit(Exception):
@@ -93,11 +93,12 @@ def curl_head_command(ip, port, device, part, target, policy_index):
     else:
         formatted_ip = ip
 
-    cmd = 'curl --path-as-is -g -I -XHEAD "http://%s:%s/%s/%s/%s"' % (
+    cmd = 'curl -g -I -XHEAD "http://%s:%s/%s/%s/%s"' % (
         formatted_ip, port, device, part, urllib.parse.quote(target))
     if policy_index is not None:
         cmd += ' -H "%s: %s"' % ('X-Backend-Storage-Policy-Index',
                                  policy_index)
+    cmd += ' --path-as-is'
     return cmd
 
 
@@ -545,7 +546,7 @@ def print_obj(datafile, check_etag=True, swift_dir='/etc/swift',
         # Optional integrity check; it's useful, but slow.
         file_len = None
         if check_etag:
-            h = md5()
+            h = md5(usedforsecurity=False)
             file_len = 0
             while True:
                 data = fp.read(64 * 1024)
