@@ -212,6 +212,32 @@ def _set_weight_values(devs, weight, opts):
                                        dev['weight']))
 
 
+def _set_region_values(devs, region, opts):
+
+    input_question = 'Are you sure you want to update the region for these ' \
+                     '%s devices? (y/N) ' % len(devs)
+    abort_msg = 'Aborting device modifications'
+    check_devs(devs, input_question, opts, abort_msg)
+
+    for dev in devs:
+        builder.set_dev_region(dev['id'], region)
+        print('%s region set to %s' % (format_device(dev),
+                                       dev['region']))
+
+
+def _set_zone_values(devs, zone, opts):
+
+    input_question = 'Are you sure you want to update the zone for these ' \
+                     '%s devices? (y/N) ' % len(devs)
+    abort_msg = 'Aborting device modifications'
+    check_devs(devs, input_question, opts, abort_msg)
+
+    for dev in devs:
+        builder.set_dev_zone(dev['id'], zone)
+        print('%s zone set to %s' % (format_device(dev),
+                                     dev['zone']))
+
+
 def _parse_set_weight_values(argvish):
 
     new_cmd_format, opts, args = validate_args(argvish)
@@ -301,6 +327,76 @@ def calculate_change_value(change_value, change, v_name, v_name_port):
     return change_value
 
 
+def _parse_set_region_values(argvish):
+
+    new_cmd_format, opts, args = validate_args(argvish)
+
+    # We'll either parse the all-in-one-string format or the
+    # --options format,
+    # but not both. If both are specified, raise an error.
+    try:
+        devs = []
+        if not new_cmd_format:
+            if len(args) % 2 != 0:
+                print(Commands.set_region.__doc__.strip())
+                exit(EXIT_ERROR)
+
+            devs_and_regions = izip(islice(argvish, 0, len(argvish), 2),
+                                    islice(argvish, 1, len(argvish), 2))
+            for devstr, regionstr in devs_and_regions:
+                devs.extend(builder.search_devs(
+                    parse_search_value(devstr)) or [])
+                region = int(regionstr)
+                _set_region_values(devs, region, opts)
+        else:
+            if len(args) != 1:
+                print(Commands.set_region.__doc__.strip())
+                exit(EXIT_ERROR)
+
+            devs.extend(builder.search_devs(
+                parse_search_values_from_opts(opts)) or [])
+            region = int(args[0])
+            _set_region_values(devs, region, opts)
+    except ValueError as e:
+        print(e)
+        exit(EXIT_ERROR)
+
+
+def _parse_set_zone_values(argvish):
+
+    new_cmd_format, opts, args = validate_args(argvish)
+
+    # We'll either parse the all-in-one-string format or the
+    # --options format,
+    # but not both. If both are specified, raise an error.
+    try:
+        devs = []
+        if not new_cmd_format:
+            if len(args) % 2 != 0:
+                print(Commands.set_zone.__doc__.strip())
+                exit(EXIT_ERROR)
+
+            devs_and_zones = izip(islice(argvish, 0, len(argvish), 2),
+                                  islice(argvish, 1, len(argvish), 2))
+            for devstr, zonestr in devs_and_zones:
+                devs.extend(builder.search_devs(
+                    parse_search_value(devstr)) or [])
+                zone = int(zonestr)
+                _set_zone_values(devs, zone, opts)
+        else:
+            if len(args) != 1:
+                print(Commands.set_zone.__doc__.strip())
+                exit(EXIT_ERROR)
+
+            devs.extend(builder.search_devs(
+                parse_search_values_from_opts(opts)) or [])
+            zone = int(args[0])
+            _set_zone_values(devs, zone, opts)
+    except ValueError as e:
+        print(e)
+        exit(EXIT_ERROR)
+
+
 def _parse_set_info_values(argvish):
 
     new_cmd_format, opts, args = validate_args(argvish)
@@ -380,6 +476,7 @@ def _make_display_device_table(builder):
     rep_ip_width = 14
     rep_port_width = 4
     ip_ipv6 = rep_ipv6 = False
+    weight_width = 6
     for dev in builder._iter_devs():
         if is_valid_ipv6(dev['ip']):
             ip_ipv6 = True
@@ -390,6 +487,8 @@ def _make_display_device_table(builder):
         port_width = max(len(str(dev['port'])), port_width)
         rep_port_width = max(len(str(dev['replication_port'])),
                              rep_port_width)
+        weight_width = max(len('%6.02f' % dev['weight']),
+                           weight_width)
     if ip_ipv6:
         ip_width += 2
     if rep_ipv6:
@@ -397,7 +496,7 @@ def _make_display_device_table(builder):
     header_line = ('Devices:%5s %6s %4s %' + str(ip_width)
                    + 's:%-' + str(port_width) + 's %' +
                    str(rep_ip_width) + 's:%-' + str(rep_port_width) +
-                   's %5s %6s %10s %7s %5s %s') % (
+                   's %5s %' + str(weight_width) + 's %10s %7s %5s %s') % (
                        'id', 'region', 'zone', 'ip address',
                        'port', 'replication ip', 'port', 'name',
                        'weight', 'partitions', 'balance', 'flags',
@@ -415,7 +514,8 @@ def _make_display_device_table(builder):
                                  '%', str(ip_width), 's:%-',
                                  str(port_width), 'd ', '%',
                                  str(rep_ip_width), 's', ':%-',
-                                 str(rep_port_width), 'd %5s %6.02f'
+                                 str(rep_port_width), 'd %5s %',
+                                 str(weight_width), '.02f'
                                  ' %10s %7.02f %5s %s'])
         args = (dev['id'], dev['region'], dev['zone'], dev_ip, dev['port'],
                 dev_replication_ip, dev['replication_port'], dev['device'],
@@ -681,7 +781,7 @@ swift-ring-builder <builder_file> add
         if builder.next_part_power:
             print('Partition power increase in progress. You need ')
             print('to finish the increase first before adding devices.')
-            exit(EXIT_WARNING)
+            exit(EXIT_ERROR)
 
         try:
             for new_dev in _parse_add_values(argv[3:]):
@@ -745,6 +845,75 @@ swift-ring-builder <builder_file> set_weight
         builder.save(builder_file)
         exit(EXIT_SUCCESS)
 
+    @staticmethod
+    def set_region():
+        """
+swift-ring-builder <builder_file> set_region <search-value> <region>
+    [<search-value> <region] ...
+
+or
+
+swift-ring-builder <builder_file> set_region
+    --region <region> --zone <zone> --ip <ip or hostname> --port <port>
+    --replication-ip <r_ip or r_hostname> --replication-port <r_port>
+    --device <device_name> --meta <meta> <new region> [--yes]
+
+    Where <r_ip>, <r_hostname> and <r_port> are replication ip, hostname
+    and port.
+    Any of the options are optional in both cases.
+
+    Resets the devices' regions. No partitions will be reassigned to or from
+    the device until after running 'rebalance'. This is so you can make
+    multiple device changes and rebalance them all just once.
+
+    Option --yes assume a yes response to all questions.
+        """
+        if len(argv) < 5:
+            print(Commands.set_region.__doc__.strip())
+            print()
+            print(parse_search_value.__doc__.strip())
+            exit(EXIT_ERROR)
+
+        _parse_set_region_values(argv[3:])
+
+        builder.save(builder_file)
+        exit(EXIT_SUCCESS)
+
+    @staticmethod
+    def set_zone():
+        """
+swift-ring-builder <builder_file> set_zone <search-value> <zone>
+    [<search-value> <zone] ...
+
+or
+
+swift-ring-builder <builder_file> set_zone
+    --region <region> --zone <zone> --ip <ip or hostname> --port <port>
+    --replication-ip <r_ip or r_hostname> --replication-port <r_port>
+    --device <device_name> --meta <meta> <new zone> [--yes]
+
+    Where <r_ip>, <r_hostname> and <r_port> are replication ip, hostname
+    and port.
+    Any of the options are optional in both cases.
+
+    Resets the devices' zones. No partitions will be reassigned to or from
+    the device until after running 'rebalance'. This is so you can make
+    multiple device changes and rebalance them all just once.
+
+    Option --yes assume a yes response to all questions.
+        """
+        # if len(argv) < 5 or len(argv) % 2 != 1:
+        if len(argv) < 5:
+            print(Commands.set_zone.__doc__.strip())
+            print()
+            print(parse_search_value.__doc__.strip())
+            exit(EXIT_ERROR)
+
+        _parse_set_zone_values(argv[3:])
+
+        builder.save(builder_file)
+        exit(EXIT_SUCCESS)
+
     @staticmethod
     def set_info():
         """
@@ -831,7 +1000,7 @@ swift-ring-builder <builder_file> remove
         if builder.next_part_power:
             print('Partition power increase in progress. You need ')
             print('to finish the increase first before removing devices.')
-            exit(EXIT_WARNING)
+            exit(EXIT_ERROR)
 
         devs, opts = _parse_remove_values(argv[3:])
 
@@ -898,7 +1067,7 @@ swift-ring-builder <builder_file> rebalance [options]
         if builder.next_part_power:
             print('Partition power increase in progress.')
             print('You need to finish the increase first before rebalancing.')
-            exit(EXIT_WARNING)
+            exit(EXIT_ERROR)
 
         devs_changed = builder.devs_changed
         min_part_seconds_left = builder.min_part_seconds_left
@@ -999,7 +1168,7 @@ swift-ring-builder <builder_file> rebalance [options]
 
     @staticmethod
     def dispersion():
-        """
+        r"""
 swift-ring-builder <builder_file> dispersion <search_filter> [options]
 
     Output report on dispersion.
@@ -1167,7 +1336,7 @@ swift-ring-builder <ring_file> write_builder [min_part_hours]
             'parts': ring.partition_count,
             'devs': ring.devs,
             'devs_changed': False,
-            'version': 0,
+            'version': ring.version or 0,
             '_replica2part2dev': ring._replica2part2dev_id,
             '_last_part_moves_epoch': None,
             '_last_part_moves': None,
@@ -1493,7 +1662,7 @@ def main(arguments=None):
         print(msg)
         exit(EXIT_ERROR)
     except (exceptions.FileNotFoundError, exceptions.PermissionError) as e:
-        if len(argv) < 3 or argv[2] not in('create', 'write_builder'):
+        if len(argv) < 3 or argv[2] not in ('create', 'write_builder'):
             print(e)
             exit(EXIT_ERROR)
     except Exception as e:
