@@ -138,13 +138,14 @@ class DaemonStrategy(object):
 
         def kill_children(*args):
             self.running = False
-            self.logger.info('SIGTERM received')
+            self.logger.notice('SIGTERM received (%s)', os.getpid())
             signal.signal(signal.SIGTERM, signal.SIG_IGN)
             os.killpg(0, signal.SIGTERM)
             os._exit(0)
 
         signal.signal(signal.SIGTERM, kill_children)
         self.running = True
+        utils.systemd_notify(self.logger)
 
     def _run_inline(self, once=False, **kwargs):
         """Run the daemon"""
