@@ -113,7 +113,11 @@ class DeviceUnavailable(SwiftException):
     pass
 
 
-class InvalidAccountInfo(SwiftException):
+class DatabaseAuditorException(SwiftException):
+    pass
+
+
+class InvalidAccountInfo(DatabaseAuditorException):
     pass
 
 
@@ -125,6 +129,10 @@ class ChunkReadError(SwiftException):
     pass
 
 
+class ShortReadError(SwiftException):
+    pass
+
+
 class ChunkReadTimeout(Timeout):
     pass
 
@@ -211,6 +219,10 @@ class ReplicationLockTimeout(LockTimeout):
     pass
 
 
+class PartitionLockTimeout(LockTimeout):
+    pass
+
+
 class MimeInvalid(SwiftException):
     pass
 
@@ -227,6 +239,10 @@ class UnknownSecretIdError(EncryptionException):
     pass
 
 
+class QuarantineRequest(SwiftException):
+    pass
+
+
 class ClientException(Exception):
 
     def __init__(self, msg, http_scheme='', http_host='', http_port='',
