@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from eventlet import sleep, Timeout
+from eventlet import sleep, Timeout, spawn
 from eventlet.green import httplib, socket
 import json
 import six
@@ -25,14 +25,13 @@ import zlib
 from time import gmtime, strftime, time
 from zlib import compressobj
 
-from swift.common.constraints import AUTO_CREATE_ACCOUNT_PREFIX
 from swift.common.exceptions import ClientException
 from swift.common.http import (HTTP_NOT_FOUND, HTTP_MULTIPLE_CHOICES,
                                is_client_error, is_server_error)
 from swift.common.request_helpers import USE_REPLICATION_NETWORK_HEADER
 from swift.common.swob import Request, bytes_to_wsgi
-from swift.common.utils import quote, closing_if_possible
-from swift.common.wsgi import loadapp, pipeline_property
+from swift.common.utils import quote, close_if_possible, drain_and_close
+from swift.common.wsgi import loadapp
 
 if six.PY3:
     from eventlet.green.urllib import request as urllib2
@@ -145,23 +144,28 @@ class InternalClient(object):
     :param user_agent: User agent to be sent to requests to Swift.
     :param request_tries: Number of tries before InternalClient.make_request()
                           gives up.
+    :param global_conf: a dict of options to update the loaded proxy config.
+        Options in ``global_conf`` will override those in ``conf_path`` except
+        where the ``conf_path`` option is preceded by ``set``.
+    :param app: Optionally provide a WSGI app for the internal client to use.
     """
 
     def __init__(self, conf_path, user_agent, request_tries,
-                 allow_modify_pipeline=False, use_replication_network=False):
+                 allow_modify_pipeline=False, use_replication_network=False,
+                 global_conf=None, app=None):
         if request_tries < 1:
             raise ValueError('request_tries must be positive')
-        self.app = loadapp(conf_path,
-                           allow_modify_pipeline=allow_modify_pipeline)
-        self.user_agent = user_agent
+        self.app = app or loadapp(conf_path, global_conf=global_conf,
+                                  allow_modify_pipeline=allow_modify_pipeline,)
+        self.user_agent = \
+            self.app._pipeline_final_app.backend_user_agent = user_agent
         self.request_tries = request_tries
         self.use_replication_network = use_replication_network
-
-    get_object_ring = pipeline_property('get_object_ring')
-    container_ring = pipeline_property('container_ring')
-    account_ring = pipeline_property('account_ring')
-    auto_create_account_prefix = pipeline_property(
-        'auto_create_account_prefix', default=AUTO_CREATE_ACCOUNT_PREFIX)
+        self.get_object_ring = self.app._pipeline_final_app.get_object_ring
+        self.container_ring = self.app._pipeline_final_app.container_ring
+        self.account_ring = self.app._pipeline_final_app.account_ring
+        self.auto_create_account_prefix = \
+            self.app._pipeline_final_app.auto_create_account_prefix
 
     def make_request(
             self, method, path, headers, acceptable_statuses, body_file=None,
@@ -202,7 +206,8 @@ class InternalClient(object):
             if params:
                 req.params = params
             try:
-                resp = req.get_response(self.app)
+                # execute in a separate greenthread to not polute corolocals
+                resp = spawn(req.get_response, self.app).wait()
             except (Exception, Timeout):
                 exc_type, exc_value, exc_traceback = exc_info()
             else:
@@ -215,13 +220,13 @@ class InternalClient(object):
             # sleep only between tries, not after each one
             if attempt < self.request_tries - 1:
                 if resp:
-                    # always close any resp.app_iter before we discard it
-                    with closing_if_possible(resp.app_iter):
-                        # for non 2XX requests it's safe and useful to drain
-                        # the response body so we log the correct status code
-                        if resp.status_int // 100 != 2:
-                            for iter_body in resp.app_iter:
-                                pass
+                    # for non 2XX requests it's safe and useful to drain
+                    # the response body so we log the correct status code
+                    if resp.status_int // 100 != 2:
+                        drain_and_close(resp)
+                    else:
+                        # Just close; the 499 is appropriate
+                        close_if_possible(resp.app_iter)
                 sleep(2 ** (attempt + 1))
         if resp:
             msg = 'Unexpected response: %s' % resp.status
@@ -234,6 +239,12 @@ class InternalClient(object):
             # To make pep8 tool happy, in place of raise t, v, tb:
             six.reraise(exc_type, exc_value, exc_traceback)
 
+    def handle_request(self, *args, **kwargs):
+        resp = self.make_request(*args, **kwargs)
+        # Drain the response body to prevent unexpected disconnect
+        # in proxy-server
+        drain_and_close(resp)
+
     def _get_metadata(
             self, path, metadata_prefix='', acceptable_statuses=(2,),
             headers=None, params=None):
@@ -363,7 +374,7 @@ class InternalClient(object):
                 headers[k] = v
             else:
                 headers['%s%s' % (metadata_prefix, k)] = v
-        self.make_request('POST', path, headers, acceptable_statuses)
+        self.handle_request('POST', path, headers, acceptable_statuses)
 
     # account methods
 
@@ -402,7 +413,7 @@ class InternalClient(object):
                            unexpected way.
         """
         path = self.make_path(account)
-        self.make_request('PUT', path, {}, (201, 202))
+        self.handle_request('PUT', path, {}, (201, 202))
 
     def delete_account(self, account, acceptable_statuses=(2, HTTP_NOT_FOUND)):
         """
@@ -417,7 +428,7 @@ class InternalClient(object):
                            unexpected way.
         """
         path = self.make_path(account)
-        self.make_request('DELETE', path, {}, acceptable_statuses)
+        self.handle_request('DELETE', path, {}, acceptable_statuses)
 
     def get_account_info(
             self, account, acceptable_statuses=(2, HTTP_NOT_FOUND)):
@@ -531,7 +542,7 @@ class InternalClient(object):
 
         headers = headers or {}
         path = self.make_path(account, container)
-        self.make_request('PUT', path, headers, acceptable_statuses)
+        self.handle_request('PUT', path, headers, acceptable_statuses)
 
     def delete_container(
             self, account, container, headers=None,
@@ -552,7 +563,7 @@ class InternalClient(object):
 
         headers = headers or {}
         path = self.make_path(account, container)
-        self.make_request('DELETE', path, headers, acceptable_statuses)
+        self.handle_request('DELETE', path, headers, acceptable_statuses)
 
     def get_container_metadata(
             self, account, container, metadata_prefix='',
@@ -655,14 +666,8 @@ class InternalClient(object):
         """
 
         path = self.make_path(account, container, obj)
-        resp = self.make_request('DELETE', path, (headers or {}),
-                                 acceptable_statuses)
-
-        # Drain the response body to prevent unexpected disconnect
-        # in proxy-server
-        with closing_if_possible(resp.app_iter):
-            for iter_body in resp.app_iter:
-                pass
+        self.handle_request('DELETE', path, (headers or {}),
+                            acceptable_statuses)
 
     def get_object_metadata(
             self, account, container, obj, metadata_prefix='',
@@ -815,8 +820,8 @@ class InternalClient(object):
         if 'Content-Length' not in headers:
             headers['Transfer-Encoding'] = 'chunked'
         path = self.make_path(account, container, obj)
-        self.make_request('PUT', path, headers, acceptable_statuses, fobj,
-                          params=params)
+        self.handle_request('PUT', path, headers, acceptable_statuses, fobj,
+                            params=params)
 
 
 def get_auth(url, user, key, auth_version='1.0', **kwargs):
