@@ -17,13 +17,16 @@ import base64
 import json
 
 from swift import gettext_ as _
+from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.http import is_success
 from swift.common.middleware.crypto.crypto_utils import CryptoWSGIContext, \
     load_crypto_meta, extract_crypto_meta, Crypto
 from swift.common.exceptions import EncryptionException, UnknownSecretIdError
 from swift.common.request_helpers import get_object_transient_sysmeta, \
-    get_sys_meta_prefix, get_user_meta_prefix
-from swift.common.swob import Request, HTTPException, HTTPInternalServerError
+    get_sys_meta_prefix, get_user_meta_prefix, \
+    get_container_update_override_key
+from swift.common.swob import Request, HTTPException, \
+    HTTPInternalServerError, wsgi_to_bytes, bytes_to_wsgi
 from swift.common.utils import get_logger, config_true_value, \
     parse_content_range, closing_if_possible, parse_content_type, \
     FileLikeIter, multipart_byteranges_to_document_iters
@@ -83,13 +86,13 @@ class BaseDecrypterContext(CryptoWSGIContext):
             body='Error decrypting %s' % self.server_type,
             content_type='text/plain')
 
-    def decrypt_value_with_meta(self, value, key, required=False):
+    def decrypt_value_with_meta(self, value, key, required, decoder):
         """
         Base64-decode and decrypt a value if crypto meta can be extracted from
         the value itself, otherwise return the value unmodified.
 
         A value should either be a string that does not contain the ';'
-        character or should be of the form:
+        character or should be of the form::
 
             <base64-encoded ciphertext>;swift_meta=<crypto meta>
 
@@ -98,6 +101,7 @@ class BaseDecrypterContext(CryptoWSGIContext):
         :param required: if True then the value is required to be decrypted
                          and an EncryptionException will be raised if the
                          header cannot be decrypted due to missing crypto meta.
+        :param decoder: function to turn the decrypted bytes into useful data
         :returns: decrypted value if crypto meta is found, otherwise the
                   unmodified value
         :raises EncryptionException: if an error occurs while parsing crypto
@@ -108,14 +112,15 @@ class BaseDecrypterContext(CryptoWSGIContext):
         extracted_value, crypto_meta = extract_crypto_meta(value)
         if crypto_meta:
             self.crypto.check_crypto_meta(crypto_meta)
-            value = self.decrypt_value(extracted_value, key, crypto_meta)
+            value = self.decrypt_value(
+                extracted_value, key, crypto_meta, decoder)
         elif required:
             raise EncryptionException(
                 "Missing crypto meta in value %s" % value)
 
         return value
 
-    def decrypt_value(self, value, key, crypto_meta):
+    def decrypt_value(self, value, key, crypto_meta, decoder):
         """
         Base64-decode and decrypt a value using the crypto_meta provided.
 
@@ -123,13 +128,14 @@ class BaseDecrypterContext(CryptoWSGIContext):
         :param key: crypto key to use
         :param crypto_meta: a crypto-meta dict of form returned by
             :py:func:`~swift.common.middleware.crypto.Crypto.get_crypto_meta`
+        :param decoder: function to turn the decrypted bytes into useful data
         :returns: decrypted value
         """
         if not value:
-            return ''
+            return decoder(b'')
         crypto_ctxt = self.crypto.create_decryption_ctxt(
             key, crypto_meta['iv'], 0)
-        return crypto_ctxt.update(base64.b64decode(value))
+        return decoder(crypto_ctxt.update(base64.b64decode(value)))
 
     def get_decryption_keys(self, req, crypto_meta=None):
         """
@@ -168,7 +174,8 @@ class DecrypterObjContext(BaseDecrypterContext):
                                          found.
         """
         try:
-            return self.decrypt_value_with_meta(value, key, required)
+            return self.decrypt_value_with_meta(
+                value, key, required, bytes_to_wsgi)
         except EncryptionException as err:
             self.logger.error(
                 _("Error decrypting header %(header)s: %(error)s"),
@@ -214,7 +221,7 @@ class DecrypterObjContext(BaseDecrypterContext):
                     required=True)
                 mod_hdr_pairs.append(('Etag', decrypted_etag))
 
-            etag_header = 'X-Object-Sysmeta-Container-Update-Override-Etag'
+            etag_header = get_container_update_override_key('etag')
             encrypted_etag = self._response_header_value(etag_header)
             if encrypted_etag:
                 decrypted_etag = self._decrypt_header(
@@ -250,21 +257,22 @@ class DecrypterObjContext(BaseDecrypterContext):
             parts_iter = multipart_byteranges_to_document_iters(
                 FileLikeIter(resp), boundary)
             for first_byte, last_byte, length, headers, body in parts_iter:
-                yield "--" + boundary + "\r\n"
+                yield b"--" + boundary + b"\r\n"
 
-                for header_pair in headers:
-                    yield "%s: %s\r\n" % header_pair
+                for header, value in headers:
+                    yield b"%s: %s\r\n" % (wsgi_to_bytes(header),
+                                           wsgi_to_bytes(value))
 
-                yield "\r\n"
+                yield b"\r\n"
 
                 decrypt_ctxt = self.crypto.create_decryption_ctxt(
                     body_key, crypto_meta['iv'], first_byte)
-                for chunk in iter(lambda: body.read(DECRYPT_CHUNK_SIZE), ''):
+                for chunk in iter(lambda: body.read(DECRYPT_CHUNK_SIZE), b''):
                     yield decrypt_ctxt.update(chunk)
 
-                yield "\r\n"
+                yield b"\r\n"
 
-            yield "--" + boundary + "--"
+            yield b"--" + boundary + b"--"
 
     def response_iter(self, resp, body_key, crypto_meta, offset):
         """
@@ -294,15 +302,24 @@ class DecrypterObjContext(BaseDecrypterContext):
                     body='Error decrypting object', content_type='text/plain')
         return crypto_meta
 
-    def handle_get(self, req, start_response):
+    def handle(self, req, start_response):
         app_resp = self._app_call(req.environ)
 
-        put_crypto_meta = self._read_crypto_meta(
-            'X-Object-Sysmeta-Crypto-Body-Meta', True)
-        put_keys = self.get_decryption_keys(req, put_crypto_meta)
-        post_crypto_meta = self._read_crypto_meta(
-            'X-Object-Transient-Sysmeta-Crypto-Meta', False)
-        post_keys = self.get_decryption_keys(req, post_crypto_meta)
+        try:
+            put_crypto_meta = self._read_crypto_meta(
+                'X-Object-Sysmeta-Crypto-Body-Meta', True)
+            put_keys = self.get_decryption_keys(req, put_crypto_meta)
+            post_crypto_meta = self._read_crypto_meta(
+                'X-Object-Transient-Sysmeta-Crypto-Meta', False)
+            post_keys = self.get_decryption_keys(req, post_crypto_meta)
+        except EncryptionException as err:
+            self.logger.error(
+                "Error decrypting object: %s",
+                err)
+            raise HTTPInternalServerError(
+                body='Error decrypting object',
+                content_type='text/plain')
+
         if put_keys is None and post_keys is None:
             # skip decryption
             start_response(self._response_status, self._response_headers,
@@ -311,7 +328,8 @@ class DecrypterObjContext(BaseDecrypterContext):
 
         mod_resp_headers = self.decrypt_resp_headers(put_keys, post_keys)
 
-        if put_crypto_meta and is_success(self._get_status_int()):
+        if put_crypto_meta and req.method == 'GET' and \
+                is_success(self._get_status_int()):
             # 2xx response and encrypted body
             body_key = self.get_unwrapped_key(
                 put_crypto_meta, put_keys['object'])
@@ -320,7 +338,7 @@ class DecrypterObjContext(BaseDecrypterContext):
 
             if (self._get_status_int() == 206 and
                     content_type == 'multipart/byteranges'):
-                boundary = dict(content_type_attrs)["boundary"]
+                boundary = wsgi_to_bytes(dict(content_type_attrs)["boundary"])
                 resp_iter = self.multipart_response_iter(
                     app_resp, boundary, body_key, put_crypto_meta)
             else:
@@ -341,54 +359,21 @@ class DecrypterObjContext(BaseDecrypterContext):
 
         return resp_iter
 
-    def handle_head(self, req, start_response):
-        app_resp = self._app_call(req.environ)
-        put_crypto_meta = self._read_crypto_meta(
-            'X-Object-Sysmeta-Crypto-Body-Meta', True)
-        put_keys = self.get_decryption_keys(req, put_crypto_meta)
-        post_crypto_meta = self._read_crypto_meta(
-            'X-Object-Transient-Sysmeta-Crypto-Meta', False)
-        post_keys = self.get_decryption_keys(req, post_crypto_meta)
-
-        if put_keys is None and post_keys is None:
-            # skip decryption
-            start_response(self._response_status, self._response_headers,
-                           self._response_exc_info)
-        else:
-            mod_resp_headers = self.decrypt_resp_headers(put_keys, post_keys)
-            mod_resp_headers = purge_crypto_sysmeta_headers(mod_resp_headers)
-            start_response(self._response_status, mod_resp_headers,
-                           self._response_exc_info)
-
-        return app_resp
-
 
 class DecrypterContContext(BaseDecrypterContext):
     def __init__(self, decrypter, logger):
         super(DecrypterContContext, self).__init__(
             decrypter, 'container', logger)
 
-    def handle_get(self, req, start_response):
+    def handle(self, req, start_response):
         app_resp = self._app_call(req.environ)
 
         if is_success(self._get_status_int()):
             # only decrypt body of 2xx responses
-            handler = None
-            for header, value in self._response_headers:
-                if header.lower() == 'content-type' and \
-                        value.split(';', 1)[0] == 'application/json':
-                    handler = self.process_json_resp
-
-            if handler:
-                try:
-                    app_resp = handler(req, app_resp)
-                except EncryptionException as err:
-                    self.logger.error(
-                        "Error decrypting container listing: %s",
-                        err)
-                    raise HTTPInternalServerError(
-                        body='Error decrypting container listing',
-                        content_type='text/plain')
+            headers = HeaderKeyDict(self._response_headers)
+            content_type = headers.get('content-type', '').split(';', 1)[0]
+            if content_type == 'application/json':
+                app_resp = self.process_json_resp(req, app_resp)
 
         start_response(self._response_status,
                        self._response_headers,
@@ -402,10 +387,10 @@ class DecrypterContContext(BaseDecrypterContext):
         Content-Length header with new body length and return a body iter.
         """
         with closing_if_possible(resp_iter):
-            resp_body = ''.join(resp_iter)
+            resp_body = b''.join(resp_iter)
         body_json = json.loads(resp_body)
         new_body = json.dumps([self.decrypt_obj_dict(req, obj_dict)
-                               for obj_dict in body_json])
+                               for obj_dict in body_json]).encode('ascii')
         self.update_content_length(len(new_body))
         return [new_body]
 
@@ -419,8 +404,11 @@ class DecrypterContContext(BaseDecrypterContext):
                 try:
                     self.crypto.check_crypto_meta(crypto_meta)
                     keys = self.get_decryption_keys(req, crypto_meta)
+                    # Note that symlinks (for example) may put swift paths in
+                    # the listing ETag, so we can't just use ASCII.
                     obj_dict['hash'] = self.decrypt_value(
-                        ciphertext, keys['container'], crypto_meta)
+                        ciphertext, keys['container'], crypto_meta,
+                        decoder=lambda x: x.decode('utf-8'))
                 except EncryptionException as err:
                     if not isinstance(err, UnknownSecretIdError) or \
                             err.args[0] not in bad_keys:
@@ -446,15 +434,16 @@ class Decrypter(object):
         req = Request(env)
         try:
             parts = req.split_path(3, 4, True)
+            is_cont_or_obj_req = True
         except ValueError:
+            is_cont_or_obj_req = False
+        if not is_cont_or_obj_req:
             return self.app(env, start_response)
 
-        if parts[3] and req.method == 'GET':
-            handler = DecrypterObjContext(self, self.logger).handle_get
-        elif parts[3] and req.method == 'HEAD':
-            handler = DecrypterObjContext(self, self.logger).handle_head
+        if parts[3] and req.method in ('GET', 'HEAD'):
+            handler = DecrypterObjContext(self, self.logger).handle
         elif parts[2] and req.method == 'GET':
-            handler = DecrypterContContext(self, self.logger).handle_get
+            handler = DecrypterContContext(self, self.logger).handle
         else:
             # url and/or request verb is not handled by decrypter
             return self.app(env, start_response)
