@@ -43,6 +43,8 @@ http://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html
 """
 from functools import partial
 
+import six
+
 from swift.common.utils import json
 
 from swift.common.middleware.s3api.s3response import InvalidArgument, \
@@ -180,18 +182,19 @@ class Grantee(object):
         """
         Convert a grantee string in the HTTP header to an Grantee instance.
         """
-        type, value = grantee.split('=', 1)
+        grantee_type, value = grantee.split('=', 1)
+        grantee_type = grantee_type.lower()
         value = value.strip('"\'')
-        if type == 'id':
+        if grantee_type == 'id':
             return User(value)
-        elif type == 'emailAddress':
+        elif grantee_type == 'emailaddress':
             raise S3NotImplemented()
-        elif type == 'uri':
+        elif grantee_type == 'uri':
             # return a subclass instance of Group class
             subclass = get_group_subclass_from_uri(value)
             return subclass()
         else:
-            raise InvalidArgument(type, value,
+            raise InvalidArgument(grantee_type, value,
                                   'Argument format not recognized')
 
 
@@ -218,6 +221,11 @@ class User(Grantee):
     def __str__(self):
         return self.display_name
 
+    def __lt__(self, other):
+        if not isinstance(other, User):
+            return NotImplemented
+        return self.id < other.id
+
 
 class Owner(object):
     """
@@ -225,6 +233,8 @@ class Owner(object):
     """
     def __init__(self, id, name):
         self.id = id
+        if not (name is None or isinstance(name, six.string_types)):
+            raise TypeError('name must be a string or None')
         self.name = name
 
 
@@ -415,9 +425,14 @@ class ACL(object):
         self.s3_acl = s3_acl
         self.allow_no_owner = allow_no_owner
 
-    def __repr__(self):
+    def __bytes__(self):
         return tostring(self.elem())
 
+    def __repr__(self):
+        if six.PY2:
+            return self.__bytes__()
+        return self.__bytes__().decode('utf8')
+
     @classmethod
     def from_elem(cls, elem, s3_acl=False, allow_no_owner=False):
         """
