@@ -19,7 +19,6 @@ import socket
 
 from collections import defaultdict
 
-from swift import gettext_ as _
 from random import shuffle
 from time import time
 import functools
@@ -35,13 +34,15 @@ from swift.common.ring import Ring
 from swift.common.utils import Watchdog, get_logger, \
     get_remote_client, split_path, config_true_value, generate_trans_id, \
     affinity_key_function, affinity_locality_predicate, list_from_csv, \
-    register_swift_info, readconf, config_auto_int_value
+    parse_prefixed_conf, config_auto_int_value, \
+    config_request_node_count_value, config_percent_value
+from swift.common.registry import register_swift_info
 from swift.common.constraints import check_utf8, valid_api_version
 from swift.proxy.controllers import AccountController, ContainerController, \
     ObjectControllerRouter, InfoController
 from swift.proxy.controllers.base import get_container_info, NodeIter, \
     DEFAULT_RECHECK_CONTAINER_EXISTENCE, DEFAULT_RECHECK_ACCOUNT_EXISTENCE, \
-    DEFAULT_RECHECK_UPDATING_SHARD_RANGES
+    DEFAULT_RECHECK_UPDATING_SHARD_RANGES, DEFAULT_RECHECK_LISTING_SHARD_RANGES
 from swift.common.swob import HTTPBadRequest, HTTPForbidden, \
     HTTPMethodNotAllowed, HTTPNotFound, HTTPPreconditionFailed, \
     HTTPServerError, HTTPException, Request, HTTPServiceUnavailable, \
@@ -195,10 +196,12 @@ class Application(object):
         if conf is None:
             conf = {}
         if logger is None:
-            self.logger = get_logger(conf, log_route='proxy-server')
+            self.logger = get_logger(conf, log_route='proxy-server',
+                                     statsd_tail_prefix='proxy-server')
         else:
             self.logger = logger
         self._error_limiting = {}
+        self.backend_user_agent = 'proxy-server %s' % os.getpid()
 
         swift_dir = conf.get('swift_dir', '/etc/swift')
         self.swift_dir = swift_dir
@@ -212,7 +215,7 @@ class Application(object):
         self.trans_id_suffix = conf.get('trans_id_suffix', '')
         self.post_quorum_timeout = float(conf.get('post_quorum_timeout', 0.5))
         self.error_suppression_interval = \
-            int(conf.get('error_suppression_interval', 60))
+            float(conf.get('error_suppression_interval', 60))
         self.error_suppression_limit = \
             int(conf.get('error_suppression_limit', 10))
         self.recheck_container_existence = \
@@ -221,9 +224,18 @@ class Application(object):
         self.recheck_updating_shard_ranges = \
             int(conf.get('recheck_updating_shard_ranges',
                          DEFAULT_RECHECK_UPDATING_SHARD_RANGES))
+        self.recheck_listing_shard_ranges = \
+            int(conf.get('recheck_listing_shard_ranges',
+                         DEFAULT_RECHECK_LISTING_SHARD_RANGES))
         self.recheck_account_existence = \
             int(conf.get('recheck_account_existence',
                          DEFAULT_RECHECK_ACCOUNT_EXISTENCE))
+        self.container_updating_shard_ranges_skip_cache = \
+            config_percent_value(conf.get(
+                'container_updating_shard_ranges_skip_cache_pct', 0))
+        self.container_listing_shard_ranges_skip_cache = \
+            config_percent_value(conf.get(
+                'container_listing_shard_ranges_skip_cache_pct', 0))
         self.allow_account_management = \
             config_true_value(conf.get('allow_account_management', 'no'))
         self.container_ring = container_ring or Ring(swift_dir,
@@ -276,16 +288,8 @@ class Application(object):
             conf.get('strict_cors_mode', 't'))
         self.node_timings = {}
         self.timing_expiry = int(conf.get('timing_expiry', 300))
-        value = conf.get('request_node_count', '2 * replicas').lower().split()
-        if len(value) == 1:
-            rnc_value = int(value[0])
-            self.request_node_count = lambda replicas: rnc_value
-        elif len(value) == 3 and value[1] == '*' and value[2] == 'replicas':
-            rnc_value = int(value[0])
-            self.request_node_count = lambda replicas: rnc_value * replicas
-        else:
-            raise ValueError(
-                'Invalid request_node_count value: %r' % ''.join(value))
+        value = conf.get('request_node_count', '2 * replicas')
+        self.request_node_count = config_request_node_count_value(value)
         # swift_owner_headers are stripped by the account and container
         # controllers; we should extend header stripping to object controller
         # when a privileged object header is implemented.
@@ -299,6 +303,17 @@ class Application(object):
         self.swift_owner_headers = [
             name.strip().title()
             for name in swift_owner_headers.split(',') if name.strip()]
+
+        # When upgrading from liberasurecode<=1.5.0, you may want to continue
+        # writing legacy CRCs until all nodes are upgraded and capabale of
+        # reading fragments with zlib CRCs.
+        # See https://bugs.launchpad.net/liberasurecode/+bug/1886088 for more
+        # information.
+        if 'write_legacy_ec_crc' in conf:
+            os.environ['LIBERASURECODE_WRITE_LEGACY_CRC'] = \
+                '1' if config_true_value(conf['write_legacy_ec_crc']) else '0'
+        # else, assume operators know what they're doing and leave env alone
+
         # Initialization was successful, so now apply the client chunk size
         # parameter as the default read / write buffer size for the network
         # sockets.
@@ -397,8 +412,8 @@ class Application(object):
                       else POLICIES.get_by_index(policy_idx))
             if options.read_affinity and options.sorting_method != 'affinity':
                 self.logger.warning(
-                    _("sorting_method is set to '%(method)s', not 'affinity'; "
-                      "%(label)s read_affinity setting will have no effect."),
+                    "sorting_method is set to '%(method)s', not 'affinity'; "
+                    "%(label)s read_affinity setting will have no effect.",
                     {'label': _label_for_policy(policy),
                      'method': options.sorting_method})
 
@@ -502,7 +517,6 @@ class Application(object):
         :param req: swob.Request object
         """
         try:
-            self.logger.set_statsd_prefix('proxy-server')
             if req.content_length and req.content_length < 0:
                 self.logger.increment('errors')
                 return HTTPBadRequest(request=req,
@@ -534,8 +548,6 @@ class Application(object):
                     req.host.split(':')[0] in self.deny_host_headers:
                 return HTTPForbidden(request=req, body='Invalid host header')
 
-            self.logger.set_statsd_prefix('proxy-server.' +
-                                          controller.server_type.lower())
             controller = controller(self, **path_parts)
             if 'swift.trans_id' not in req.environ:
                 # if this wasn't set by an earlier middleware, set it now
@@ -590,7 +602,7 @@ class Application(object):
         except HTTPException as error_response:
             return error_response
         except (Exception, Timeout):
-            self.logger.exception(_('ERROR Unhandled exception in request'))
+            self.logger.exception('ERROR Unhandled exception in request')
             return HTTPServerError(request=req)
 
     def sort_nodes(self, nodes, policy=None):
@@ -649,7 +661,7 @@ class Application(object):
         limited = error_stats['errors'] > self.error_suppression_limit
         if limited:
             self.logger.debug(
-                _('Node error limited %(ip)s:%(port)s (%(device)s)'), node)
+                'Node error limited %(ip)s:%(port)s (%(device)s)', node)
         return limited
 
     def error_limit(self, node, msg):
@@ -666,9 +678,9 @@ class Application(object):
         error_stats = self._error_limiting.setdefault(node_key, {})
         error_stats['errors'] = self.error_suppression_limit + 1
         error_stats['last_error'] = time()
-        self.logger.error(_('%(msg)s %(ip)s:%(port)s/%(device)s'),
-                          {'msg': msg, 'ip': node['ip'],
-                          'port': node['port'], 'device': node['device']})
+        self.logger.error('%(msg)s %(ip)s:%(port)s/%(device)s', {
+            'msg': msg, 'ip': node['ip'],
+            'port': node['port'], 'device': node['device']})
 
     def _incr_node_errors(self, node):
         node_key = self._error_limit_node_key(node)
@@ -686,12 +698,12 @@ class Application(object):
         self._incr_node_errors(node)
         if isinstance(msg, bytes):
             msg = msg.decode('utf-8')
-        self.logger.error(_('%(msg)s %(ip)s:%(port)s/%(device)s'),
-                          {'msg': msg, 'ip': node['ip'],
-                          'port': node['port'], 'device': node['device']})
+        self.logger.error('%(msg)s %(ip)s:%(port)s/%(device)s', {
+            'msg': msg, 'ip': node['ip'],
+            'port': node['port'], 'device': node['device']})
 
-    def iter_nodes(self, ring, partition, node_iter=None, policy=None):
-        return NodeIter(self, ring, partition, node_iter=node_iter,
+    def iter_nodes(self, ring, partition, logger, node_iter=None, policy=None):
+        return NodeIter(self, ring, partition, logger, node_iter=node_iter,
                         policy=policy)
 
     def exception_occurred(self, node, typ, additional_info,
@@ -712,8 +724,8 @@ class Application(object):
             log = self.logger.exception
         if isinstance(additional_info, bytes):
             additional_info = additional_info.decode('utf-8')
-        log(_('ERROR with %(type)s server %(ip)s:%(port)s/%(device)s'
-              ' re: %(info)s'),
+        log('ERROR with %(type)s server %(ip)s:%(port)s/%(device)s'
+            ' re: %(info)s',
             {'type': typ, 'ip': node['ip'],
              'port': node['port'], 'device': node['device'],
              'info': additional_info},
@@ -738,18 +750,18 @@ class Application(object):
                     except ValueError:  # not in pipeline; ignore it
                         pass
                 self.logger.info(
-                    _('Adding required filter %(filter_name)s to pipeline at '
-                      'position %(insert_at)d'),
+                    'Adding required filter %(filter_name)s to pipeline at '
+                    'position %(insert_at)d',
                     {'filter_name': filter_name, 'insert_at': insert_at})
                 ctx = pipe.create_filter(filter_name)
                 pipe.insert_filter(ctx, index=insert_at)
                 pipeline_was_modified = True
 
         if pipeline_was_modified:
-            self.logger.info(_("Pipeline was modified. "
-                               "New pipeline is \"%s\"."), pipe)
+            self.logger.info("Pipeline was modified. "
+                             "New pipeline is \"%s\".", pipe)
         else:
-            self.logger.debug(_("Pipeline is \"%s\""), pipe)
+            self.logger.debug("Pipeline is \"%s\"", pipe)
 
 
 def parse_per_policy_config(conf):
@@ -762,15 +774,8 @@ def parse_per_policy_config(conf):
     :return: a dict mapping policy reference -> dict of policy options
     :raises ValueError: if a policy config section has an invalid name
     """
-    policy_config = {}
-    all_conf = readconf(conf['__file__'])
     policy_section_prefix = conf['__name__'] + ':policy:'
-    for section, options in all_conf.items():
-        if not section.startswith(policy_section_prefix):
-            continue
-        policy_ref = section[len(policy_section_prefix):]
-        policy_config[policy_ref] = options
-    return policy_config
+    return parse_prefixed_conf(conf['__file__'], policy_section_prefix)
 
 
 def app_factory(global_conf, **local_conf):
