@@ -13,25 +13,21 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-import errno
-import json
+import itertools
 from contextlib import contextmanager
-from hashlib import md5
 import unittest
 import uuid
-import shutil
 import random
-import os
 import time
 import six
 
 from swift.common.direct_client import DirectClientException
+from swift.common.manager import Manager
+from swift.common.utils import md5
+from swift.obj.reconstructor import ObjectReconstructor
 from test.probe.common import ECProbeTest
 
 from swift.common import direct_client
-from swift.common.storage_policy import EC_POLICY
-from swift.common.manager import Manager
 
 from swiftclient import client, ClientException
 
@@ -40,7 +36,7 @@ class Body(object):
 
     def __init__(self, total=3.5 * 2 ** 20):
         self.total = int(total)
-        self.hasher = md5()
+        self.hasher = md5(usedforsecurity=False)
         self.size = 0
         self.chunk = b'test' * 16 * 2 ** 10
 
@@ -64,17 +60,8 @@ class Body(object):
 
 class TestReconstructorRebuild(ECProbeTest):
 
-    def _make_name(self, prefix):
-        return ('%s%s' % (prefix, uuid.uuid4())).encode()
-
     def setUp(self):
         super(TestReconstructorRebuild, self).setUp()
-        self.container_name = self._make_name('container-')
-        self.object_name = self._make_name('object-')
-        # sanity
-        self.assertEqual(self.policy.policy_type, EC_POLICY)
-        self.reconstructor = Manager(["object-reconstructor"])
-
         # create EC container
         headers = {'X-Storage-Policy': self.policy.name}
         client.put_container(self.url, self.token, self.container_name,
@@ -99,80 +86,6 @@ class TestReconstructorRebuild(ECProbeTest):
                 'X-Backend-Durable-Timestamp', hdrs,
                 'Missing durable timestamp in %r' % self.frag_headers)
 
-    def proxy_put(self, extra_headers=None):
-        contents = Body()
-        headers = {
-            self._make_name('x-object-meta-').decode('utf8'):
-                self._make_name('meta-foo-').decode('utf8'),
-        }
-        if extra_headers:
-            headers.update(extra_headers)
-        self.etag = client.put_object(self.url, self.token,
-                                      self.container_name,
-                                      self.object_name,
-                                      contents=contents, headers=headers)
-
-    def proxy_get(self):
-        # GET object
-        headers, body = client.get_object(self.url, self.token,
-                                          self.container_name,
-                                          self.object_name,
-                                          resp_chunk_size=64 * 2 ** 10)
-        resp_checksum = md5()
-        for chunk in body:
-            resp_checksum.update(chunk)
-        return headers, resp_checksum.hexdigest()
-
-    def direct_get(self, node, part, require_durable=True, extra_headers=None):
-        req_headers = {'X-Backend-Storage-Policy-Index': int(self.policy)}
-        if extra_headers:
-            req_headers.update(extra_headers)
-        if not require_durable:
-            req_headers.update(
-                {'X-Backend-Fragment-Preferences': json.dumps([])})
-        # node dict has unicode values so utf8 decode our path parts too in
-        # case they have non-ascii characters
-        if six.PY2:
-            acc, con, obj = (s.decode('utf8') for s in (
-                self.account, self.container_name, self.object_name))
-        else:
-            acc, con, obj = self.account, self.container_name, self.object_name
-        headers, data = direct_client.direct_get_object(
-            node, part, acc, con, obj, headers=req_headers,
-            resp_chunk_size=64 * 2 ** 20)
-        hasher = md5()
-        for chunk in data:
-            hasher.update(chunk)
-        return headers, hasher.hexdigest()
-
-    def _break_nodes(self, failed, non_durable):
-        # delete partitions on the failed nodes and remove durable marker from
-        # non-durable nodes
-        for i, node in enumerate(self.onodes):
-            part_dir = self.storage_dir(node, part=self.opart)
-            if i in failed:
-                shutil.rmtree(part_dir, True)
-                try:
-                    self.direct_get(node, self.opart)
-                except direct_client.DirectClientException as err:
-                    self.assertEqual(err.http_status, 404)
-            elif i in non_durable:
-                for dirs, subdirs, files in os.walk(part_dir):
-                    for fname in files:
-                        if fname.endswith('.data'):
-                            non_durable_fname = fname.replace('#d', '')
-                            os.rename(os.path.join(dirs, fname),
-                                      os.path.join(dirs, non_durable_fname))
-                            break
-                headers, etag = self.direct_get(node, self.opart,
-                                                require_durable=False)
-                self.assertNotIn('X-Backend-Durable-Timestamp', headers)
-            try:
-                os.remove(os.path.join(part_dir, 'hashes.pkl'))
-            except OSError as e:
-                if e.errno != errno.ENOENT:
-                    raise
-
     def _format_node(self, node):
         return '%s#%s' % (node['device'], node['index'])
 
@@ -213,7 +126,7 @@ class TestReconstructorRebuild(ECProbeTest):
         # helper method to test a scenario with some nodes missing their
         # fragment and some nodes having non-durable fragments
         with self._annotate_failure_with_scenario(failed, non_durable):
-            self._break_nodes(failed, non_durable)
+            self.break_nodes(self.onodes, self.opart, failed, non_durable)
 
         # make sure we can still GET the object and it is correct; the
         # proxy is doing decode on remaining fragments to get the obj
@@ -370,10 +283,12 @@ class TestReconstructorRebuild(ECProbeTest):
 
         # sanity check - X-Backend-Replication let's us get expired frag...
         fail_node = random.choice(self.onodes)
-        self.direct_get(fail_node, self.opart,
-                        extra_headers={'X-Backend-Replication': 'True'})
+        self.assert_direct_get_succeeds(
+            fail_node, self.opart,
+            extra_headers={'X-Backend-Replication': 'True'})
         # ...until we remove the frag from fail_node
-        self._break_nodes([self.onodes.index(fail_node)], [])
+        self.break_nodes(
+            self.onodes, self.opart, [self.onodes.index(fail_node)], [])
         # ...now it's really gone
         with self.assertRaises(DirectClientException) as cm:
             self.direct_get(fail_node, self.opart,
@@ -456,8 +371,165 @@ class TestReconstructorRebuild(ECProbeTest):
                     'X-Backend-Storage-Policy-Index': int(self.policy)})
             self.assertNotIn('X-Delete-At', headers)
 
+    def test_rebuild_quarantines_lonely_frag(self):
+        # fail one device while the object is deleted so we are left with one
+        # fragment and some tombstones
+        failed_node = self.onodes[0]
+        device_path = self.device_dir(failed_node)
+        self.kill_drive(device_path)
+        self.assert_direct_get_fails(failed_node, self.opart, 507)  # sanity
+
+        # delete object
+        client.delete_object(self.url, self.token, self.container_name,
+                             self.object_name)
+
+        # check we have tombstones
+        for node in self.onodes[1:]:
+            err = self.assert_direct_get_fails(node, self.opart, 404)
+            self.assertIn('X-Backend-Timestamp', err.http_headers)
+
+        # run the reconstructor with zero reclaim age to clean up tombstones
+        for conf_index in self.configs['object-reconstructor'].keys():
+            self.run_custom_daemon(
+                ObjectReconstructor, 'object-reconstructor', conf_index,
+                {'reclaim_age': '0'})
+
+        # check we no longer have tombstones
+        for node in self.onodes[1:]:
+            err = self.assert_direct_get_fails(node, self.opart, 404)
+            self.assertNotIn('X-Timestamp', err.http_headers)
+
+        # revive the failed device and check it has a fragment
+        self.revive_drive(device_path)
+        self.assert_direct_get_succeeds(failed_node, self.opart)
+
+        # restart proxy to clear error-limiting so that the revived drive
+        # participates again
+        Manager(['proxy-server']).restart()
+
+        # client GET will fail with 503 ...
+        with self.assertRaises(ClientException) as cm:
+            client.get_object(self.url, self.token, self.container_name,
+                              self.object_name)
+        self.assertEqual(503, cm.exception.http_status)
+        # ... but client GET succeeds
+        headers = client.head_object(self.url, self.token, self.container_name,
+                                     self.object_name)
+        for key in self.headers_post:
+            self.assertIn(key, headers)
+            self.assertEqual(self.headers_post[key], headers[key])
+
+        # run the reconstructor without quarantine_threshold set
+        error_lines = []
+        warning_lines = []
+        for conf_index in self.configs['object-reconstructor'].keys():
+            reconstructor = self.run_custom_daemon(
+                ObjectReconstructor, 'object-reconstructor', conf_index,
+                {'quarantine_age': '0'})
+            logger = reconstructor.logger.logger
+            error_lines.append(logger.get_lines_for_level('error'))
+            warning_lines.append(logger.get_lines_for_level('warning'))
+
+        # check logs for errors
+        found_lines = False
+        for lines in error_lines:
+            if not lines:
+                continue
+            self.assertFalse(found_lines, error_lines)
+            found_lines = True
+            for line in itertools.islice(lines, 0, 6, 2):
+                self.assertIn(
+                    'Unable to get enough responses (1/4 from 1 ok '
+                    'responses)', line, lines)
+            for line in itertools.islice(lines, 1, 7, 2):
+                self.assertIn(
+                    'Unable to get enough responses (4 x 404 error '
+                    'responses)', line, lines)
+        self.assertTrue(found_lines, 'error lines not found')
+
+        for lines in warning_lines:
+            self.assertEqual([], lines)
+
+        # check we have still have a single fragment and no tombstones
+        self.assert_direct_get_succeeds(failed_node, self.opart)
+        for node in self.onodes[1:]:
+            err = self.assert_direct_get_fails(node, self.opart, 404)
+            self.assertNotIn('X-Timestamp', err.http_headers)
+
+        # run the reconstructor to quarantine the lonely frag
+        error_lines = []
+        warning_lines = []
+        for conf_index in self.configs['object-reconstructor'].keys():
+            reconstructor = self.run_custom_daemon(
+                ObjectReconstructor, 'object-reconstructor', conf_index,
+                {'quarantine_age': '0', 'quarantine_threshold': '1'})
+            logger = reconstructor.logger.logger
+            error_lines.append(logger.get_lines_for_level('error'))
+            warning_lines.append(logger.get_lines_for_level('warning'))
+
+        # check logs for errors
+        found_lines = False
+        for index, lines in enumerate(error_lines):
+            if not lines:
+                continue
+            self.assertFalse(found_lines, error_lines)
+            found_lines = True
+            for line in itertools.islice(lines, 0, 6, 2):
+                self.assertIn(
+                    'Unable to get enough responses (1/4 from 1 ok '
+                    'responses)', line, lines)
+            for line in itertools.islice(lines, 1, 7, 2):
+                self.assertIn(
+                    'Unable to get enough responses (6 x 404 error '
+                    'responses)', line, lines)
+        self.assertTrue(found_lines, 'error lines not found')
+
+        # check logs for quarantine warning
+        found_lines = False
+        for lines in warning_lines:
+            if not lines:
+                continue
+            self.assertFalse(found_lines, warning_lines)
+            found_lines = True
+            self.assertEqual(1, len(lines), lines)
+            self.assertIn('Quarantined object', lines[0])
+        self.assertTrue(found_lines, 'warning lines not found')
+
+        # check we have nothing
+        for node in self.onodes:
+            err = self.assert_direct_get_fails(node, self.opart, 404)
+            self.assertNotIn('X-Backend-Timestamp', err.http_headers)
+        # client HEAD and GET now both 404
+        with self.assertRaises(ClientException) as cm:
+            client.get_object(self.url, self.token, self.container_name,
+                              self.object_name)
+        self.assertEqual(404, cm.exception.http_status)
+        with self.assertRaises(ClientException) as cm:
+            client.head_object(self.url, self.token, self.container_name,
+                               self.object_name)
+        self.assertEqual(404, cm.exception.http_status)
+
+        # run the reconstructor once more - should see no errors in logs!
+        error_lines = []
+        warning_lines = []
+        for conf_index in self.configs['object-reconstructor'].keys():
+            reconstructor = self.run_custom_daemon(
+                ObjectReconstructor, 'object-reconstructor', conf_index,
+                {'quarantine_age': '0', 'quarantine_threshold': '1'})
+            logger = reconstructor.logger.logger
+            error_lines.append(logger.get_lines_for_level('error'))
+            warning_lines.append(logger.get_lines_for_level('warning'))
+
+        for lines in error_lines:
+            self.assertEqual([], lines)
+        for lines in warning_lines:
+            self.assertEqual([], lines)
+
 
 if six.PY2:
+    # The non-ASCII chars in metadata cause test hangs in
+    # _assert_all_nodes_have_frag because of https://bugs.python.org/issue37093
+
     class TestReconstructorRebuildUTF8(TestReconstructorRebuild):
 
         def _make_name(self, prefix):
