@@ -22,8 +22,11 @@ import logging
 import logging.handlers
 import sys
 from contextlib import contextmanager, closing
-from collections import defaultdict, Iterable
-from hashlib import md5
+from collections import defaultdict
+try:
+    from collections.abc import Iterable
+except ImportError:
+    from collections import Iterable  # py2
 import itertools
 from numbers import Number
 from tempfile import NamedTemporaryFile
@@ -46,10 +49,12 @@ from six.moves import range
 from six.moves.http_client import HTTPException
 
 from swift.common import storage_policy, swob, utils
+from swift.common.memcached import MemcacheConnectionError
 from swift.common.storage_policy import (StoragePolicy, ECStoragePolicy,
                                          VALID_EC_TYPES)
-from swift.common.utils import Timestamp, NOTICE
+from swift.common.utils import Timestamp, md5
 from test import get_config
+from test.debug_logger import FakeLogger
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.ring import Ring, RingData, RingBuilder
 from swift.obj import server
@@ -65,7 +70,7 @@ class SkipTest(unittest.SkipTest):
     pass
 
 
-EMPTY_ETAG = md5().hexdigest()
+EMPTY_ETAG = md5(usedforsecurity=False).hexdigest()
 
 # try not to import this module from swift
 if not os.path.basename(sys.argv[0]).startswith('swift'):
@@ -178,7 +183,7 @@ class PatchPolicies(object):
 
         def unpatch_cleanup(cls_self):
             if cls_self._policies_patched:
-                self.__exit__()
+                self.__exit__(None, None, None)
                 cls_self._policies_patched = False
 
         def setUp(cls_self):
@@ -205,7 +210,7 @@ class PatchPolicies(object):
         try:
             self._setup_rings()
         except:  # noqa
-            self.__exit__()
+            self.__exit__(None, None, None)
             raise
 
     def __exit__(self, *args):
@@ -215,7 +220,8 @@ class PatchPolicies(object):
 class FakeRing(Ring):
 
     def __init__(self, replicas=3, max_more_nodes=0, part_power=0,
-                 base_port=1000, separate_replication=False):
+                 base_port=1000, separate_replication=False,
+                 next_part_power=None, reload_time=15):
         self.serialized_path = '/foo/bar/object.ring.gz'
         self._base_port = base_port
         self.max_more_nodes = max_more_nodes
@@ -224,7 +230,9 @@ class FakeRing(Ring):
         self.separate_replication = separate_replication
         # 9 total nodes (6 more past the initial 3) is the cap, no matter if
         # this is set higher, or R^2 for R replicas
+        self.reload_time = reload_time
         self.set_replicas(replicas)
+        self._next_part_power = next_part_power
         self._reload()
 
     def has_changed(self):
@@ -271,6 +279,7 @@ class FakeRing(Ring):
                 'zone': x % 3,
                 'region': x % 2,
                 'id': x,
+                'weight': 1,
             }
             self.add_node(dev)
 
@@ -393,29 +402,71 @@ class FabricatedRing(Ring):
         self._update_bookkeeping()
 
 
+def track(f):
+    def wrapper(self, *a, **kw):
+        self.calls.append(getattr(mocklib.call, f.__name__)(*a, **kw))
+        return f(self, *a, **kw)
+    return wrapper
+
+
 class FakeMemcache(object):
 
-    def __init__(self):
+    def __init__(self, error_on_set=None, error_on_get=None):
         self.store = {}
+        self.calls = []
+        self.error_on_incr = False
+        self.error_on_get = error_on_get or []
+        self.error_on_set = error_on_set or []
+        self.init_incr_return_neg = False
 
-    def get(self, key):
+    def clear_calls(self):
+        del self.calls[:]
+
+    @track
+    def get(self, key, raise_on_error=False):
+        if self.error_on_get and self.error_on_get.pop(0):
+            if raise_on_error:
+                raise MemcacheConnectionError()
         return self.store.get(key)
 
+    @property
     def keys(self):
-        return self.store.keys()
+        return self.store.keys
 
-    def set(self, key, value, time=0):
+    @track
+    def set(self, key, value, serialize=True, time=0, raise_on_error=False):
+        if self.error_on_set and self.error_on_set.pop(0):
+            if raise_on_error:
+                raise MemcacheConnectionError()
+        if serialize:
+            value = json.loads(json.dumps(value))
+        else:
+            assert isinstance(value, (str, bytes))
         self.store[key] = value
         return True
 
-    def incr(self, key, time=0):
-        self.store[key] = self.store.setdefault(key, 0) + 1
+    @track
+    def incr(self, key, delta=1, time=0):
+        if self.error_on_incr:
+            raise MemcacheConnectionError('Memcache restarting')
+        if self.init_incr_return_neg:
+            # simulate initial hit, force reset of memcache
+            self.init_incr_return_neg = False
+            return -10000000
+        self.store[key] = int(self.store.setdefault(key, 0)) + delta
+        if self.store[key] < 0:
+            self.store[key] = 0
         return self.store[key]
 
+    # tracked via incr()
+    def decr(self, key, delta=1, time=0):
+        return self.incr(key, delta=-delta, time=time)
+
     @contextmanager
     def soft_lock(self, key, timeout=0, retries=5):
         yield True
 
+    @track
     def delete(self, key):
         try:
             del self.store[key]
@@ -423,6 +474,33 @@ class FakeMemcache(object):
             pass
         return True
 
+    def delete_all(self):
+        self.store.clear()
+
+
+# This decorator only makes sense in the context of FakeMemcache;
+# may as well clean it up now
+del track
+
+
+class FakeIterable(object):
+    def __init__(self, values):
+        self.next_call_count = 0
+        self.close_call_count = 0
+        self.values = iter(values)
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        self.next_call_count += 1
+        return next(self.values)
+
+    next = __next__  # py2
+
+    def close(self):
+        self.close_call_count += 1
+
 
 def readuntil2crlfs(fd):
     rv = b''
@@ -520,197 +598,6 @@ class UnmockTimeModule(object):
 logging.time = UnmockTimeModule()
 
 
-class WARN_DEPRECATED(Exception):
-    def __init__(self, msg):
-        self.msg = msg
-        print(self.msg)
-
-
-class FakeLogger(logging.Logger, object):
-    # a thread safe fake logger
-
-    def __init__(self, *args, **kwargs):
-        self._clear()
-        self.name = 'swift.unit.fake_logger'
-        self.level = logging.NOTSET
-        if 'facility' in kwargs:
-            self.facility = kwargs['facility']
-        self.statsd_client = None
-        self.thread_locals = None
-        self.parent = None
-
-    store_in = {
-        logging.ERROR: 'error',
-        logging.WARNING: 'warning',
-        logging.INFO: 'info',
-        logging.DEBUG: 'debug',
-        logging.CRITICAL: 'critical',
-        NOTICE: 'notice',
-    }
-
-    def warn(self, *args, **kwargs):
-        raise WARN_DEPRECATED("Deprecated Method warn use warning instead")
-
-    def notice(self, msg, *args, **kwargs):
-        """
-        Convenience function for syslog priority LOG_NOTICE. The python
-        logging lvl is set to 25, just above info.  SysLogHandler is
-        monkey patched to map this log lvl to the LOG_NOTICE syslog
-        priority.
-        """
-        self.log(NOTICE, msg, *args, **kwargs)
-
-    def _log(self, level, msg, *args, **kwargs):
-        store_name = self.store_in[level]
-        cargs = [msg]
-        if any(args):
-            cargs.extend(args)
-        captured = dict(kwargs)
-        if 'exc_info' in kwargs and \
-                not isinstance(kwargs['exc_info'], tuple):
-            captured['exc_info'] = sys.exc_info()
-        self.log_dict[store_name].append((tuple(cargs), captured))
-        super(FakeLogger, self)._log(level, msg, *args, **kwargs)
-
-    def _clear(self):
-        self.log_dict = defaultdict(list)
-        self.lines_dict = {'critical': [], 'error': [], 'info': [],
-                           'warning': [], 'debug': [], 'notice': []}
-
-    clear = _clear  # this is a public interface
-
-    def get_lines_for_level(self, level):
-        if level not in self.lines_dict:
-            raise KeyError(
-                "Invalid log level '%s'; valid levels are %s" %
-                (level,
-                 ', '.join("'%s'" % lvl for lvl in sorted(self.lines_dict))))
-        return self.lines_dict[level]
-
-    def all_log_lines(self):
-        return dict((level, msgs) for level, msgs in self.lines_dict.items()
-                    if len(msgs) > 0)
-
-    def _store_in(store_name):
-        def stub_fn(self, *args, **kwargs):
-            self.log_dict[store_name].append((args, kwargs))
-        return stub_fn
-
-    # mock out the StatsD logging methods:
-    update_stats = _store_in('update_stats')
-    increment = _store_in('increment')
-    decrement = _store_in('decrement')
-    timing = _store_in('timing')
-    timing_since = _store_in('timing_since')
-    transfer_rate = _store_in('transfer_rate')
-    set_statsd_prefix = _store_in('set_statsd_prefix')
-
-    def get_increments(self):
-        return [call[0][0] for call in self.log_dict['increment']]
-
-    def get_increment_counts(self):
-        counts = {}
-        for metric in self.get_increments():
-            if metric not in counts:
-                counts[metric] = 0
-            counts[metric] += 1
-        return counts
-
-    def setFormatter(self, obj):
-        self.formatter = obj
-
-    def close(self):
-        self._clear()
-
-    def set_name(self, name):
-        # don't touch _handlers
-        self._name = name
-
-    def acquire(self):
-        pass
-
-    def release(self):
-        pass
-
-    def createLock(self):
-        pass
-
-    def emit(self, record):
-        pass
-
-    def _handle(self, record):
-        try:
-            line = record.getMessage()
-        except TypeError:
-            print('WARNING: unable to format log message %r %% %r' % (
-                record.msg, record.args))
-            raise
-        self.lines_dict[record.levelname.lower()].append(line)
-
-    def handle(self, record):
-        self._handle(record)
-
-    def flush(self):
-        pass
-
-    def handleError(self, record):
-        pass
-
-    def isEnabledFor(self, level):
-        return True
-
-
-class DebugSwiftLogFormatter(utils.SwiftLogFormatter):
-
-    def format(self, record):
-        msg = super(DebugSwiftLogFormatter, self).format(record)
-        return msg.replace('#012', '\n')
-
-
-class DebugLogger(FakeLogger):
-    """A simple stdout logging version of FakeLogger"""
-
-    def __init__(self, *args, **kwargs):
-        FakeLogger.__init__(self, *args, **kwargs)
-        self.formatter = DebugSwiftLogFormatter(
-            "%(server)s %(levelname)s: %(message)s")
-        self.records = defaultdict(list)
-
-    def handle(self, record):
-        self._handle(record)
-        formatted = self.formatter.format(record)
-        print(formatted)
-        self.records[record.levelname].append(formatted)
-
-
-class DebugLogAdapter(utils.LogAdapter):
-
-    def _send_to_logger(name):
-        def stub_fn(self, *args, **kwargs):
-            return getattr(self.logger, name)(*args, **kwargs)
-        return stub_fn
-
-    # delegate to FakeLogger's mocks
-    update_stats = _send_to_logger('update_stats')
-    increment = _send_to_logger('increment')
-    decrement = _send_to_logger('decrement')
-    timing = _send_to_logger('timing')
-    timing_since = _send_to_logger('timing_since')
-    transfer_rate = _send_to_logger('transfer_rate')
-    set_statsd_prefix = _send_to_logger('set_statsd_prefix')
-
-    def __getattribute__(self, name):
-        try:
-            return object.__getattribute__(self, name)
-        except AttributeError:
-            return getattr(self.__dict__['logger'], name)
-
-
-def debug_logger(name='test'):
-    """get a named adapted debug logger"""
-    return DebugLogAdapter(DebugLogger(), name)
-
-
 original_syslog_handler = logging.handlers.SysLogHandler
 
 
@@ -838,6 +725,11 @@ class FakeStatus(object):
             self.expect_sleep_list.append(None)
         self.response_sleep = response_sleep
 
+    def __repr__(self):
+        return '%s(%s, expect_status=%r, response_sleep=%s)' % (
+            self.__class__.__name__, self.status,
+            self.expect_status, self.response_sleep)
+
     def get_response_status(self):
         if self.response_sleep is not None:
             eventlet.sleep(self.response_sleep)
@@ -965,7 +857,8 @@ def fake_http_connect(*code_iter, **kwargs):
             etag = self.etag
             if not etag:
                 if isinstance(self.body, bytes):
-                    etag = '"' + md5(self.body).hexdigest() + '"'
+                    etag = ('"' + md5(
+                        self.body, usedforsecurity=False).hexdigest() + '"')
                 else:
                     etag = '"68b329da9893e34099c7d8ad5cb9c940"'
 
@@ -1078,14 +971,20 @@ def fake_http_connect(*code_iter, **kwargs):
             # the code under test may swallow the StopIteration, so by logging
             # unexpected requests here we allow the test framework to check for
             # them after the connect function has been used.
-            unexpected_requests.append((args, kwargs))
+            unexpected_requests.append((args, ckwargs))
             raise
 
         if 'give_connect' in kwargs:
             give_conn_fn = kwargs['give_connect']
-            argspec = inspect.getargspec(give_conn_fn)
-            if argspec.keywords or 'connection_id' in argspec.args:
-                ckwargs['connection_id'] = i
+
+            if six.PY2:
+                argspec = inspect.getargspec(give_conn_fn)
+                if argspec.keywords or 'connection_id' in argspec.args:
+                    ckwargs['connection_id'] = i
+            else:
+                argspec = inspect.getfullargspec(give_conn_fn)
+                if argspec.varkw or 'connection_id' in argspec.args:
+                    ckwargs['connection_id'] = i
             give_conn_fn(*args, **ckwargs)
         etag = next(etag_iter)
         headers = next(headers_iter)
@@ -1142,8 +1041,8 @@ def mocked_http_conn(*args, **kwargs):
         if left_over_status:
             raise AssertionError('left over status %r' % left_over_status)
         if fake_conn.unexpected_requests:
-            raise AssertionError('unexpected requests %r' %
-                                 fake_conn.unexpected_requests)
+            raise AssertionError('unexpected requests:\n%s' % '\n  '.join(
+                '%r' % (req,) for req in fake_conn.unexpected_requests))
 
 
 def make_timestamp_iter(offset=0):
@@ -1152,14 +1051,21 @@ def make_timestamp_iter(offset=0):
 
 
 @contextmanager
-def mock_timestamp_now(now=None):
+def mock_timestamp_now(now=None, klass=Timestamp):
     if now is None:
-        now = Timestamp.now()
+        now = klass.now()
     with mocklib.patch('swift.common.utils.Timestamp.now',
                        classmethod(lambda c: now)):
         yield now
 
 
+@contextmanager
+def mock_timestamp_now_with_iter(ts_iter):
+    with mocklib.patch('swift.common.utils.Timestamp.now',
+                       side_effect=ts_iter):
+        yield
+
+
 class Timeout(object):
     def __init__(self, seconds):
         self.seconds = seconds
@@ -1188,16 +1094,29 @@ def requires_o_tmpfile_support_in_tmp(func):
 
 class StubResponse(object):
 
-    def __init__(self, status, body=b'', headers=None, frag_index=None):
+    def __init__(self, status, body=b'', headers=None, frag_index=None,
+                 slowdown=None):
         self.status = status
         self.body = body
         self.readable = BytesIO(body)
+        try:
+            self._slowdown = iter(slowdown)
+        except TypeError:
+            self._slowdown = iter([slowdown])
         self.headers = HeaderKeyDict(headers)
         if frag_index is not None:
             self.headers['X-Object-Sysmeta-Ec-Frag-Index'] = frag_index
         fake_reason = ('Fake', 'This response is a lie.')
         self.reason = swob.RESPONSE_REASONS.get(status, fake_reason)[0]
 
+    def slowdown(self):
+        try:
+            wait = next(self._slowdown)
+        except StopIteration:
+            wait = None
+        if wait is not None:
+            eventlet.sleep(wait)
+
     def nuke_from_orbit(self):
         if hasattr(self, 'swift_conn'):
             self.swift_conn.close()
@@ -1211,8 +1130,13 @@ class StubResponse(object):
         return self.headers.items()
 
     def read(self, amt=0):
+        self.slowdown()
         return self.readable.read(amt)
 
+    def readline(self, size=-1):
+        self.slowdown()
+        return self.readable.readline(size)
+
     def __repr__(self):
         info = ['Status: %s' % self.status]
         if self.headers:
@@ -1257,7 +1181,7 @@ def make_ec_object_stub(test_body, policy, timestamp):
     test_body = test_body or (
         b'test' * segment_size)[:-random.randint(1, 1000)]
     timestamp = timestamp or utils.Timestamp.now()
-    etag = md5(test_body).hexdigest()
+    etag = md5(test_body, usedforsecurity=False).hexdigest()
     ec_archive_bodies = encode_frag_archive_bodies(policy, test_body)
 
     return {
