@@ -16,7 +16,6 @@
 """ Tests for swift.account.backend """
 
 from collections import defaultdict
-import hashlib
 import json
 import unittest
 import pickle
@@ -31,15 +30,17 @@ from contextlib import contextmanager
 import random
 import mock
 import base64
+import shutil
 
 import six
 
 from swift.account.backend import AccountBroker
 from swift.common.utils import Timestamp
 from test.unit import patch_policies, with_tempdir, make_timestamp_iter
-from swift.common.db import DatabaseConnectionError
+from swift.common.db import DatabaseConnectionError, TombstoneReclaimer
 from swift.common.request_helpers import get_reserved_name
 from swift.common.storage_policy import StoragePolicy, POLICIES
+from swift.common.utils import md5
 
 from test.unit.common import test_db
 
@@ -218,15 +219,17 @@ class TestAccountBroker(unittest.TestCase):
             self.assertEqual(count_reclaimable(conn, reclaim_age),
                              num_of_containers / 4)
 
-        orig__reclaim = broker._reclaim
         trace = []
 
-        def tracing_reclaim(conn, age_timestamp, marker):
-            trace.append((age_timestamp, marker,
-                          count_reclaimable(conn, age_timestamp)))
-            return orig__reclaim(conn, age_timestamp, marker)
+        class TracingReclaimer(TombstoneReclaimer):
+            def _reclaim(self, conn):
+                trace.append(
+                    (self.age_timestamp, self.marker,
+                     count_reclaimable(conn, self.age_timestamp)))
+                return super(TracingReclaimer, self)._reclaim(conn)
 
-        with mock.patch.object(broker, '_reclaim', new=tracing_reclaim), \
+        with mock.patch(
+                'swift.common.db.TombstoneReclaimer', TracingReclaimer), \
                 mock.patch('swift.common.db.RECLAIM_PAGE_SIZE', 10):
             broker.reclaim(reclaim_age, reclaim_age)
         with broker.get() as conn:
@@ -821,10 +824,10 @@ class TestAccountBroker(unittest.TestCase):
                              POLICIES.default.idx)
         text = '%s-%s' % ('a', "%s-%s-%s-%s" % (
                Timestamp(1).internal, Timestamp(0).internal, 0, 0))
-        hasha = hashlib.md5(text.encode('ascii')).digest()
+        hasha = md5(text.encode('ascii'), usedforsecurity=False).digest()
         text = '%s-%s' % ('b', "%s-%s-%s-%s" % (
                Timestamp(2).internal, Timestamp(0).internal, 0, 0))
-        hashb = hashlib.md5(text.encode('ascii')).digest()
+        hashb = md5(text.encode('ascii'), usedforsecurity=False).digest()
         hashc = ''.join(('%02x' % (ord(a) ^ ord(b) if six.PY2 else a ^ b)
                          for a, b in zip(hasha, hashb)))
         self.assertEqual(broker.get_info()['hash'], hashc)
@@ -833,7 +836,7 @@ class TestAccountBroker(unittest.TestCase):
                              POLICIES.default.idx)
         text = '%s-%s' % ('b', "%s-%s-%s-%s" % (
                Timestamp(3).internal, Timestamp(0).internal, 0, 0))
-        hashb = hashlib.md5(text.encode('ascii')).digest()
+        hashb = md5(text.encode('ascii'), usedforsecurity=False).digest()
         hashc = ''.join(('%02x' % (ord(a) ^ ord(b) if six.PY2 else a ^ b)
                          for a, b in zip(hasha, hashb)))
         self.assertEqual(broker.get_info()['hash'], hashc)
@@ -1069,6 +1072,40 @@ class TestAccountBroker(unittest.TestCase):
                 "SELECT COUNT(*) FROM policy_stat").fetchall()[0][0]
         self.assertEqual(nrows, 2)
 
+    @with_tempdir
+    def test_newid(self, tempdir):
+        # test DatabaseBroker.newid
+        db_path = os.path.join(
+            tempdir, "d1234", 'accounts', 'part', 'suffix', 'hsh')
+        os.makedirs(db_path)
+        broker = AccountBroker(os.path.join(db_path, 'my.db'),
+                               account='a')
+        broker.initialize(Timestamp('1').internal, 0)
+        id = broker.get_info()['id']
+        broker.newid('someid')
+        self.assertNotEqual(id, broker.get_info()['id'])
+        # ends in the device name (from the path) unless it's an old
+        # container with just a uuid4 (tested in legecy broker
+        # tests e.g *BeforeMetaData)
+        if len(id) > 36:
+            self.assertTrue(id.endswith('d1234'))
+        # But the newid'ed version will now have the decide
+        self.assertTrue(broker.get_info()['id'].endswith('d1234'))
+
+        # if we move the broker (happens after an rsync)
+        new_db_path = os.path.join(
+            tempdir, "d5678", 'contianers', 'part', 'suffix', 'hsh')
+        os.makedirs(new_db_path)
+        shutil.copy(os.path.join(db_path, 'my.db'),
+                    os.path.join(new_db_path, 'my.db'))
+
+        new_broker = AccountBroker(os.path.join(new_db_path, 'my.db'),
+                                   account='a')
+        new_broker.newid(id)
+        # ends in the device name (from the path)
+        self.assertFalse(new_broker.get_info()['id'].endswith('d1234'))
+        self.assertTrue(new_broker.get_info()['id'].endswith('d5678'))
+
 
 def prespi_AccountBroker_initialize(self, conn, put_timestamp, **kwargs):
     """
@@ -1894,7 +1931,7 @@ class TestAccountBrokerBeforePerPolicyContainerTrack(
                 broker, 'create_policy_stat_table',
                 side_effect=sqlite3.OperationalError('foobar')):
             with broker.get() as conn:
-                self.assertRaisesRegexp(
+                self.assertRaisesRegex(
                     sqlite3.OperationalError, '.*foobar.*',
                     broker._migrate_add_storage_policy_index,
                     conn=conn)
