@@ -23,9 +23,8 @@ from swift.common import swob
 
 from swift.common.middleware.s3api.s3api import filter_factory
 from swift.common.middleware.s3api.etree import fromstring
-from swift.common.middleware.s3api.utils import Config
 
-from test.unit import debug_logger
+from test.debug_logger import debug_logger
 from test.unit.common.middleware.s3api.helpers import FakeSwift
 
 
@@ -70,8 +69,8 @@ class S3ApiTestCase(unittest.TestCase):
         unittest.TestCase.__init__(self, name)
 
     def setUp(self):
-        # setup default config
-        self.conf = Config({
+        # setup default config dict
+        self.conf = {
             'allow_no_owner': False,
             'location': 'us-east-1',
             'dns_compliant_bucket_names': True,
@@ -81,19 +80,20 @@ class S3ApiTestCase(unittest.TestCase):
             's3_acl': False,
             'storage_domain': 'localhost',
             'auth_pipeline_check': True,
-            'max_upload_part_num': 1000,
+            'max_upload_part_num': 10000,
             'check_bucket_owner': False,
             'force_swift_request_proxy_log': False,
             'allow_multipart_uploads': True,
             'min_segment_size': 5242880,
-        })
-        # those 2 settings has existed the original test setup
-        self.conf.log_level = 'debug'
+            'log_level': 'debug'
+        }
 
         self.app = FakeApp()
         self.swift = self.app.swift
+        # note: self.conf has no __file__ key so check_pipeline will be skipped
+        # when constructing self.s3api
         self.s3api = filter_factory({}, **self.conf)(self.app)
-        self.s3api.logger = self.swift.logger = debug_logger()
+        self.logger = self.s3api.logger = self.swift.logger = debug_logger()
 
         self.swift.register('HEAD', '/v1/AUTH_test',
                             swob.HTTPOk, {}, None)
@@ -135,7 +135,8 @@ class S3ApiTestCase(unittest.TestCase):
         return elem.find('./Message').text
 
     def _test_method_error(self, method, path, response_class, headers={},
-                           env={}, expected_xml_tags=None):
+                           env={}, expected_xml_tags=None,
+                           expected_status=None):
         if not path.startswith('/'):
             path = '/' + path  # add a missing slash before the path
 
@@ -149,6 +150,8 @@ class S3ApiTestCase(unittest.TestCase):
         env.update({'REQUEST_METHOD': method})
         req = swob.Request.blank(path, environ=env, headers=headers)
         status, headers, body = self.call_s3api(req)
+        if expected_status is not None:
+            self.assertEqual(status, expected_status)
         if expected_xml_tags is not None:
             elem = fromstring(body, 'Error')
             self.assertEqual(set(expected_xml_tags),
