@@ -16,7 +16,8 @@
 import binascii
 import unittest
 from datetime import datetime
-import hashlib
+import functools
+from hashlib import sha256
 import os
 from os.path import join
 import time
@@ -26,15 +27,19 @@ import json
 
 from swift.common import swob
 from swift.common.swob import Request
+from swift.common.middleware.proxy_logging import ProxyLoggingMiddleware
+from test.unit import mock_timestamp_now
 
 from test.unit.common.middleware.s3api import S3ApiTestCase
 from test.unit.common.middleware.s3api.test_s3_acl import s3acl
+from swift.common.middleware.s3api.s3request import SigV4Request
 from swift.common.middleware.s3api.subresource import ACL, User, encode_acl, \
     Owner, Grant
 from swift.common.middleware.s3api.etree import fromstring
 from swift.common.middleware.s3api.utils import mktime, S3Timestamp
 from swift.common.middleware.versioned_writes.object_versioning import \
     DELETE_MARKER_CONTENT_TYPE
+from swift.common.utils import md5
 
 
 class TestS3ApiObj(S3ApiTestCase):
@@ -43,7 +48,7 @@ class TestS3ApiObj(S3ApiTestCase):
         super(TestS3ApiObj, self).setUp()
 
         self.object_body = b'hello'
-        self.etag = hashlib.md5(self.object_body).hexdigest()
+        self.etag = md5(self.object_body, usedforsecurity=False).hexdigest()
         self.last_modified = 'Fri, 01 Apr 2014 12:00:00 GMT'
 
         self.response_headers = {'Content-Type': 'text/html',
@@ -77,6 +82,8 @@ class TestS3ApiObj(S3ApiTestCase):
                                      'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '200')
+        # we'll want this for logging
+        self.assertEqual(req.headers['X-Backend-Storage-Policy-Index'], '2')
 
         unexpected_headers = []
         for key, val in self.response_headers.items():
@@ -173,6 +180,19 @@ class TestS3ApiObj(S3ApiTestCase):
     def test_object_HEAD(self):
         self._test_object_GETorHEAD('HEAD')
 
+    def test_object_policy_index_logging(self):
+        req = Request.blank('/bucket/object',
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header()})
+        self.s3api = ProxyLoggingMiddleware(self.s3api, {}, logger=self.logger)
+        status, headers, body = self.call_s3api(req)
+        access_lines = self.logger.get_lines_for_level('info')
+        self.assertEqual(1, len(access_lines))
+        parts = access_lines[0].split()
+        self.assertEqual(' '.join(parts[3:7]),
+                         'GET /bucket/object HTTP/1.0 200')
+        self.assertEqual(parts[-1], '2')
+
     def _test_object_HEAD_Range(self, range_value):
         req = Request.blank('/bucket/object',
                             environ={'REQUEST_METHOD': 'HEAD'},
@@ -288,6 +308,22 @@ class TestS3ApiObj(S3ApiTestCase):
         code = self._test_method_error('GET', '/bucket/object',
                                        swob.HTTPServiceUnavailable)
         self.assertEqual(code, 'ServiceUnavailable')
+        code = self._test_method_error('GET', '/bucket/object',
+                                       swob.HTTPConflict)
+        self.assertEqual(code, 'BrokenMPU')
+
+        code = self._test_method_error(
+            'GET', '/bucket/object',
+            functools.partial(swob.Response, status='498 Rate Limited'),
+            expected_status='503 Slow Down')
+        self.assertEqual(code, 'SlowDown')
+
+        with patch.object(self.s3api.conf, 'ratelimit_as_client_error', True):
+            code = self._test_method_error(
+                'GET', '/bucket/object',
+                functools.partial(swob.Response, status='498 Rate Limited'),
+                expected_status='429 Slow Down')
+            self.assertEqual(code, 'SlowDown')
 
     @s3acl
     def test_object_GET(self):
@@ -544,6 +580,10 @@ class TestS3ApiObj(S3ApiTestCase):
         code = self._test_method_error('PUT', '/bucket/object',
                                        swob.HTTPRequestTimeout)
         self.assertEqual(code, 'RequestTimeout')
+        code = self._test_method_error('PUT', '/bucket/object',
+                                       swob.HTTPClientDisconnect,
+                                       {})
+        self.assertEqual(code, 'RequestTimeout')
 
     def test_object_PUT_with_version(self):
         self.swift.register('GET',
@@ -627,7 +667,7 @@ class TestS3ApiObj(S3ApiTestCase):
 
     @s3acl
     def test_object_PUT_v4(self):
-        body_sha = hashlib.sha256(self.object_body).hexdigest()
+        body_sha = sha256(self.object_body).hexdigest()
         req = Request.blank(
             '/bucket/object',
             environ={'REQUEST_METHOD': 'PUT'},
@@ -706,6 +746,8 @@ class TestS3ApiObj(S3ApiTestCase):
         _, _, headers = self.swift.calls_with_headers[-1]
         # No way to determine ETag to send
         self.assertNotIn('etag', headers)
+        self.assertIn(b'UNSIGNED-PAYLOAD', SigV4Request(
+            req.environ, self.s3api.conf)._canonical_request())
 
     def test_object_PUT_headers(self):
         content_md5 = binascii.b2a_base64(binascii.a2b_hex(self.etag)).strip()
@@ -793,6 +835,30 @@ class TestS3ApiObj(S3ApiTestCase):
                    return_value=timestamp):
             return self.call_s3api(req)
 
+    def test_simple_object_copy(self):
+        self.swift.register('HEAD', '/v1/AUTH_test/some/source',
+                            swob.HTTPOk, {
+                                'x-backend-storage-policy-index': '1',
+                            }, None)
+        req = Request.blank(
+            '/bucket/object', method='PUT',
+            headers={
+                'Authorization': 'AWS test:tester:hmac',
+                'X-Amz-Copy-Source': '/some/source',
+                'Date': self.get_date_header(),
+            },
+        )
+        timestamp = time.time()
+        with patch('swift.common.middleware.s3api.utils.time.time',
+                   return_value=timestamp):
+            status, headers, body = self.call_s3api(req)
+        self.assertEqual(status.split()[0], '200')
+        head_call, put_call = self.swift.calls_with_headers
+        self.assertEqual(
+            head_call.headers['x-backend-storage-policy-index'], '1')
+        self.assertEqual(put_call.headers['x-copy-from'], '/some/source')
+        self.assertNotIn('x-backend-storage-policy-index', put_call.headers)
+
     @s3acl
     def test_object_PUT_copy(self):
         def do_test(src_path):
@@ -831,33 +897,29 @@ class TestS3ApiObj(S3ApiTestCase):
 
     @s3acl
     def test_object_PUT_copy_metadata_replace(self):
-        date_header = self.get_date_header()
-        timestamp = mktime(date_header)
-        allowed_last_modified = [S3Timestamp(timestamp).s3xmlformat]
-        status, headers, body = \
-            self._test_object_PUT_copy(
-                swob.HTTPOk,
-                {'X-Amz-Metadata-Directive': 'REPLACE',
-                 'X-Amz-Meta-Something': 'oh hai',
-                 'X-Amz-Meta-Unreadable-Prefix': '\x04w',
-                 'X-Amz-Meta-Unreadable-Suffix': 'h\x04',
-                 'X-Amz-Meta-Lots-Of-Unprintable': 5 * '\x04',
-                 'Cache-Control': 'hello',
-                 'content-disposition': 'how are you',
-                 'content-encoding': 'good and you',
-                 'content-language': 'great',
-                 'content-type': 'so',
-                 'expires': 'yeah',
-                 'x-robots-tag': 'bye'})
-        date_header = self.get_date_header()
-        timestamp = mktime(date_header)
-        allowed_last_modified.append(S3Timestamp(timestamp).s3xmlformat)
+        with mock_timestamp_now(klass=S3Timestamp) as now:
+            status, headers, body = \
+                self._test_object_PUT_copy(
+                    swob.HTTPOk,
+                    {'X-Amz-Metadata-Directive': 'REPLACE',
+                     'X-Amz-Meta-Something': 'oh hai',
+                     'X-Amz-Meta-Unreadable-Prefix': '\x04w',
+                     'X-Amz-Meta-Unreadable-Suffix': 'h\x04',
+                     'X-Amz-Meta-Lots-Of-Unprintable': 5 * '\x04',
+                     'Cache-Control': 'hello',
+                     'content-disposition': 'how are you',
+                     'content-encoding': 'good and you',
+                     'content-language': 'great',
+                     'content-type': 'so',
+                     'expires': 'yeah',
+                     'x-robots-tag': 'bye'})
 
         self.assertEqual(status.split()[0], '200')
         self.assertEqual(headers['Content-Type'], 'application/xml')
         self.assertIsNone(headers.get('etag'))
         elem = fromstring(body, 'CopyObjectResult')
-        self.assertIn(elem.find('LastModified').text, allowed_last_modified)
+        self.assertEqual(S3Timestamp(now.ceil()).s3xmlformat,
+                         elem.find('LastModified').text)
         self.assertEqual(elem.find('ETag').text, '"%s"' % self.etag)
 
         _, _, headers = self.swift.calls_with_headers[-1]
@@ -885,34 +947,30 @@ class TestS3ApiObj(S3ApiTestCase):
 
     @s3acl
     def test_object_PUT_copy_metadata_copy(self):
-        date_header = self.get_date_header()
-        timestamp = mktime(date_header)
-        allowed_last_modified = [S3Timestamp(timestamp).s3xmlformat]
-        status, headers, body = \
-            self._test_object_PUT_copy(
-                swob.HTTPOk,
-                {'X-Amz-Metadata-Directive': 'COPY',
-                 'X-Amz-Meta-Something': 'oh hai',
-                 'X-Amz-Meta-Unreadable-Prefix': '\x04w',
-                 'X-Amz-Meta-Unreadable-Suffix': 'h\x04',
-                 'X-Amz-Meta-Lots-Of-Unprintable': 5 * '\x04',
-                 'Cache-Control': 'hello',
-                 'content-disposition': 'how are you',
-                 'content-encoding': 'good and you',
-                 'content-language': 'great',
-                 'content-type': 'so',
-                 'expires': 'yeah',
-                 'x-robots-tag': 'bye'})
-        date_header = self.get_date_header()
-        timestamp = mktime(date_header)
-        allowed_last_modified.append(S3Timestamp(timestamp).s3xmlformat)
+        with mock_timestamp_now(klass=S3Timestamp) as now:
+            status, headers, body = \
+                self._test_object_PUT_copy(
+                    swob.HTTPOk,
+                    {'X-Amz-Metadata-Directive': 'COPY',
+                     'X-Amz-Meta-Something': 'oh hai',
+                     'X-Amz-Meta-Unreadable-Prefix': '\x04w',
+                     'X-Amz-Meta-Unreadable-Suffix': 'h\x04',
+                     'X-Amz-Meta-Lots-Of-Unprintable': 5 * '\x04',
+                     'Cache-Control': 'hello',
+                     'content-disposition': 'how are you',
+                     'content-encoding': 'good and you',
+                     'content-language': 'great',
+                     'content-type': 'so',
+                     'expires': 'yeah',
+                     'x-robots-tag': 'bye'})
 
         self.assertEqual(status.split()[0], '200')
         self.assertEqual(headers['Content-Type'], 'application/xml')
         self.assertIsNone(headers.get('etag'))
 
         elem = fromstring(body, 'CopyObjectResult')
-        self.assertIn(elem.find('LastModified').text, allowed_last_modified)
+        self.assertEqual(S3Timestamp(now.ceil()).s3xmlformat,
+                         elem.find('LastModified').text)
         self.assertEqual(elem.find('ETag').text, '"%s"' % self.etag)
 
         _, _, headers = self.swift.calls_with_headers[-1]
@@ -1519,6 +1577,41 @@ class TestS3ApiObj(S3ApiTestCase):
             key, arg = q.split('=')
             query[key] = arg
         self.assertEqual(query['multipart-manifest'], 'delete')
+        # HEAD did not indicate that it was an S3 MPU, so no async delete
+        self.assertNotIn('async', query)
+        self.assertNotIn('Content-Type', headers)
+
+    @s3acl
+    def test_slo_object_async_DELETE(self):
+        self.swift.register('HEAD', '/v1/AUTH_test/bucket/object',
+                            swob.HTTPOk,
+                            {'x-static-large-object': 'True',
+                             'x-object-sysmeta-s3api-etag': 's3-style-etag'},
+                            None)
+        self.swift.register('DELETE', '/v1/AUTH_test/bucket/object',
+                            swob.HTTPNoContent, {}, '')
+        req = Request.blank('/bucket/object',
+                            environ={'REQUEST_METHOD': 'DELETE'},
+                            headers={'Authorization': 'AWS test:tester:hmac',
+                                     'Date': self.get_date_header(),
+                                     'Content-Type': 'foo/bar'})
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status.split()[0], '204')
+        self.assertEqual(body, b'')
+
+        self.assertIn(('HEAD', '/v1/AUTH_test/bucket/object?symlink=get'),
+                      self.swift.calls)
+        self.assertIn(('DELETE', '/v1/AUTH_test/bucket/object'
+                                 '?async=on&multipart-manifest=delete'),
+                      self.swift.calls)
+        _, path, headers = self.swift.calls_with_headers[-1]
+        path, query_string = path.split('?', 1)
+        query = {}
+        for q in query_string.split('&'):
+            key, arg = q.split('=')
+            query[key] = arg
+        self.assertEqual(query['multipart-manifest'], 'delete')
+        self.assertEqual(query['async'], 'on')
         self.assertNotIn('Content-Type', headers)
 
     def _test_object_for_s3acl(self, method, account):
@@ -1669,6 +1762,155 @@ class TestS3ApiObj(S3ApiTestCase):
             'test:write', 'READ', src_path='')
         self.assertEqual(status.split()[0], '400')
 
+    def test_cors_preflight(self):
+        req = Request.blank(
+            '/bucket/cors-object',
+            environ={'REQUEST_METHOD': 'OPTIONS'},
+            headers={'Origin': 'http://example.com',
+                     'Access-Control-Request-Method': 'GET',
+                     'Access-Control-Request-Headers': 'authorization'})
+        self.s3api.conf.cors_preflight_allow_origin = ['*']
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status, '200 OK')
+        self.assertDictEqual(headers, {
+            'Allow': 'GET, HEAD, PUT, POST, DELETE, OPTIONS',
+            'Access-Control-Allow-Origin': 'http://example.com',
+            'Access-Control-Allow-Methods': ('GET, HEAD, PUT, POST, DELETE, '
+                                             'OPTIONS'),
+            'Access-Control-Allow-Headers': 'authorization',
+            'Vary': 'Origin, Access-Control-Request-Headers',
+        })
+
+        # test more allow_origins
+        self.s3api.conf.cors_preflight_allow_origin = ['http://example.com',
+                                                       'http://other.com']
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status, '200 OK')
+        self.assertDictEqual(headers, {
+            'Allow': 'GET, HEAD, PUT, POST, DELETE, OPTIONS',
+            'Access-Control-Allow-Origin': 'http://example.com',
+            'Access-Control-Allow-Methods': ('GET, HEAD, PUT, POST, DELETE, '
+                                             'OPTIONS'),
+            'Access-Control-Allow-Headers': 'authorization',
+            'Vary': 'Origin, Access-Control-Request-Headers',
+        })
+
+        # test presigned urls
+        req = Request.blank(
+            '/bucket/cors-object?AWSAccessKeyId=test%3Atester&'
+            'Expires=1621558415&Signature=MKMdW3FpYcoFEJlTLF3EhP7AJgc%3D',
+            environ={'REQUEST_METHOD': 'OPTIONS'},
+            headers={'Origin': 'http://example.com',
+                     'Access-Control-Request-Method': 'PUT'})
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status, '200 OK')
+        self.assertDictEqual(headers, {
+            'Allow': 'GET, HEAD, PUT, POST, DELETE, OPTIONS',
+            'Access-Control-Allow-Origin': 'http://example.com',
+            'Access-Control-Allow-Methods': ('GET, HEAD, PUT, POST, DELETE, '
+                                             'OPTIONS'),
+            'Vary': 'Origin, Access-Control-Request-Headers',
+        })
+        req = Request.blank(
+            '/bucket/cors-object?X-Amz-Algorithm=AWS4-HMAC-SHA256&'
+            'X-Amz-Credential=test%3Atester%2F20210521%2Fus-east-1%2Fs3%2F'
+            'aws4_request&X-Amz-Date=20210521T003835Z&X-Amz-Expires=900&'
+            'X-Amz-Signature=e413549f2cbeddb457c5fddb2d28820ce58de514bb900'
+            '5d588800d7ebb1a6a2d&X-Amz-SignedHeaders=host',
+            environ={'REQUEST_METHOD': 'OPTIONS'},
+            headers={'Origin': 'http://example.com',
+                     'Access-Control-Request-Method': 'DELETE'})
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status, '200 OK')
+        self.assertDictEqual(headers, {
+            'Allow': 'GET, HEAD, PUT, POST, DELETE, OPTIONS',
+            'Access-Control-Allow-Origin': 'http://example.com',
+            'Access-Control-Allow-Methods': ('GET, HEAD, PUT, POST, DELETE, '
+                                             'OPTIONS'),
+            'Vary': 'Origin, Access-Control-Request-Headers',
+        })
+
+        # Wrong protocol
+        self.s3api.conf.cors_preflight_allow_origin = ['https://example.com']
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status, '401 Unauthorized')
+        self.assertEqual(headers, {
+            'Allow': 'GET, HEAD, PUT, POST, DELETE, OPTIONS',
+        })
+
+    def test_cors_headers(self):
+        # note: Access-Control-Allow-Methods would normally be expected in
+        # response to an OPTIONS request but its included here in GET/PUT tests
+        # to check that it is always passed back in S3Response
+        cors_headers = {
+            'Access-Control-Allow-Origin': '*',
+            'Access-Control-Allow-Methods': ('GET, PUT, POST, COPY, '
+                                             'DELETE, PUT, OPTIONS'),
+            'Access-Control-Expose-Headers':
+                'x-object-meta-test, x-object-meta-test=5funderscore, etag',
+        }
+        get_resp_headers = self.response_headers
+        get_resp_headers['x-object-meta-test=5funderscore'] = 'underscored'
+        self.swift.register(
+            'GET', '/v1/AUTH_test/bucket/cors-object', swob.HTTPOk,
+            dict(get_resp_headers, **cors_headers),
+            self.object_body)
+        self.swift.register(
+            'PUT', '/v1/AUTH_test/bucket/cors-object', swob.HTTPCreated,
+            dict({'etag': self.etag,
+                  'last-modified': self.last_modified,
+                  'x-object-meta-something': 'oh hai',
+                  'x-object-meta-test=5funderscore': 'underscored'},
+                 **cors_headers),
+            None)
+
+        req = Request.blank(
+            '/bucket/cors-object',
+            environ={'REQUEST_METHOD': 'GET'},
+            headers={'Authorization': 'AWS test:tester:hmac',
+                     'Date': self.get_date_header(),
+                     'Origin': 'http://example.com',
+                     'Access-Control-Request-Method': 'GET',
+                     'Access-Control-Request-Headers': 'authorization'})
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status, '200 OK')
+        self.assertIn('Access-Control-Allow-Origin', headers)
+        self.assertEqual(headers['Access-Control-Allow-Origin'], '*')
+        self.assertIn('Access-Control-Expose-Headers', headers)
+        self.assertEqual(
+            headers['Access-Control-Expose-Headers'],
+            'x-amz-meta-test, x-amz-meta-test_underscore, etag, '
+            'x-amz-request-id, x-amz-id-2')
+        self.assertIn('Access-Control-Allow-Methods', headers)
+        self.assertEqual(
+            headers['Access-Control-Allow-Methods'],
+            'GET, PUT, POST, DELETE, PUT, OPTIONS')
+        self.assertIn('x-amz-meta-test_underscore', headers)
+        self.assertEqual('underscored', headers['x-amz-meta-test_underscore'])
+
+        req = Request.blank(
+            '/bucket/cors-object',
+            environ={'REQUEST_METHOD': 'PUT'},
+            headers={'Authorization': 'AWS test:tester:hmac',
+                     'Date': self.get_date_header(),
+                     'Origin': 'http://example.com',
+                     'Access-Control-Request-Method': 'PUT',
+                     'Access-Control-Request-Headers': 'authorization'})
+        status, headers, body = self.call_s3api(req)
+        self.assertEqual(status, '200 OK')
+        self.assertIn('Access-Control-Allow-Origin', headers)
+        self.assertEqual(headers['Access-Control-Allow-Origin'], '*')
+        self.assertIn('Access-Control-Expose-Headers', headers)
+        self.assertEqual(
+            headers['Access-Control-Expose-Headers'],
+            'x-amz-meta-test, x-amz-meta-test_underscore, etag, '
+            'x-amz-request-id, x-amz-id-2')
+        self.assertIn('Access-Control-Allow-Methods', headers)
+        self.assertEqual(
+            headers['Access-Control-Allow-Methods'],
+            'GET, PUT, POST, DELETE, PUT, OPTIONS')
+        self.assertEqual('underscored', headers['x-amz-meta-test_underscore'])
+
 
 class TestS3ApiObjNonUTC(TestS3ApiObj):
     def setUp(self):
