@@ -14,7 +14,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import hashlib
 import json
 import mock
 import shutil
@@ -26,7 +25,7 @@ import unittest
 from swift.common import swob
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.middleware import dlo
-from swift.common.utils import closing_if_possible
+from swift.common.utils import closing_if_possible, md5
 from test.unit.common.middleware.helpers import FakeSwift
 
 
@@ -36,7 +35,7 @@ LIMIT = 'swift.common.constraints.CONTAINER_LISTING_LIMIT'
 def md5hex(s):
     if not isinstance(s, bytes):
         s = s.encode('utf-8')
-    return hashlib.md5(s).hexdigest()
+    return md5(s, usedforsecurity=False).hexdigest()
 
 
 class DloTestCase(unittest.TestCase):
@@ -583,7 +582,23 @@ class TestDloGetManifest(DloTestCase):
             self.assertFalse('If-Modified-Since' in hdrs)
             self.assertFalse('If-Unmodified-Since' in hdrs)
 
-    def test_error_fetching_first_segment(self):
+    def test_server_error_fetching_first_segment(self):
+        self.app.register(
+            'GET', '/v1/AUTH_test/c/seg_01',
+            swob.HTTPServiceUnavailable, {}, None)
+
+        req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
+                                 environ={'REQUEST_METHOD': 'GET'})
+        status, headers, body = self.call_dlo(req)
+        self.assertEqual(status, "503 Service Unavailable")
+        self.assertEqual(self.app.unread_requests, {})
+        self.assertEqual(self.dlo.logger.get_lines_for_level('error'), [
+            'While processing manifest /v1/AUTH_test/mancon/manifest, '
+            'got 503 (<html><h1>Service Unavailable</h1><p>The server is '
+            'curren...) while retrieving /v1/AUTH_test/c/seg_01',
+        ])
+
+    def test_client_error_fetching_first_segment(self):
         self.app.register(
             'GET', '/v1/AUTH_test/c/seg_01',
             swob.HTTPForbidden, {}, None)
@@ -663,7 +678,7 @@ class TestDloGetManifest(DloTestCase):
         self.app.register(
             'GET', '/v1/AUTH_test/c/seg_02',
             swob.HTTPOk, {'Content-Length': '5', 'Etag': md5hex("bbbbb")},
-            'bbWRONGbb')
+            'WRONG')
 
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
                                  environ={'REQUEST_METHOD': 'GET'})
@@ -671,8 +686,35 @@ class TestDloGetManifest(DloTestCase):
         headers = HeaderKeyDict(headers)
 
         self.assertEqual(status, "200 OK")
+        self.assertEqual(headers['Content-Length'], "25")
         # stop after error
-        self.assertEqual(body, b"aaaaabbWRONGbb")
+        self.assertEqual(body, b"aaaaaWRONG")
+        log_lines = self.dlo.logger.get_lines_for_level('error')
+        self.assertEqual(len(log_lines), 1,
+                         'Expected one log line, got %r' % log_lines)
+        self.assertEqual(log_lines[0][:21], 'Bad MD5 checksum for ')
+
+    def test_mismatched_length_fetching_second_segment(self):
+        self.app.register(
+            'GET', '/v1/AUTH_test/c/seg_02',
+            swob.HTTPOk, {'Content-Length': '5', 'Etag': md5hex("bbbb")},
+            # Use a list so we can get a discrepency between content-length and
+            # number of bytes in the app_iter
+            [b'b' * 4])
+
+        req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
+                                 environ={'REQUEST_METHOD': 'GET'})
+        status, headers, body = self.call_dlo(req)
+        headers = HeaderKeyDict(headers)
+
+        self.assertEqual(status, "200 OK")
+        self.assertEqual(headers['Content-Length'], "25")
+        # stop after error
+        self.assertEqual(body, b"aaaaabbbb")
+        log_lines = self.dlo.logger.get_lines_for_level('error')
+        self.assertEqual(len(log_lines), 1,
+                         'Expected one log line, got %r' % log_lines)
+        self.assertEqual(log_lines[0][:24], 'Bad response length for ')
 
     def test_etag_comparison_ignores_quotes(self):
         # a little future-proofing here in case we ever fix this in swob
@@ -695,7 +737,8 @@ class TestDloGetManifest(DloTestCase):
         status, headers, body = self.call_dlo(req)
         headers = HeaderKeyDict(headers)
         self.assertEqual(headers["Etag"],
-                         '"' + hashlib.md5(b"abcdef").hexdigest() + '"')
+                         '"' + md5(b"abcdef",
+                                   usedforsecurity=False).hexdigest() + '"')
 
     def test_object_prefix_quoting(self):
         self.app.register(
