@@ -17,6 +17,7 @@ import os
 from textwrap import dedent
 import unittest
 
+from eventlet.green import ssl
 import mock
 from six.moves.configparser import NoSectionError, NoOptionError
 
@@ -36,7 +37,7 @@ class FakeApp(object):
 class ExcConfigParser(object):
 
     def read(self, path):
-        raise Exception('read called with %r' % path)
+        raise RuntimeError('read called with %r' % path)
 
 
 class EmptyConfigParser(object):
@@ -46,13 +47,13 @@ class EmptyConfigParser(object):
 
 
 def get_config_parser(memcache_servers='1.2.3.4:5',
-                      memcache_serialization_support='1',
                       memcache_max_connections='4',
-                      section='memcache'):
+                      section='memcache',
+                      item_size_warning_threshold='75'):
     _srvs = memcache_servers
-    _sers = memcache_serialization_support
     _maxc = memcache_max_connections
     _section = section
+    _warn_threshold = item_size_warning_threshold
 
     class SetConfigParser(object):
 
@@ -61,9 +62,7 @@ def get_config_parser(memcache_servers='1.2.3.4:5',
                 raise NoSectionError(section_name)
             return {
                 'memcache_servers': memcache_servers,
-                'memcache_serialization_support':
-                memcache_serialization_support,
-                'memcache_max_connections': memcache_max_connections,
+                'memcache_max_connections': memcache_max_connections
             }
 
         def read(self, path):
@@ -75,15 +74,15 @@ def get_config_parser(memcache_servers='1.2.3.4:5',
                     if _srvs == 'error':
                         raise NoOptionError(option, section)
                     return _srvs
-                elif option == 'memcache_serialization_support':
-                    if _sers == 'error':
-                        raise NoOptionError(option, section)
-                    return _sers
                 elif option in ('memcache_max_connections',
                                 'max_connections'):
                     if _maxc == 'error':
                         raise NoOptionError(option, section)
                     return _maxc
+                elif option == 'item_size_warning_threshold':
+                    if _warn_threshold == 'error':
+                        raise NoOptionError(option, section)
+                    return _warn_threshold
                 else:
                     raise NoOptionError(option, section)
             else:
@@ -111,16 +110,14 @@ class TestCacheMiddleware(unittest.TestCase):
         with mock.patch.object(memcache, 'ConfigParser', ExcConfigParser):
             for d in ({},
                       {'memcache_servers': '6.7.8.9:10'},
-                      {'memcache_serialization_support': '0'},
                       {'memcache_max_connections': '30'},
+                      {'item_size_warning_threshold': 75},
                       {'memcache_servers': '6.7.8.9:10',
-                       'memcache_serialization_support': '0'},
-                      {'memcache_servers': '6.7.8.9:10',
+                       'item_size_warning_threshold': '75'},
+                      {'item_size_warning_threshold': '75',
                        'memcache_max_connections': '30'},
-                      {'memcache_serialization_support': '0',
-                       'memcache_max_connections': '30'}
                       ):
-                with self.assertRaises(Exception) as catcher:
+                with self.assertRaises(RuntimeError) as catcher:
                     memcache.MemcacheMiddleware(FakeApp(), d)
                 self.assertEqual(
                     str(catcher.exception),
@@ -128,57 +125,72 @@ class TestCacheMiddleware(unittest.TestCase):
 
     def test_conf_set_no_read(self):
         with mock.patch.object(memcache, 'ConfigParser', ExcConfigParser):
-            exc = None
-            try:
-                memcache.MemcacheMiddleware(
-                    FakeApp(), {'memcache_servers': '1.2.3.4:5',
-                                'memcache_serialization_support': '2',
-                                'memcache_max_connections': '30'})
-            except Exception as err:
-                exc = err
-        self.assertIsNone(exc)
+            memcache.MemcacheMiddleware(
+                FakeApp(), {'memcache_servers': '1.2.3.4:5',
+                            'memcache_max_connections': '30',
+                            'item_size_warning_threshold': '80'})
 
     def test_conf_default(self):
         with mock.patch.object(memcache, 'ConfigParser', EmptyConfigParser):
             app = memcache.MemcacheMiddleware(FakeApp(), {})
         self.assertEqual(app.memcache_servers, '127.0.0.1:11211')
-        self.assertEqual(app.memcache._allow_pickle, False)
-        self.assertEqual(app.memcache._allow_unpickle, False)
         self.assertEqual(
             app.memcache._client_cache['127.0.0.1:11211'].max_size, 2)
+        self.assertEqual(app.memcache.item_size_warning_threshold, -1)
 
     def test_conf_inline(self):
         with mock.patch.object(memcache, 'ConfigParser', get_config_parser()):
             app = memcache.MemcacheMiddleware(
                 FakeApp(),
                 {'memcache_servers': '6.7.8.9:10',
-                 'memcache_serialization_support': '0',
-                 'memcache_max_connections': '5'})
+                 'memcache_max_connections': '5',
+                 'item_size_warning_threshold': '75'})
         self.assertEqual(app.memcache_servers, '6.7.8.9:10')
-        self.assertEqual(app.memcache._allow_pickle, True)
-        self.assertEqual(app.memcache._allow_unpickle, True)
         self.assertEqual(
             app.memcache._client_cache['6.7.8.9:10'].max_size, 5)
+        self.assertEqual(app.memcache.item_size_warning_threshold, 75)
+
+    def test_conf_inline_ratelimiting(self):
+        with mock.patch.object(memcache, 'ConfigParser', get_config_parser()):
+            app = memcache.MemcacheMiddleware(
+                FakeApp(),
+                {'error_suppression_limit': '5',
+                 'error_suppression_interval': '2.5'})
+        self.assertEqual(app.memcache._error_limit_count, 5)
+        self.assertEqual(app.memcache._error_limit_time, 2.5)
+        self.assertEqual(app.memcache._error_limit_duration, 2.5)
+
+    def test_conf_inline_tls(self):
+        fake_context = mock.Mock()
+        with mock.patch.object(ssl, 'create_default_context',
+                               return_value=fake_context):
+            with mock.patch.object(memcache, 'ConfigParser',
+                                   get_config_parser()):
+                memcache.MemcacheMiddleware(
+                    FakeApp(),
+                    {'tls_enabled': 'true',
+                     'tls_cafile': 'cafile',
+                     'tls_certfile': 'certfile',
+                     'tls_keyfile': 'keyfile'})
+            ssl.create_default_context.assert_called_with(cafile='cafile')
+            fake_context.load_cert_chain.assert_called_with('certfile',
+                                                            'keyfile')
 
     def test_conf_extra_no_section(self):
         with mock.patch.object(memcache, 'ConfigParser',
                                get_config_parser(section='foobar')):
             app = memcache.MemcacheMiddleware(FakeApp(), {})
         self.assertEqual(app.memcache_servers, '127.0.0.1:11211')
-        self.assertEqual(app.memcache._allow_pickle, False)
-        self.assertEqual(app.memcache._allow_unpickle, False)
         self.assertEqual(
             app.memcache._client_cache['127.0.0.1:11211'].max_size, 2)
 
     def test_conf_extra_no_option(self):
         replacement_parser = get_config_parser(
-            memcache_servers='error', memcache_serialization_support='error',
+            memcache_servers='error',
             memcache_max_connections='error')
         with mock.patch.object(memcache, 'ConfigParser', replacement_parser):
             app = memcache.MemcacheMiddleware(FakeApp(), {})
         self.assertEqual(app.memcache_servers, '127.0.0.1:11211')
-        self.assertEqual(app.memcache._allow_pickle, False)
-        self.assertEqual(app.memcache._allow_unpickle, False)
         self.assertEqual(
             app.memcache._client_cache['127.0.0.1:11211'].max_size, 2)
 
@@ -187,11 +199,8 @@ class TestCacheMiddleware(unittest.TestCase):
             app = memcache.MemcacheMiddleware(
                 FakeApp(),
                 {'memcache_servers': '6.7.8.9:10',
-                 'memcache_serialization_support': '0',
                  'max_connections': '5'})
         self.assertEqual(app.memcache_servers, '6.7.8.9:10')
-        self.assertEqual(app.memcache._allow_pickle, True)
-        self.assertEqual(app.memcache._allow_unpickle, True)
         self.assertEqual(
             app.memcache._client_cache['6.7.8.9:10'].max_size, 5)
 
@@ -200,20 +209,26 @@ class TestCacheMiddleware(unittest.TestCase):
             app = memcache.MemcacheMiddleware(
                 FakeApp(),
                 {'memcache_servers': '6.7.8.9:10',
-                 'memcache_serialization_support': '0',
                  'max_connections': 'bad42'})
         self.assertEqual(app.memcache_servers, '6.7.8.9:10')
-        self.assertEqual(app.memcache._allow_pickle, True)
-        self.assertEqual(app.memcache._allow_unpickle, True)
         self.assertEqual(
             app.memcache._client_cache['6.7.8.9:10'].max_size, 4)
 
+    def test_conf_inline_bad_item_warning_threshold(self):
+        with mock.patch.object(memcache, 'ConfigParser', get_config_parser()):
+            with self.assertRaises(ValueError) as err:
+                memcache.MemcacheMiddleware(
+                    FakeApp(),
+                    {'memcache_servers': '6.7.8.9:10',
+                     'memcache_serialization_support': '0',
+                     'item_size_warning_threshold': 'bad42'})
+        self.assertIn('invalid literal for int() with base 10:',
+                      str(err.exception))
+
     def test_conf_from_extra_conf(self):
         with mock.patch.object(memcache, 'ConfigParser', get_config_parser()):
             app = memcache.MemcacheMiddleware(FakeApp(), {})
         self.assertEqual(app.memcache_servers, '1.2.3.4:5')
-        self.assertEqual(app.memcache._allow_pickle, False)
-        self.assertEqual(app.memcache._allow_unpickle, True)
         self.assertEqual(
             app.memcache._client_cache['1.2.3.4:5'].max_size, 4)
 
@@ -222,8 +237,6 @@ class TestCacheMiddleware(unittest.TestCase):
                 memcache_max_connections='bad42')):
             app = memcache.MemcacheMiddleware(FakeApp(), {})
         self.assertEqual(app.memcache_servers, '1.2.3.4:5')
-        self.assertEqual(app.memcache._allow_pickle, False)
-        self.assertEqual(app.memcache._allow_unpickle, True)
         self.assertEqual(
             app.memcache._client_cache['1.2.3.4:5'].max_size, 2)
 
@@ -231,11 +244,8 @@ class TestCacheMiddleware(unittest.TestCase):
         with mock.patch.object(memcache, 'ConfigParser', get_config_parser()):
             app = memcache.MemcacheMiddleware(
                 FakeApp(),
-                {'memcache_servers': '6.7.8.9:10',
-                 'memcache_serialization_support': '0'})
+                {'memcache_servers': '6.7.8.9:10'})
         self.assertEqual(app.memcache_servers, '6.7.8.9:10')
-        self.assertEqual(app.memcache._allow_pickle, True)
-        self.assertEqual(app.memcache._allow_unpickle, True)
         self.assertEqual(
             app.memcache._client_cache['6.7.8.9:10'].max_size, 4)
 
@@ -246,20 +256,15 @@ class TestCacheMiddleware(unittest.TestCase):
                 {'memcache_servers': '6.7.8.9:10',
                  'memcache_max_connections': '42'})
         self.assertEqual(app.memcache_servers, '6.7.8.9:10')
-        self.assertEqual(app.memcache._allow_pickle, False)
-        self.assertEqual(app.memcache._allow_unpickle, True)
         self.assertEqual(
             app.memcache._client_cache['6.7.8.9:10'].max_size, 42)
 
     def test_filter_factory(self):
         factory = memcache.filter_factory({'max_connections': '3'},
-                                          memcache_servers='10.10.10.10:10',
-                                          memcache_serialization_support='1')
+                                          memcache_servers='10.10.10.10:10')
         thefilter = factory('myapp')
         self.assertEqual(thefilter.app, 'myapp')
         self.assertEqual(thefilter.memcache_servers, '10.10.10.10:10')
-        self.assertEqual(thefilter.memcache._allow_pickle, False)
-        self.assertEqual(thefilter.memcache._allow_unpickle, True)
         self.assertEqual(
             thefilter.memcache._client_cache['10.10.10.10:10'].max_size, 3)
 
@@ -305,6 +310,7 @@ class TestCacheMiddleware(unittest.TestCase):
         # tries is limited to server count
         self.assertEqual(memcache_ring._tries, 1)
         self.assertEqual(memcache_ring._io_timeout, 2.0)
+        self.assertEqual(memcache_ring.item_size_warning_threshold, -1)
 
     @with_tempdir
     def test_real_config_with_options(self, tempdir):
@@ -323,6 +329,8 @@ class TestCacheMiddleware(unittest.TestCase):
         pool_timeout = 0.5
         tries = 4
         io_timeout = 1.0
+        tls_enabled = true
+        item_size_warning_threshold = 1000
         """
         config_path = os.path.join(tempdir, 'test.conf')
         with open(config_path, 'w') as f:
@@ -336,6 +344,13 @@ class TestCacheMiddleware(unittest.TestCase):
         # tries is limited to server count
         self.assertEqual(memcache_ring._tries, 4)
         self.assertEqual(memcache_ring._io_timeout, 1.0)
+        self.assertEqual(memcache_ring._error_limit_count, 10)
+        self.assertEqual(memcache_ring._error_limit_time, 60)
+        self.assertEqual(memcache_ring._error_limit_duration, 60)
+        self.assertIsInstance(
+            list(memcache_ring._client_cache.values())[0]._tls_context,
+            ssl.SSLContext)
+        self.assertEqual(memcache_ring.item_size_warning_threshold, 1000)
 
     @with_tempdir
     def test_real_memcache_config(self, tempdir):
@@ -363,6 +378,9 @@ class TestCacheMiddleware(unittest.TestCase):
             10.0.0.4:11211
         connect_timeout = 0.5
         io_timeout = 1.0
+        error_suppression_limit = 0
+        error_suppression_interval = 1.5
+        item_size_warning_threshold = 50
         """
         memcache_config_path = os.path.join(tempdir, 'memcache.conf')
         with open(memcache_config_path, 'w') as f:
@@ -376,6 +394,10 @@ class TestCacheMiddleware(unittest.TestCase):
         self.assertEqual(memcache_ring._tries, 3)
         # memcache conf options are defaults
         self.assertEqual(memcache_ring._io_timeout, 1.0)
+        self.assertEqual(memcache_ring._error_limit_count, 0)
+        self.assertEqual(memcache_ring._error_limit_time, 1.5)
+        self.assertEqual(memcache_ring._error_limit_duration, 1.5)
+        self.assertEqual(memcache_ring.item_size_warning_threshold, 50)
 
 
 if __name__ == '__main__':
