@@ -21,19 +21,22 @@ from logging.handlers import SysLogHandler
 import six
 from six.moves.urllib.parse import unquote
 
-from swift.common.utils import get_logger, split_path
+from swift.common.utils import get_logger, split_path, StatsdClient
 from swift.common.middleware import proxy_logging
+from swift.common.registry import register_sensitive_header, \
+    register_sensitive_param, get_sensitive_headers
 from swift.common.swob import Request, Response
-from swift.common import constraints
+from swift.common import constraints, registry
 from swift.common.storage_policy import StoragePolicy
-from test.unit import FakeLogger
+from test.debug_logger import debug_logger
 from test.unit import patch_policies
 from test.unit.common.middleware.helpers import FakeAppThatExcepts
 
 
 class FakeApp(object):
 
-    def __init__(self, body=None, response_str='200 OK', policy_idx='0'):
+    def __init__(self, body=None, response_str='200 OK', policy_idx='0',
+                 chunked=False):
         if body is None:
             body = [b'FAKE APP']
         elif isinstance(body, six.binary_type):
@@ -42,6 +45,7 @@ class FakeApp(object):
         self.body = body
         self.response_str = response_str
         self.policy_idx = policy_idx
+        self.chunked = chunked
 
     def __call__(self, env, start_response):
         try:
@@ -51,8 +55,13 @@ class FakeApp(object):
         except ValueError:
             is_container_or_object_req = False
 
-        headers = [('Content-Type', 'text/plain'),
-                   ('Content-Length', str(sum(map(len, self.body))))]
+        headers = [('Content-Type', 'text/plain')]
+        if self.chunked:
+            headers.append(('Transfer-Encoding', 'chunked'))
+        elif not hasattr(self.body, 'close'):
+            content_length = sum(map(len, self.body))
+            headers.append(('Content-Length', str(content_length)))
+
         if is_container_or_object_req and self.policy_idx is not None:
             headers.append(('X-Backend-Storage-Policy-Index',
                            str(self.policy_idx)))
@@ -142,9 +151,29 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEqual(got_metrics_values_and_kwargs,
                          exp_metrics_values_and_kwargs)
 
+    def test_logger_statsd_prefix(self):
+        app = proxy_logging.ProxyLoggingMiddleware(
+            FakeApp(), {'log_statsd_host': 'example.com'})
+        self.assertIsNotNone(app.access_logger.logger.statsd_client)
+        self.assertIsInstance(app.access_logger.logger.statsd_client,
+                              StatsdClient)
+        self.assertEqual('proxy-server.',
+                         app.access_logger.logger.statsd_client._prefix)
+
+        app = proxy_logging.ProxyLoggingMiddleware(
+            FakeApp(), {'log_statsd_metric_prefix': 'foo',  # set base prefix
+                        'access_log_name': 'bar',  # not used as tail prefix
+                        'log_name': 'baz',  # not used as tail prefix
+                        'log_statsd_host': 'example.com'})
+        self.assertIsNotNone(app.access_logger.logger.statsd_client)
+        self.assertIsInstance(app.access_logger.logger.statsd_client,
+                              StatsdClient)
+        self.assertEqual('foo.proxy-server.',
+                         app.access_logger.logger.statsd_client._prefix)
+
     def test_log_request_statsd_invalid_stats_types(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         for url in ['/', '/foo', '/foo/bar', '/v1']:
             req = Request.blank(url, environ={'REQUEST_METHOD': 'GET'})
             resp = app(req.environ, start_response)
@@ -157,7 +186,7 @@ class TestProxyLogging(unittest.TestCase):
         for bad_path in ['', '/', '/bad', '/baddy/mc_badderson', '/v1',
                          '/v1/']:
             app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-            app.access_logger = FakeLogger()
+            app.access_logger = debug_logger()
             req = Request.blank(bad_path, environ={'REQUEST_METHOD': 'GET'})
             now = 10000.0
             app.log_request(req, 123, 7, 13, now, now + 2.71828182846)
@@ -192,7 +221,7 @@ class TestProxyLogging(unittest.TestCase):
                 # GET
                 app = proxy_logging.ProxyLoggingMiddleware(
                     FakeApp(body=b'7654321', response_str='321 Fubar'), {})
-                app.access_logger = FakeLogger()
+                app.access_logger = debug_logger()
                 req = Request.blank(path, environ={
                     'REQUEST_METHOD': 'GET',
                     'wsgi.input': BytesIO(b'4321')})
@@ -223,7 +252,7 @@ class TestProxyLogging(unittest.TestCase):
                     app = proxy_logging.ProxyLoggingMiddleware(
                         FakeApp(body=b'7654321', response_str='321 Fubar',
                                 policy_idx='-1'), {})
-                    app.access_logger = FakeLogger()
+                    app.access_logger = debug_logger()
                     req = Request.blank(path, environ={
                         'REQUEST_METHOD': 'GET',
                         'wsgi.input': BytesIO(b'4321')})
@@ -241,7 +270,7 @@ class TestProxyLogging(unittest.TestCase):
                 # GET with swift.proxy_access_log_made already set
                 app = proxy_logging.ProxyLoggingMiddleware(
                     FakeApp(body=b'7654321', response_str='321 Fubar'), {})
-                app.access_logger = FakeLogger()
+                app.access_logger = debug_logger()
                 req = Request.blank(path, environ={
                     'REQUEST_METHOD': 'GET',
                     'swift.proxy_access_log_made': True,
@@ -258,7 +287,7 @@ class TestProxyLogging(unittest.TestCase):
                 # PUT (no first-byte timing!)
                 app = proxy_logging.ProxyLoggingMiddleware(
                     FakeApp(body=b'87654321', response_str='314 PiTown'), {})
-                app.access_logger = FakeLogger()
+                app.access_logger = debug_logger()
                 req = Request.blank(path, environ={
                     'REQUEST_METHOD': 'PUT',
                     'wsgi.input': BytesIO(b'654321')})
@@ -291,7 +320,7 @@ class TestProxyLogging(unittest.TestCase):
                     app = proxy_logging.ProxyLoggingMiddleware(
                         FakeApp(body=b'87654321', response_str='314 PiTown',
                                 policy_idx='-1'), {})
-                    app.access_logger = FakeLogger()
+                    app.access_logger = debug_logger()
                     req = Request.blank(path, environ={
                         'REQUEST_METHOD': 'PUT',
                         'wsgi.input': BytesIO(b'654321')})
@@ -325,7 +354,7 @@ class TestProxyLogging(unittest.TestCase):
         }
         for method, exp_method in method_map.items():
             app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-            app.access_logger = FakeLogger()
+            app.access_logger = debug_logger()
             req = Request.blank('/v1/a/', environ={'REQUEST_METHOD': method})
             now = 10000.0
             app.log_request(req, 299, 11, 3, now, now + 1.17)
@@ -351,7 +380,7 @@ class TestProxyLogging(unittest.TestCase):
                 app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {
                     conf_key: 'SPECIAL,  GET,PUT ',  # crazy spaces ok
                 })
-                app.access_logger = FakeLogger()
+                app.access_logger = debug_logger()
                 req = Request.blank('/v1/a/c',
                                     environ={'REQUEST_METHOD': method})
                 now = 10000.0
@@ -363,7 +392,7 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_basic_req(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         resp = app(req.environ, start_response)
         resp_body = b''.join(resp)
@@ -377,7 +406,7 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_basic_req_second_time(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={
             'swift.proxy_access_log_made': True,
             'REQUEST_METHOD': 'GET'})
@@ -394,9 +423,10 @@ class TestProxyLogging(unittest.TestCase):
                 'template which can be edited in config: '
                 '{protocol} {path} {method} '
                 '{path.anonymized} {container.anonymized} '
-                '{request_time} {start_time.datetime} {end_time} {ttfb}')})
-        app.access_logger = FakeLogger()
-        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
+                '{request_time} {start_time.datetime} {end_time} {ttfb} '
+                '{domain}')})
+        app.access_logger = debug_logger()
+        req = Request.blank('/', headers={'Host': 'example.com'})
         with mock.patch('time.time',
                         mock.MagicMock(
                             side_effect=[10000000.0, 10000000.5, 10000001.0])):
@@ -414,6 +444,7 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEqual(log_parts[13], '26/Apr/1970/17/46/40')
         self.assertEqual(log_parts[14], '10000001.000000000')
         self.assertEqual(log_parts[15], '0.5')
+        self.assertEqual(log_parts[16], 'example.com')
         self.assertEqual(resp_body, b'FAKE APP')
 
     def test_log_msg_template_s3api(self):
@@ -422,7 +453,7 @@ class TestProxyLogging(unittest.TestCase):
             'log_msg_template': (
                 '{protocol} {path} {method} '
                 '{account} {container} {object}')})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/bucket/path/to/key', environ={
             'REQUEST_METHOD': 'GET',
             # This would actually get set in the app, but w/e
@@ -463,7 +494,7 @@ class TestProxyLogging(unittest.TestCase):
     def test_multi_segment_resp(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(
             [b'some', b'chunks', b'of data']), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
                                           'swift.source': 'SOS'})
         resp = app(req.environ, start_response)
@@ -481,7 +512,7 @@ class TestProxyLogging(unittest.TestCase):
         for conf_key in ['access_log_headers', 'log_headers']:
             app = proxy_logging.ProxyLoggingMiddleware(FakeApp(),
                                                        {conf_key: 'yes'})
-            app.access_logger = FakeLogger()
+            app.access_logger = debug_logger()
             req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
             resp = app(req.environ, start_response)
             # exhaust generator
@@ -494,7 +525,7 @@ class TestProxyLogging(unittest.TestCase):
         app = proxy_logging.ProxyLoggingMiddleware(
             FakeApp(), {'log_headers': 'yes',
                         'access_log_headers_only': 'FIRST, seCond'})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/',
                             environ={'REQUEST_METHOD': 'GET'},
                             headers={'First': '1',
@@ -514,7 +545,7 @@ class TestProxyLogging(unittest.TestCase):
         # Using default policy
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(),
                                                    {'log_headers': 'yes'})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank(
             '/v1/a/c/o/foo',
             environ={'REQUEST_METHOD': 'PUT',
@@ -534,7 +565,7 @@ class TestProxyLogging(unittest.TestCase):
         # Using a non-existent policy
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(policy_idx='-1'),
                                                    {'log_headers': 'yes'})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank(
             '/v1/a/c/o/foo',
             environ={'REQUEST_METHOD': 'PUT',
@@ -552,7 +583,7 @@ class TestProxyLogging(unittest.TestCase):
     def test_upload_size_no_policy(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(policy_idx=None),
                                                    {'log_headers': 'yes'})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank(
             '/v1/a/c/o/foo',
             environ={'REQUEST_METHOD': 'PUT',
@@ -570,7 +601,7 @@ class TestProxyLogging(unittest.TestCase):
     def test_upload_line(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeAppReadline(),
                                                    {'log_headers': 'yes'})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank(
             '/v1/a/c',
             environ={'REQUEST_METHOD': 'POST',
@@ -587,7 +618,7 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_log_query_string(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
                                           'QUERY_STRING': 'x=3'})
         resp = app(req.environ, start_response)
@@ -598,7 +629,7 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_client_logging(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
                                           'REMOTE_ADDR': '1.2.3.4'})
         resp = app(req.environ, start_response)
@@ -612,13 +643,22 @@ class TestProxyLogging(unittest.TestCase):
 
         class CloseableBody(object):
             def __init__(self):
+                self.msg = b"CloseableBody"
                 self.closed = False
 
             def close(self):
                 self.closed = True
 
             def __iter__(self):
-                return iter(["CloseableBody"])
+                return self
+
+            def __next__(self):
+                if not self.msg:
+                    raise StopIteration
+                result, self.msg = self.msg, b''
+                return result
+
+            next = __next__  # py2
 
         body = CloseableBody()
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(body), {})
@@ -629,9 +669,16 @@ class TestProxyLogging(unittest.TestCase):
         [x for x in resp]
         self.assertTrue(body.closed)
 
+    def test_chunked_response(self):
+        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(chunked=True), {})
+        req = Request.blank('/')
+        resp = app(req.environ, start_response)
+        # exhaust generator
+        [x for x in resp]
+
     def test_proxy_client_logging(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={
             'REQUEST_METHOD': 'GET',
             'REMOTE_ADDR': '1.2.3.4',
@@ -644,7 +691,7 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEqual(log_parts[1], '1.2.3.4')  # remote addr
 
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={
             'REQUEST_METHOD': 'GET',
             'REMOTE_ADDR': '1.2.3.4',
@@ -669,10 +716,18 @@ class TestProxyLogging(unittest.TestCase):
         self.assertTrue(callable(factory))
         self.assertTrue(callable(factory(FakeApp())))
 
+    def test_sensitive_headers_registered(self):
+        with mock.patch.object(registry, '_sensitive_headers', set()):
+            self.assertNotIn('x-auth-token', get_sensitive_headers())
+            self.assertNotIn('x-storage-token', get_sensitive_headers())
+            proxy_logging.filter_factory({})(FakeApp())
+            self.assertIn('x-auth-token', get_sensitive_headers())
+            self.assertIn('x-storage-token', get_sensitive_headers())
+
     def test_unread_body(self):
         app = proxy_logging.ProxyLoggingMiddleware(
             FakeApp(['some', 'stuff']), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         resp = app(req.environ, start_response)
         # read first chunk
@@ -682,9 +737,30 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEqual(log_parts[6], '499')
         self.assertEqual(log_parts[11], '4')  # write length
 
+    def test_exploding_body(self):
+
+        def exploding_body():
+            yield 'some'
+            yield 'stuff'
+            raise Exception('kaboom!')
+
+        app = proxy_logging.ProxyLoggingMiddleware(
+            FakeApp(exploding_body()), {
+                'log_msg_template': '{method} {path} '
+                '{status_int} {wire_status_int}',
+            })
+        app.access_logger = debug_logger()
+        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
+        resp = req.get_response(app)
+        with self.assertRaises(Exception) as ctx:
+            resp.body
+        self.assertEqual('kaboom!', str(ctx.exception))
+        log_parts = self._log_parts(app)
+        self.assertEqual(log_parts, ['GET', '/', '500', '200'])
+
     def test_disconnect_on_readline(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeAppReadline(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
                                           'wsgi.input': FileLikeExceptor()})
         try:
@@ -700,7 +776,7 @@ class TestProxyLogging(unittest.TestCase):
     def test_disconnect_on_read(self):
         app = proxy_logging.ProxyLoggingMiddleware(
             FakeApp(['some', 'stuff']), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
                                           'wsgi.input': FileLikeExceptor()})
         try:
@@ -716,7 +792,7 @@ class TestProxyLogging(unittest.TestCase):
     def test_app_exception(self):
         app = proxy_logging.ProxyLoggingMiddleware(
             FakeAppThatExcepts(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         try:
             app(req.environ, start_response)
@@ -732,7 +808,7 @@ class TestProxyLogging(unittest.TestCase):
                 # test the "while not chunk: chunk = next(iterator)"
                 body=[b'', b'', b'line1\n', b'line2\n'],
             ), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         resp = app(req.environ, start_response)
         resp_body = b''.join(resp)
@@ -748,9 +824,9 @@ class TestProxyLogging(unittest.TestCase):
         app = proxy_logging.ProxyLoggingMiddleware(
             FakeAppNoContentLengthNoTransferEncoding(
                 # test the "while not chunk: chunk = next(iterator)"
-                body=['', '', ''],
+                body=[b'', b'', b''],
             ), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         resp = app(req.environ, start_response)
         resp_body = b''.join(resp)
@@ -775,7 +851,7 @@ class TestProxyLogging(unittest.TestCase):
             FakeAppNoContentLengthNoTransferEncoding(
                 body=BodyGen(b'abc'),
             ), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         resp = app(req.environ, start_response)
         resp_body = b''.join(resp)
@@ -789,7 +865,7 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_req_path_info_popping(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/v1/something', environ={'REQUEST_METHOD': 'GET'})
         req.path_info_pop()
         self.assertEqual(req.environ['PATH_INFO'], '/something')
@@ -806,7 +882,7 @@ class TestProxyLogging(unittest.TestCase):
     def test_ipv6(self):
         ipv6addr = '2001:db8:85a3:8d3:1319:8a2e:370:7348'
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         req.remote_addr = ipv6addr
         resp = app(req.environ, start_response)
@@ -823,14 +899,14 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_log_info_none(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         list(app(req.environ, start_response))
         log_parts = self._log_parts(app)
         self.assertEqual(log_parts[17], '-')
 
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         req.environ['swift.log_info'] = []
         list(app(req.environ, start_response))
@@ -839,7 +915,7 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_log_info_single(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         req.environ['swift.log_info'] = ['one']
         list(app(req.environ, start_response))
@@ -848,7 +924,7 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_log_info_multiple(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         req.environ['swift.log_info'] = ['one', 'and two']
         list(app(req.environ, start_response))
@@ -857,93 +933,94 @@ class TestProxyLogging(unittest.TestCase):
 
     def test_log_auth_token(self):
         auth_token = 'b05bf940-0464-4c0e-8c70-87717d2d73e8'
+        with mock.patch.object(registry, '_sensitive_headers', set()):
+            # Default - reveal_sensitive_prefix is 16
+            # No x-auth-token header
+            app = proxy_logging.filter_factory({})(FakeApp())
+            app.access_logger = debug_logger()
+            req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
+            resp = app(req.environ, start_response)
+            resp_body = b''.join(resp)
+            log_parts = self._log_parts(app)
+            self.assertEqual(log_parts[9], '-')
+            # Has x-auth-token header
+            app = proxy_logging.filter_factory({})(FakeApp())
+            app.access_logger = debug_logger()
+            req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
+                                              'HTTP_X_AUTH_TOKEN': auth_token})
+            resp = app(req.environ, start_response)
+            resp_body = b''.join(resp)
+            log_parts = self._log_parts(app)
+            self.assertEqual(log_parts[9], 'b05bf940-0464-4c...', log_parts)
 
-        # Default - reveal_sensitive_prefix is 16
-        # No x-auth-token header
-        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
-        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
-        resp = app(req.environ, start_response)
-        resp_body = b''.join(resp)
-        log_parts = self._log_parts(app)
-        self.assertEqual(log_parts[9], '-')
-        # Has x-auth-token header
-        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
-        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
-                                          'HTTP_X_AUTH_TOKEN': auth_token})
-        resp = app(req.environ, start_response)
-        resp_body = b''.join(resp)
-        log_parts = self._log_parts(app)
-        self.assertEqual(log_parts[9], 'b05bf940-0464-4c...')
+            # Truncate to first 8 characters
+            app = proxy_logging.filter_factory(
+                {'reveal_sensitive_prefix': '8'})(FakeApp())
+            app.access_logger = debug_logger()
+            req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
+            resp = app(req.environ, start_response)
+            resp_body = b''.join(resp)
+            log_parts = self._log_parts(app)
+            self.assertEqual(log_parts[9], '-')
+            app = proxy_logging.filter_factory(
+                {'reveal_sensitive_prefix': '8'})(FakeApp())
+            app.access_logger = debug_logger()
+            req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
+                                              'HTTP_X_AUTH_TOKEN': auth_token})
+            resp = app(req.environ, start_response)
+            resp_body = b''.join(resp)
+            log_parts = self._log_parts(app)
+            self.assertEqual(log_parts[9], 'b05bf940...')
 
-        # Truncate to first 8 characters
-        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {
-            'reveal_sensitive_prefix': '8'})
-        app.access_logger = FakeLogger()
-        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
-        resp = app(req.environ, start_response)
-        resp_body = b''.join(resp)
-        log_parts = self._log_parts(app)
-        self.assertEqual(log_parts[9], '-')
-        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {
-            'reveal_sensitive_prefix': '8'})
-        app.access_logger = FakeLogger()
-        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
-                                          'HTTP_X_AUTH_TOKEN': auth_token})
-        resp = app(req.environ, start_response)
-        resp_body = b''.join(resp)
-        log_parts = self._log_parts(app)
-        self.assertEqual(log_parts[9], 'b05bf940...')
+            # Token length and reveal_sensitive_prefix are same (no truncate)
+            app = proxy_logging.filter_factory(
+                {'reveal_sensitive_prefix': str(len(auth_token))})(FakeApp())
+            app.access_logger = debug_logger()
+            req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
+                                              'HTTP_X_AUTH_TOKEN': auth_token})
+            resp = app(req.environ, start_response)
+            resp_body = b''.join(resp)
+            log_parts = self._log_parts(app)
+            self.assertEqual(log_parts[9], auth_token)
 
-        # Token length and reveal_sensitive_prefix are same (no truncate)
-        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {
-            'reveal_sensitive_prefix': str(len(auth_token))})
-        app.access_logger = FakeLogger()
-        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
-                                          'HTTP_X_AUTH_TOKEN': auth_token})
-        resp = app(req.environ, start_response)
-        resp_body = b''.join(resp)
-        log_parts = self._log_parts(app)
-        self.assertEqual(log_parts[9], auth_token)
+            # No effective limit on auth token
+            app = proxy_logging.filter_factory(
+                {'reveal_sensitive_prefix': constraints.MAX_HEADER_SIZE}
+            )(FakeApp())
+            app.access_logger = debug_logger()
+            req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
+                                              'HTTP_X_AUTH_TOKEN': auth_token})
+            resp = app(req.environ, start_response)
+            resp_body = b''.join(resp)
+            log_parts = self._log_parts(app)
+            self.assertEqual(log_parts[9], auth_token)
 
-        # No effective limit on auth token
-        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {
-            'reveal_sensitive_prefix': constraints.MAX_HEADER_SIZE})
-        app.access_logger = FakeLogger()
-        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
-                                          'HTTP_X_AUTH_TOKEN': auth_token})
-        resp = app(req.environ, start_response)
-        resp_body = b''.join(resp)
-        log_parts = self._log_parts(app)
-        self.assertEqual(log_parts[9], auth_token)
+            # Don't log x-auth-token
+            app = proxy_logging.filter_factory(
+                {'reveal_sensitive_prefix': '0'})(FakeApp())
+            app.access_logger = debug_logger()
+            req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
+            resp = app(req.environ, start_response)
+            resp_body = b''.join(resp)
+            log_parts = self._log_parts(app)
+            self.assertEqual(log_parts[9], '-')
+            app = proxy_logging.filter_factory(
+                {'reveal_sensitive_prefix': '0'})(FakeApp())
+            app.access_logger = debug_logger()
+            req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
+                                              'HTTP_X_AUTH_TOKEN': auth_token})
+            resp = app(req.environ, start_response)
+            resp_body = b''.join(resp)
+            log_parts = self._log_parts(app)
+            self.assertEqual(log_parts[9], '...')
 
-        # Don't log x-auth-token
-        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {
-            'reveal_sensitive_prefix': '0'})
-        app.access_logger = FakeLogger()
-        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
-        resp = app(req.environ, start_response)
-        resp_body = b''.join(resp)
-        log_parts = self._log_parts(app)
-        self.assertEqual(log_parts[9], '-')
-        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {
-            'reveal_sensitive_prefix': '0'})
-        app.access_logger = FakeLogger()
-        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET',
-                                          'HTTP_X_AUTH_TOKEN': auth_token})
-        resp = app(req.environ, start_response)
-        resp_body = b''.join(resp)
-        log_parts = self._log_parts(app)
-        self.assertEqual(log_parts[9], '...')
-
-        # Avoids pyflakes error, "local variable 'resp_body' is assigned to
-        # but never used
-        self.assertTrue(resp_body is not None)
+            # Avoids pyflakes error, "local variable 'resp_body' is assigned to
+            # but never used
+            self.assertTrue(resp_body is not None)
 
     def test_ensure_fields(self):
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         with mock.patch('time.time',
                         mock.MagicMock(
@@ -979,10 +1056,10 @@ class TestProxyLogging(unittest.TestCase):
         # Since no internal request is being made, outer most proxy logging
         # middleware, log1, should have performed the logging.
         app = FakeApp()
-        flg0 = FakeLogger()
+        flg0 = debug_logger()
         env = {}
         log0 = proxy_logging.ProxyLoggingMiddleware(app, env, logger=flg0)
-        flg1 = FakeLogger()
+        flg1 = debug_logger()
         log1 = proxy_logging.ProxyLoggingMiddleware(log0, env, logger=flg1)
 
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
@@ -1028,11 +1105,11 @@ class TestProxyLogging(unittest.TestCase):
         # Since an internal request is being made, inner most proxy logging
         # middleware, log0, should have performed the logging.
         app = FakeApp()
-        flg0 = FakeLogger()
+        flg0 = debug_logger()
         env = {}
         log0 = proxy_logging.ProxyLoggingMiddleware(app, env, logger=flg0)
         fake = FakeMiddleware(log0, env)
-        flg1 = FakeLogger()
+        flg1 = debug_logger()
         log1 = proxy_logging.ProxyLoggingMiddleware(fake, env, logger=flg1)
 
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
@@ -1060,7 +1137,7 @@ class TestProxyLogging(unittest.TestCase):
         # Policy index can be specified by X-Backend-Storage-Policy-Index
         # in the request header for object API
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(policy_idx='1'), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/v1/a/c/o', environ={'REQUEST_METHOD': 'PUT'})
         resp = app(req.environ, start_response)
         b''.join(resp)
@@ -1070,7 +1147,7 @@ class TestProxyLogging(unittest.TestCase):
         # Policy index can be specified by X-Backend-Storage-Policy-Index
         # in the response header for container API
         app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
-        app.access_logger = FakeLogger()
+        app.access_logger = debug_logger()
         req = Request.blank('/v1/a/c', environ={'REQUEST_METHOD': 'GET'})
 
         def fake_call(app, env, start_response):
@@ -1087,3 +1164,109 @@ class TestProxyLogging(unittest.TestCase):
             b''.join(resp)
         log_parts = self._log_parts(app)
         self.assertEqual(log_parts[20], '1')
+
+    def test_obscure_req(self):
+        app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
+        app.access_logger = debug_logger()
+
+        params = [('param_one',
+                   'some_long_string_that_might_need_to_be_obscured'),
+                  ('param_two',
+                   "super_secure_param_that_needs_to_be_obscured")]
+        headers = {'X-Auth-Token': 'this_is_my_auth_token',
+                   'X-Other-Header': 'another_header_that_we_may_obscure'}
+
+        req = Request.blank('a/c/o', environ={'REQUEST_METHOD': 'GET'},
+                            headers=headers)
+        req.params = params
+
+        # if nothing is sensitive, nothing will be obscured
+        with mock.patch.object(registry, '_sensitive_params', set()):
+            with mock.patch.object(registry, '_sensitive_headers', set()):
+                app.obscure_req(req)
+        # show that nothing changed
+        for header, expected_value in headers.items():
+            self.assertEqual(req.headers[header], expected_value)
+
+        for param, expected_value in params:
+            self.assertEqual(req.params[param], expected_value)
+
+        # If an obscured param or header doesn't exist in a req, that's fine
+        with mock.patch.object(registry, '_sensitive_params', set()):
+            with mock.patch.object(registry, '_sensitive_headers', set()):
+                register_sensitive_header('X-Not-Exist')
+                register_sensitive_param('non-existent-param')
+                app.obscure_req(req)
+
+        # show that nothing changed
+        for header, expected_value in headers.items():
+            self.assertEqual(req.headers[header], expected_value)
+
+        for param, expected_value in params:
+            self.assertEqual(req.params[param], expected_value)
+
+        def obscured_test(params, headers, params_to_add, headers_to_add,
+                          expected_params, expected_headers):
+            with mock.patch.object(registry, '_sensitive_params', set()):
+                with mock.patch.object(registry, '_sensitive_headers', set()):
+                    app = proxy_logging.ProxyLoggingMiddleware(FakeApp(), {})
+                    app.access_logger = debug_logger()
+                    req = Request.blank('a/c/o',
+                                        environ={'REQUEST_METHOD': 'GET'},
+                                        headers=dict(headers))
+                    req.params = params
+                    for param in params_to_add:
+                        register_sensitive_param(param)
+
+                    for header in headers_to_add:
+                        register_sensitive_header(header)
+
+                    app.obscure_req(req)
+                    for header, expected_value in expected_headers.items():
+                        self.assertEqual(req.headers[header], expected_value)
+
+                    for param, expected_value in expected_params:
+                        self.assertEqual(req.params[param], expected_value)
+
+        # first just 1 param
+        expected_params = list(params)
+        expected_params[0] = ('param_one', 'some_long_string...')
+        obscured_test(params, headers, ['param_one'], [], expected_params,
+                      headers)
+        # case sensitive
+        expected_params = list(params)
+        obscured_test(params, headers, ['Param_one'], [], expected_params,
+                      headers)
+        # Other param
+        expected_params = list(params)
+        expected_params[1] = ('param_two', 'super_secure_par...')
+        obscured_test(params, headers, ['param_two'], [], expected_params,
+                      headers)
+        # both
+        expected_params[0] = ('param_one', 'some_long_string...')
+        obscured_test(params, headers, ['param_two', 'param_one'], [],
+                      expected_params, headers)
+
+        # Now the headers
+        # first just 1 header
+        expected_headers = headers.copy()
+        expected_headers["X-Auth-Token"] = 'this_is_my_auth_...'
+        obscured_test(params, headers, [], ['X-Auth-Token'], params,
+                      expected_headers)
+        # case insensitive
+        obscured_test(params, headers, [], ['x-auth-token'], params,
+                      expected_headers)
+        # Other headers
+        expected_headers = headers.copy()
+        expected_headers["X-Other-Header"] = 'another_header_t...'
+        obscured_test(params, headers, [], ['X-Other-Header'], params,
+                      expected_headers)
+        # both
+        expected_headers["X-Auth-Token"] = 'this_is_my_auth_...'
+        obscured_test(params, headers, [], ['X-Auth-Token', 'X-Other-Header'],
+                      params, expected_headers)
+
+        # all together
+        obscured_test(params, headers, ['param_two', 'param_one'],
+                      ['X-Auth-Token', 'X-Other-Header'],
+                      expected_params, expected_headers)
