@@ -15,10 +15,8 @@
 # limitations under the License.
 
 """Tests for swift.common.utils"""
-
 from collections import defaultdict
 import errno
-from hashlib import md5
 import io
 import logging
 import six
@@ -34,8 +32,10 @@ from eventlet import GreenPool, sleep, Queue
 from eventlet.pools import Pool
 
 from swift.common import memcached
+from swift.common.memcached import MemcacheConnectionError
+from swift.common.utils import md5, human_readable
 from mock import patch, MagicMock
-from test.unit import debug_logger
+from test.debug_logger import debug_logger
 
 
 class MockedMemcachePool(memcached.MemcacheConnPool):
@@ -50,24 +50,33 @@ class MockedMemcachePool(memcached.MemcacheConnPool):
 
 
 class ExplodingMockMemcached(object):
+    should_explode = True
     exploded = False
 
     def sendall(self, string):
-        self.exploded = True
-        raise socket.error(errno.EPIPE, os.strerror(errno.EPIPE))
+        if self.should_explode:
+            self.exploded = True
+            raise socket.error(errno.EPIPE, os.strerror(errno.EPIPE))
 
     def readline(self):
-        self.exploded = True
-        raise socket.error(errno.EPIPE, os.strerror(errno.EPIPE))
+        if self.should_explode:
+            self.exploded = True
+            raise socket.error(errno.EPIPE, os.strerror(errno.EPIPE))
+        return b'STORED\r\n'
 
     def read(self, size):
-        self.exploded = True
-        raise socket.error(errno.EPIPE, os.strerror(errno.EPIPE))
+        if self.should_explode:
+            self.exploded = True
+            raise socket.error(errno.EPIPE, os.strerror(errno.EPIPE))
 
     def close(self):
         pass
 
 
+TOO_BIG_KEY = md5(
+    b'too-big', usedforsecurity=False).hexdigest().encode('ascii')
+
+
 class MockMemcached(object):
     # See https://github.com/memcached/memcached/blob/master/doc/protocol.txt
     # In particular, the "Storage commands" section may be interesting.
@@ -100,7 +109,10 @@ class MockMemcached(object):
         self.cache[key] = flags, exptime, self.inbuf[:int(num_bytes)]
         self.inbuf = self.inbuf[int(num_bytes) + 2:]
         if noreply != b'noreply':
-            self.outbuf += b'STORED\r\n'
+            if key == TOO_BIG_KEY:
+                self.outbuf += b'SERVER_ERROR object too large for cache\r\n'
+            else:
+                self.outbuf += b'STORED\r\n'
 
     def handle_add(self, key, flags, exptime, num_bytes, noreply=b''):
         value = self.inbuf[:int(num_bytes)]
@@ -194,6 +206,20 @@ class TestMemcached(unittest.TestCase):
         client = memcached.MemcacheRing([server_socket], logger=self.logger)
         self.assertIs(client.logger, self.logger)
 
+    def test_tls_context_kwarg(self):
+        with patch('swift.common.memcached.socket.socket'):
+            server = '%s:%s' % ('[::1]', 11211)
+            client = memcached.MemcacheRing([server])
+            self.assertIsNone(client._client_cache[server]._tls_context)
+
+            context = mock.Mock()
+            client = memcached.MemcacheRing([server], tls_context=context)
+            self.assertIs(client._client_cache[server]._tls_context, context)
+
+            key = uuid4().hex.encode('ascii')
+            list(client._get_conns(key))
+            context.wrap_socket.assert_called_once()
+
     def test_get_conns(self):
         sock1 = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         sock1.bind(('127.0.0.1', 0))
@@ -333,7 +359,8 @@ class TestMemcached(unittest.TestCase):
         mock = MockMemcached()
         memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
             [(mock, mock)] * 2)
-        cache_key = md5(b'some_key').hexdigest().encode('ascii')
+        cache_key = md5(b'some_key',
+                        usedforsecurity=False).hexdigest().encode('ascii')
 
         memcache_client.set('some_key', [1, 2, 3])
         self.assertEqual(memcache_client.get('some_key'), [1, 2, 3])
@@ -361,6 +388,18 @@ class TestMemcached(unittest.TestCase):
         _junk, cache_timeout, _junk = mock.cache[cache_key]
         self.assertAlmostEqual(float(cache_timeout), esttimeout, delta=1)
 
+    def test_set_error(self):
+        memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'],
+                                                 logger=self.logger)
+        mock = MockMemcached()
+        memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
+            [(mock, mock)] * 2)
+        memcache_client.set('too-big', [1, 2, 3])
+        self.assertEqual(
+            self.logger.get_lines_for_level('error'),
+            ['Error setting value in memcached: 1.2.3.4:11211: '
+             'SERVER_ERROR object too large for cache'])
+
     def test_get_failed_connection_mid_request(self):
         memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'],
                                                  logger=self.logger)
@@ -439,7 +478,8 @@ class TestMemcached(unittest.TestCase):
         mock = MockMemcached()
         memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
             [(mock, mock)] * 2)
-        cache_key = md5(b'some_key').hexdigest().encode('ascii')
+        cache_key = md5(b'some_key',
+                        usedforsecurity=False).hexdigest().encode('ascii')
 
         memcache_client.incr('some_key', delta=5, time=55)
         self.assertEqual(memcache_client.get('some_key'), b'5')
@@ -512,6 +552,174 @@ class TestMemcached(unittest.TestCase):
         self.assertEqual(memcache_client._client_cache['1.2.3.5:11211'].mocks,
                          [])
 
+    def test_error_limiting(self):
+        memcache_client = memcached.MemcacheRing(
+            ['1.2.3.4:11211', '1.2.3.5:11211'], logger=self.logger)
+        mock1 = ExplodingMockMemcached()
+        mock2 = ExplodingMockMemcached()
+        mock2.should_explode = False
+        memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
+            [(mock2, mock2)] * 12)
+        memcache_client._client_cache['1.2.3.5:11211'] = MockedMemcachePool(
+            [(mock1, mock1)] * 12)
+
+        for _ in range(12):
+            memcache_client.set('some_key', [1, 2, 3])
+        # twelfth one skips .5 because of error limiting and goes straight
+        # to .4
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.5:11211: '
+            '[Errno 32] Broken pipe',
+        ] * 11 + [
+            'Error limiting server 1.2.3.5:11211'
+        ])
+        self.logger.clear()
+
+        mock2.should_explode = True
+        for _ in range(12):
+            memcache_client.set('some_key', [1, 2, 3])
+        # as we keep going, eventually .4 gets error limited, too
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.4:11211: '
+            '[Errno 32] Broken pipe',
+        ] * 10 + [
+            'Error talking to memcached: 1.2.3.4:11211: '
+            '[Errno 32] Broken pipe',
+            'Error limiting server 1.2.3.4:11211',
+            'All memcached servers error-limited',
+        ])
+        self.logger.clear()
+
+        # continued requests just keep bypassing memcache
+        for _ in range(12):
+            memcache_client.set('some_key', [1, 2, 3])
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'All memcached servers error-limited',
+        ] * 12)
+        self.logger.clear()
+
+        # and get()s are all a "cache miss"
+        self.assertIsNone(memcache_client.get('some_key'))
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'All memcached servers error-limited',
+        ])
+
+    def test_error_disabled(self):
+        memcache_client = memcached.MemcacheRing(
+            ['1.2.3.4:11211'], logger=self.logger, error_limit_time=0)
+        mock1 = ExplodingMockMemcached()
+        memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
+            [(mock1, mock1)] * 20)
+
+        for _ in range(20):
+            memcache_client.set('some_key', [1, 2, 3])
+        # twelfth one skips .5 because of error limiting and goes straight
+        # to .4
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.4:11211: '
+            '[Errno 32] Broken pipe',
+        ] * 20)
+
+    def test_error_raising(self):
+        memcache_client = memcached.MemcacheRing(
+            ['1.2.3.4:11211'], logger=self.logger, error_limit_time=0)
+        mock1 = ExplodingMockMemcached()
+        memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
+            [(mock1, mock1)] * 20)
+
+        # expect exception when requested...
+        with self.assertRaises(MemcacheConnectionError):
+            memcache_client.set('some_key', [1, 2, 3], raise_on_error=True)
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.4:11211: '
+            '[Errno 32] Broken pipe',
+        ])
+        self.logger.clear()
+
+        with self.assertRaises(MemcacheConnectionError):
+            memcache_client.get('some_key', raise_on_error=True)
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.4:11211: '
+            '[Errno 32] Broken pipe',
+        ])
+        self.logger.clear()
+
+        # ...but default is no exception
+        memcache_client.set('some_key', [1, 2, 3])
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.4:11211: '
+            '[Errno 32] Broken pipe',
+        ])
+        self.logger.clear()
+
+        memcache_client.get('some_key')
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.4:11211: '
+            '[Errno 32] Broken pipe',
+        ])
+
+    def test_error_limiting_custom_config(self):
+        def do_calls(time_step, num_calls, **memcache_kwargs):
+            self.logger.clear()
+            memcache_client = memcached.MemcacheRing(
+                ['1.2.3.5:11211'], logger=self.logger,
+                **memcache_kwargs)
+            mock1 = ExplodingMockMemcached()
+            memcache_client._client_cache['1.2.3.5:11211'] = \
+                MockedMemcachePool([(mock1, mock1)] * num_calls)
+
+            for n in range(num_calls):
+                with mock.patch.object(memcached.time, 'time',
+                                       return_value=time_step * n):
+                    memcache_client.set('some_key', [1, 2, 3])
+
+        # with default error_limit_time of 60, one call per 5 secs, twelfth one
+        # triggers error limit
+        do_calls(5, 12)
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.5:11211: '
+            '[Errno 32] Broken pipe',
+        ] * 10 + [
+            'Error talking to memcached: 1.2.3.5:11211: '
+            '[Errno 32] Broken pipe',
+            'Error limiting server 1.2.3.5:11211',
+            'All memcached servers error-limited',
+        ])
+
+        # with default error_limit_time of 60, one call per 6 secs, error limit
+        # is not triggered
+        do_calls(6, 20)
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.5:11211: '
+            '[Errno 32] Broken pipe',
+        ] * 20)
+
+        # with error_limit_time of 66, one call per 6 secs, twelfth one
+        # triggers error limit
+        do_calls(6, 12, error_limit_time=66)
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.5:11211: '
+            '[Errno 32] Broken pipe',
+        ] * 10 + [
+            'Error talking to memcached: 1.2.3.5:11211: '
+            '[Errno 32] Broken pipe',
+            'Error limiting server 1.2.3.5:11211',
+            'All memcached servers error-limited',
+        ])
+
+        # with error_limit_time of 70, one call per 6 secs, error_limit_count
+        # of 11, 13th call triggers error limit
+        do_calls(6, 13, error_limit_time=70, error_limit_count=11)
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.5:11211: '
+            '[Errno 32] Broken pipe',
+        ] * 11 + [
+            'Error talking to memcached: 1.2.3.5:11211: '
+            '[Errno 32] Broken pipe',
+            'Error limiting server 1.2.3.5:11211',
+            'All memcached servers error-limited',
+        ])
+
     def test_delete(self):
         memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'],
                                                  logger=self.logger)
@@ -536,7 +744,7 @@ class TestMemcached(unittest.TestCase):
             memcache_client.get_multi(('some_key2', 'some_key1'), 'multi_key'),
             [[4, 5, 6], [1, 2, 3]])
         for key in (b'some_key1', b'some_key2'):
-            key = md5(key).hexdigest().encode('ascii')
+            key = md5(key, usedforsecurity=False).hexdigest().encode('ascii')
             self.assertIn(key, mock.cache)
             _junk, cache_timeout, _junk = mock.cache[key]
             self.assertEqual(cache_timeout, b'0')
@@ -545,7 +753,7 @@ class TestMemcached(unittest.TestCase):
             {'some_key1': [1, 2, 3], 'some_key2': [4, 5, 6]}, 'multi_key',
             time=20)
         for key in (b'some_key1', b'some_key2'):
-            key = md5(key).hexdigest().encode('ascii')
+            key = md5(key, usedforsecurity=False).hexdigest().encode('ascii')
             _junk, cache_timeout, _junk = mock.cache[key]
             self.assertEqual(cache_timeout, b'20')
 
@@ -555,7 +763,7 @@ class TestMemcached(unittest.TestCase):
             {'some_key1': [1, 2, 3], 'some_key2': [4, 5, 6]}, 'multi_key',
             time=fortydays)
         for key in (b'some_key1', b'some_key2'):
-            key = md5(key).hexdigest().encode('ascii')
+            key = md5(key, usedforsecurity=False).hexdigest().encode('ascii')
             _junk, cache_timeout, _junk = mock.cache[key]
             self.assertAlmostEqual(float(cache_timeout), esttimeout, delta=1)
         self.assertEqual(memcache_client.get_multi(
@@ -592,14 +800,15 @@ class TestMemcached(unittest.TestCase):
             memcache_client.get_multi(('some_key1', 'some_key0'), 'multi_key'),
             [[4, 5, 6], [1, 2, 3]])
         for key in (b'some_key0', b'some_key1'):
-            key = md5(key).hexdigest().encode('ascii')
+            key = md5(key, usedforsecurity=False).hexdigest().encode('ascii')
             self.assertIn(key, mock1.cache)
             _junk, cache_timeout, _junk = mock1.cache[key]
             self.assertEqual(cache_timeout, b'0')
 
         memcache_client.set('some_key0', [7, 8, 9])
         self.assertEqual(memcache_client.get('some_key0'), [7, 8, 9])
-        key = md5(b'some_key0').hexdigest().encode('ascii')
+        key = md5(b'some_key0',
+                  usedforsecurity=False).hexdigest().encode('ascii')
         self.assertIn(key, mock2.cache)
 
         # Delete 'some_key0' with server_key='multi_key'
@@ -614,24 +823,18 @@ class TestMemcached(unittest.TestCase):
 
     def test_serialization(self):
         memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'],
-                                                 allow_pickle=True,
                                                  logger=self.logger)
         mock = MockMemcached()
         memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
             [(mock, mock)] * 2)
         memcache_client.set('some_key', [1, 2, 3])
         self.assertEqual(memcache_client.get('some_key'), [1, 2, 3])
-        memcache_client._allow_pickle = False
-        memcache_client._allow_unpickle = True
-        self.assertEqual(memcache_client.get('some_key'), [1, 2, 3])
-        memcache_client._allow_unpickle = False
+        self.assertEqual(len(mock.cache), 1)
+        key = next(iter(mock.cache))
+        self.assertEqual(mock.cache[key][0], b'2')  # JSON_FLAG
+        # Pretend we've got some really old pickle data in there
+        mock.cache[key] = (b'1',) + mock.cache[key][1:]
         self.assertIsNone(memcache_client.get('some_key'))
-        memcache_client.set('some_key', [1, 2, 3])
-        self.assertEqual(memcache_client.get('some_key'), [1, 2, 3])
-        memcache_client._allow_unpickle = True
-        self.assertEqual(memcache_client.get('some_key'), [1, 2, 3])
-        memcache_client._allow_pickle = True
-        self.assertEqual(memcache_client.get('some_key'), [1, 2, 3])
 
     def test_connection_pooling(self):
         with patch('swift.common.memcached.socket') as mock_module:
@@ -727,11 +930,13 @@ class TestMemcached(unittest.TestCase):
             # fast. All ten (10) clients should try to talk to .5 first, and
             # then move on to .4, and we'll assert all that below.
             mock_conn = MagicMock(), MagicMock()
+            mock_conn[0].readline = lambda: b'STORED\r\n'
             mock_conn[1].sendall = lambda x: sleep(0.2)
             connections['1.2.3.5'].put(mock_conn)
             connections['1.2.3.5'].put(mock_conn)
 
             mock_conn = MagicMock(), MagicMock()
+            mock_conn[0].readline = lambda: b'STORED\r\n'
             connections['1.2.3.4'].put(mock_conn)
             connections['1.2.3.4'].put(mock_conn)
 
@@ -756,6 +961,91 @@ class TestMemcached(unittest.TestCase):
         self.assertEqual(connections['1.2.3.5'].qsize(), 2)
         self.assertEqual(connections['1.2.3.4'].qsize(), 2)
 
+    def test_connection_slow_connect(self):
+        with patch('swift.common.memcached.socket') as mock_module:
+            def mock_getaddrinfo(host, port, family=socket.AF_INET,
+                                 socktype=socket.SOCK_STREAM, proto=0,
+                                 flags=0):
+                return [(family, socktype, proto, '', (host, port))]
+
+            mock_module.getaddrinfo = mock_getaddrinfo
+
+            # patch socket, stub socket.socket, mock sock
+            mock_sock = mock_module.socket.return_value
+
+            def wait_connect(addr):
+                # slow connect gives Timeout Exception
+                sleep(1)
+
+            # patch connect method
+            mock_sock.connect = wait_connect
+
+            memcache_client = memcached.MemcacheRing(
+                ['1.2.3.4:11211'], connect_timeout=0.1)
+
+            # sanity
+            self.assertEqual(1, len(memcache_client._client_cache))
+            for server, pool in memcache_client._client_cache.items():
+                self.assertEqual(2, pool.max_size)
+
+            # try to get connect and no connection found
+            # so it will result in StopIteration
+            conn_generator = memcache_client._get_conns(b'key')
+            with self.assertRaises(StopIteration):
+                next(conn_generator)
+
+            self.assertEqual(1, mock_sock.close.call_count)
+
+    def test_item_size_warning_threshold(self):
+        mock = MockMemcached()
+        mocked_pool = MockedMemcachePool([(mock, mock)] * 2)
+
+        def do_test(d, threshold, should_warn, error=False):
+            self.logger.clear()
+            try:
+                memcache_client = memcached.MemcacheRing(
+                    ['1.2.3.4:11211'], item_size_warning_threshold=threshold,
+                    logger=self.logger)
+                memcache_client._client_cache['1.2.3.4:11211'] = mocked_pool
+                memcache_client.set('some_key', d, serialize=False)
+                warning_lines = self.logger.get_lines_for_level('warning')
+                if should_warn:
+                    self.assertIn(
+                        'Item size larger than warning threshold: '
+                        '%d (%s) >= %d (%s)' % (
+                            len(d), human_readable(len(d)), threshold,
+                            human_readable(threshold)),
+                        warning_lines[0])
+                else:
+                    self.assertFalse(warning_lines)
+            except ValueError as err:
+                if not err:
+                    self.fail(err)
+                else:
+                    self.assertIn(
+                        'Config option must be a number, greater than 0, '
+                        'less than 100, not "%s".' % threshold,
+                        str(err))
+
+        data = '1' * 100
+        # let's start with something easy, say warning at 80
+        for data_size, warn in ((79, False), (80, True), (81, True),
+                                (99, True), (100, True)):
+            do_test(data[:data_size], 80, warn)
+
+        # if we set the threshold to -1 will turn off the warning
+        for data_size, warn in ((79, False), (80, False), (81, False),
+                                (99, False), (100, False)):
+            do_test(data[:data_size], -1, warn)
+
+        # Changing to 0 should warn on everything
+        for data_size, warn in ((0, True), (1, True), (50, True),
+                                (99, True), (100, True)):
+            do_test(data[:data_size], 0, warn)
+
+        # Let's do a big number
+        do_test('1' * 2048576, 1000000, True)
+
 
 if __name__ == '__main__':
     unittest.main()
