@@ -12,33 +12,37 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-import json
 
 import unittest
+import json
 import mock
 import os
-import sys
+import pkg_resources
 import signal
-import time
 import string
+import sys
+import time
 import xattr
 from shutil import rmtree
-from hashlib import md5
 from tempfile import mkdtemp
 import textwrap
 from os.path import dirname, basename
-from test.unit import (debug_logger, patch_policies, make_timestamp_iter,
-                       DEFAULT_TEST_EC_TYPE, skip_if_no_xattrs)
+from test.debug_logger import debug_logger
+from test.unit import (
+    DEFAULT_TEST_EC_TYPE, make_timestamp_iter, patch_policies,
+    skip_if_no_xattrs)
+from test.unit.obj.common import write_diskfile
 from swift.obj import auditor, replicator
+from swift.obj.watchers.dark_data import DarkDataWatcher
 from swift.obj.diskfile import (
     DiskFile, write_metadata, invalidate_hash, get_data_dir,
     DiskFileManager, ECDiskFileManager, AuditLocation, clear_auditor_status,
     get_auditor_status, HASH_FILE, HASH_INVALIDATIONS_FILE)
+from swift.common.exceptions import ClientException
 from swift.common.utils import (
-    mkdirs, normalize_timestamp, Timestamp, readconf)
+    mkdirs, normalize_timestamp, Timestamp, readconf, md5, PrefixLoggerAdapter)
 from swift.common.storage_policy import (
     ECStoragePolicy, StoragePolicy, POLICIES, EC_POLICY)
-from test.unit.obj.common import write_diskfile
 
 _mocked_policies = [
     StoragePolicy(0, 'zero', False),
@@ -61,8 +65,53 @@ def works_only_once(callable_thing, exception):
     return only_once
 
 
-@patch_policies(_mocked_policies)
-class TestAuditor(unittest.TestCase):
+def no_audit_watchers(group, name=None):
+    if group == 'swift.object_audit_watcher':
+        return iter([])
+    else:
+        return pkg_resources.iter_entry_points(group, name)
+
+
+class FakeRing1(object):
+
+    def __init__(self, swift_dir, ring_name=None):
+        return
+
+    def get_nodes(self, *args, **kwargs):
+        x = 1
+        node1 = {'ip': '10.0.0.%s' % x,
+                 'replication_ip': '10.0.0.%s' % x,
+                 'port': 6200 + x,
+                 'replication_port': 6200 + x,
+                 'device': 'sda',
+                 'zone': x % 3,
+                 'region': x % 2,
+                 'id': x,
+                 'handoff_index': 1}
+        return (1, [node1])
+
+
+class FakeRing2(object):
+
+    def __init__(self, swift_dir, ring_name=None):
+        return
+
+    def get_nodes(self, *args, **kwargs):
+        nodes = []
+        for x in [1, 2]:
+            nodes.append({'ip': '10.0.0.%s' % x,
+                          'replication_ip': '10.0.0.%s' % x,
+                          'port': 6200 + x,
+                          'replication_port': 6200 + x,
+                          'device': 'sda',
+                          'zone': x % 3,
+                          'region': x % 2,
+                          'id': x,
+                          'handoff_index': 1})
+        return (1, nodes)
+
+
+class TestAuditorBase(unittest.TestCase):
 
     def setUp(self):
         skip_if_no_xattrs()
@@ -114,14 +163,18 @@ class TestAuditor(unittest.TestCase):
         # diskfiles for policy 0, 1, 2
         self.disk_file = self.df_mgr.get_diskfile('sda', '0', 'a', 'c', 'o',
                                                   policy=POLICIES[0])
-        self.disk_file_p1 = self.df_mgr.get_diskfile('sda', '0', 'a', 'c',
+        self.disk_file_p1 = self.df_mgr.get_diskfile('sda', '0', 'a', 'c2',
                                                      'o', policy=POLICIES[1])
         self.disk_file_ec = self.ec_df_mgr.get_diskfile(
-            'sda', '0', 'a', 'c', 'o', policy=POLICIES[2], frag_index=1)
+            'sda', '0', 'a', 'c_ec', 'o', policy=POLICIES[2], frag_index=1)
 
     def tearDown(self):
         rmtree(os.path.dirname(self.testdir), ignore_errors=1)
 
+
+@patch_policies(_mocked_policies)
+class TestAuditor(TestAuditorBase):
+
     def test_worker_conf_parms(self):
         def check_common_defaults():
             self.assertEqual(auditor_worker.max_bytes_per_second, 10000000)
@@ -160,7 +213,7 @@ class TestAuditor(unittest.TestCase):
             data = b'0' * 1024
             if disk_file.policy.policy_type == EC_POLICY:
                 data = disk_file.policy.pyeclib_driver.encode(data)[0]
-            etag = md5()
+            etag = md5(usedforsecurity=False)
             with disk_file.create() as writer:
                 writer.write(data)
                 etag.update(data)
@@ -198,7 +251,7 @@ class TestAuditor(unittest.TestCase):
         # simulate a PUT
         now = time.time()
         data = b'boots and cats and ' * 1024
-        hasher = md5()
+        hasher = md5(usedforsecurity=False)
         with disk_file.create() as writer:
             writer.write(data)
             hasher.update(data)
@@ -257,14 +310,16 @@ class TestAuditor(unittest.TestCase):
             checksum = xattr.getxattr(
                 file_path, "user.swift.metadata_checksum")
 
-            self.assertEqual(checksum,
-                             md5(metadata).hexdigest().encode('ascii'))
+            self.assertEqual(
+                checksum,
+                (md5(metadata, usedforsecurity=False).hexdigest()
+                 .encode('ascii')))
 
     def test_object_audit_diff_data(self):
         auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
                                                self.rcache, self.devices)
         data = b'0' * 1024
-        etag = md5()
+        etag = md5(usedforsecurity=False)
         timestamp = str(normalize_timestamp(time.time()))
         with self.disk_file.create() as writer:
             writer.write(data)
@@ -287,7 +342,7 @@ class TestAuditor(unittest.TestCase):
             AuditLocation(self.disk_file._datadir, 'sda', '0',
                           policy=POLICIES.legacy))
         self.assertEqual(auditor_worker.quarantines, pre_quarantines)
-        etag = md5(b'1' + b'0' * 1023).hexdigest()
+        etag = md5(b'1' + b'0' * 1023, usedforsecurity=False).hexdigest()
         metadata['ETag'] = etag
 
         with self.disk_file.create() as writer:
@@ -305,7 +360,7 @@ class TestAuditor(unittest.TestCase):
 
         def do_test(data):
             # create diskfile and set ETag and content-length to match the data
-            etag = md5(data).hexdigest()
+            etag = md5(data, usedforsecurity=False).hexdigest()
             timestamp = str(normalize_timestamp(time.time()))
             with disk_file.create() as writer:
                 writer.write(data)
@@ -515,7 +570,7 @@ class TestAuditor(unittest.TestCase):
                                  policy=self.disk_file.policy)
 
         data = b'VERIFY'
-        etag = md5()
+        etag = md5(usedforsecurity=False)
         timestamp = str(normalize_timestamp(time.time()))
         with self.disk_file.create() as writer:
             writer.write(data)
@@ -593,7 +648,7 @@ class TestAuditor(unittest.TestCase):
         timestamp = str(normalize_timestamp(time.time()))
         pre_errors = auditor_worker.errors
         data = b'0' * 1024
-        etag = md5()
+        etag = md5(usedforsecurity=False)
         with self.disk_file.create() as writer:
             writer.write(data)
             etag.update(data)
@@ -622,7 +677,7 @@ class TestAuditor(unittest.TestCase):
             with df.create() as writer:
                 writer.write(data)
                 metadata = {
-                    'ETag': md5(data).hexdigest(),
+                    'ETag': md5(data, usedforsecurity=False).hexdigest(),
                     'X-Timestamp': timestamp,
                     'Content-Length': str(os.fstat(writer._fd).st_size),
                 }
@@ -648,7 +703,7 @@ class TestAuditor(unittest.TestCase):
             with df.create() as writer:
                 writer.write(data)
                 metadata = {
-                    'ETag': md5(data).hexdigest(),
+                    'ETag': md5(data, usedforsecurity=False).hexdigest(),
                     'X-Timestamp': timestamp,
                     'Content-Length': str(os.fstat(writer._fd).st_size),
                 }
@@ -696,7 +751,7 @@ class TestAuditor(unittest.TestCase):
         with self.disk_file.create() as writer:
             writer.write(data)
             metadata = {
-                'ETag': md5(data).hexdigest(),
+                'ETag': md5(data, usedforsecurity=False).hexdigest(),
                 'X-Timestamp': ts.normal,
                 'Content-Length': str(os.fstat(writer._fd).st_size),
             }
@@ -768,7 +823,7 @@ class TestAuditor(unittest.TestCase):
         # pretend that we logged (and reset counters) just now
         auditor_worker.last_logged = time.time()
         data = b'0' * 1024
-        etag = md5()
+        etag = md5(usedforsecurity=False)
         with self.disk_file.create() as writer:
             writer.write(data)
             etag.update(data)
@@ -792,7 +847,7 @@ class TestAuditor(unittest.TestCase):
         timestamp = str(normalize_timestamp(time.time()))
         pre_quarantines = auditor_worker.quarantines
         data = b'0' * 10
-        etag = md5()
+        etag = md5(usedforsecurity=False)
         with self.disk_file.create() as writer:
             writer.write(data)
             etag.update(data)
@@ -808,7 +863,7 @@ class TestAuditor(unittest.TestCase):
         self.disk_file = self.df_mgr.get_diskfile('sda', '0', 'a', 'c', 'ob',
                                                   policy=POLICIES.legacy)
         data = b'1' * 10
-        etag = md5()
+        etag = md5(usedforsecurity=False)
         with self.disk_file.create() as writer:
             writer.write(data)
             etag.update(data)
@@ -828,7 +883,7 @@ class TestAuditor(unittest.TestCase):
         self.auditor = auditor.ObjectAuditor(self.conf)
         self.auditor.log_time = 0
         data = b'0' * 1024
-        etag = md5()
+        etag = md5(usedforsecurity=False)
         with self.disk_file.create() as writer:
             writer.write(data)
             etag.update(data)
@@ -841,7 +896,7 @@ class TestAuditor(unittest.TestCase):
             }
             writer.put(metadata)
             writer.commit(Timestamp(timestamp))
-            etag = md5()
+            etag = md5(usedforsecurity=False)
             etag.update(b'1' + b'0' * 1023)
             etag = etag.hexdigest()
             metadata['ETag'] = etag
@@ -863,7 +918,7 @@ class TestAuditor(unittest.TestCase):
             timestamp = Timestamp.now()
         self.auditor = auditor.ObjectAuditor(self.conf)
         self.auditor.log_time = 0
-        etag = md5()
+        etag = md5(usedforsecurity=False)
         with self.disk_file.create() as writer:
             etag = etag.hexdigest()
             metadata = {
@@ -873,7 +928,7 @@ class TestAuditor(unittest.TestCase):
             }
             writer.put(metadata)
             writer.commit(Timestamp(timestamp))
-            etag = md5()
+            etag = md5(usedforsecurity=False)
             etag = etag.hexdigest()
             metadata['ETag'] = etag
             write_metadata(writer._fd, metadata)
@@ -1424,7 +1479,7 @@ class TestAuditor(unittest.TestCase):
         ts = Timestamp(time.time())
         with self.disk_file.create() as writer:
             metadata = {
-                'ETag': md5(b'').hexdigest(),
+                'ETag': md5(b'', usedforsecurity=False).hexdigest(),
                 'X-Timestamp': ts.normal,
                 'Content-Length': str(os.fstat(writer._fd).st_size),
             }
@@ -1531,5 +1586,354 @@ class TestAuditor(unittest.TestCase):
                          .format(outstanding_pids))
 
 
+@mock.patch('pkg_resources.iter_entry_points', no_audit_watchers)
+@patch_policies(_mocked_policies)
+class TestAuditWatchers(TestAuditorBase):
+
+    def setUp(self):
+        super(TestAuditWatchers, self).setUp()
+
+        timestamp = Timestamp(time.time())
+
+        disk_file = self.df_mgr.get_diskfile(
+            'sda', '0', 'a', 'c', 'o0', policy=POLICIES.legacy)
+        data = b'0' * 1024
+        etag = md5()
+        with disk_file.create() as writer:
+            writer.write(data)
+            etag.update(data)
+            metadata = {
+                'ETag': etag.hexdigest(),
+                'X-Timestamp': timestamp.internal,
+                'Content-Length': str(len(data)),
+                'X-Object-Meta-Flavor': 'banana',
+            }
+            writer.put(metadata)
+            # The commit does nothing; we keep it for code copy-paste with EC.
+            writer.commit(timestamp)
+
+        disk_file = self.df_mgr.get_diskfile(
+            'sda', '0', 'a', 'c', 'o1', policy=POLICIES.legacy)
+        data = b'1' * 2048
+        etag = md5()
+        with disk_file.create() as writer:
+            writer.write(data)
+            etag.update(data)
+            metadata = {
+                'ETag': etag.hexdigest(),
+                'X-Timestamp': timestamp.internal,
+                'Content-Length': str(len(data)),
+                'X-Object-Meta-Flavor': 'orange',
+            }
+            writer.put(metadata)
+            writer.commit(timestamp)
+
+        frag_0 = self.disk_file_ec.policy.pyeclib_driver.encode(
+            b'x' * self.disk_file_ec.policy.ec_segment_size)[0]
+        etag = md5()
+        with self.disk_file_ec.create() as writer:
+            writer.write(frag_0)
+            etag.update(frag_0)
+            metadata = {
+                'ETag': etag.hexdigest(),
+                'X-Timestamp': timestamp.internal,
+                'Content-Length': str(len(frag_0)),
+                'X-Object-Meta-Flavor': 'peach',
+            }
+            writer.put(metadata)
+            writer.commit(timestamp)
+
+    def test_watchers(self):
+
+        calls = []
+
+        class TestWatcher(object):
+            def __init__(self, conf, logger):
+                self._started = False
+                self._ended = False
+                calls.append(["__init__", conf, logger])
+
+                # Make sure the logger is capable of quacking like a logger
+                logger.debug("getting started")
+
+            def start(self, audit_type, **other_kwargs):
+                if self._started:
+                    raise Exception("don't call it twice")
+                self._started = True
+                calls.append(['start', audit_type])
+
+            def see_object(self, object_metadata,
+                           data_file_path, **other_kwargs):
+                calls.append(['see_object', object_metadata,
+                              data_file_path, other_kwargs])
+
+            def end(self, **other_kwargs):
+                if self._ended:
+                    raise Exception("don't call it twice")
+                self._ended = True
+                calls.append(['end'])
+
+        conf = self.conf.copy()
+        conf['watchers'] = 'test_watcher1'
+        conf['__file__'] = '/etc/swift/swift.conf'
+        ret_config = {'swift#dark_data': {'action': 'log'}}
+        with mock.patch('swift.obj.auditor.parse_prefixed_conf',
+                        return_value=ret_config), \
+                mock.patch('swift.obj.auditor.load_pkg_resource',
+                           side_effect=[TestWatcher]) as mock_load, \
+                mock.patch('swift.obj.auditor.get_logger',
+                           lambda *a, **kw: self.logger):
+            my_auditor = auditor.ObjectAuditor(conf)
+
+        self.assertEqual(mock_load.mock_calls, [
+            mock.call('swift.object_audit_watcher', 'test_watcher1'),
+        ])
+
+        my_auditor.run_audit(mode='once', zero_byte_fps=float("inf"))
+
+        self.assertEqual(len(calls), 6)
+
+        self.assertEqual(calls[0], ["__init__", conf, mock.ANY])
+        self.assertIsInstance(calls[0][2], PrefixLoggerAdapter)
+        self.assertIs(calls[0][2].logger, self.logger)
+
+        self.assertEqual(calls[1], ["start", "ZBF"])
+
+        self.assertEqual(calls[2][0], "see_object")
+        self.assertEqual(calls[3][0], "see_object")
+
+        # The order in which the auditor finds things on the filesystem is
+        # irrelevant; what matters is that it finds all the things.
+        calls[2:5] = sorted(calls[2:5], key=lambda item: item[1]['name'])
+
+        self.assertDictContainsSubset({'name': '/a/c/o0',
+                                       'X-Object-Meta-Flavor': 'banana'},
+                                      calls[2][1])
+        self.assertIn('node/sda/objects/0/', calls[2][2])  # data_file_path
+        self.assertTrue(calls[2][2].endswith('.data'))  # data_file_path
+        self.assertEqual({}, calls[2][3])
+
+        self.assertDictContainsSubset({'name': '/a/c/o1',
+                                       'X-Object-Meta-Flavor': 'orange'},
+                                      calls[3][1])
+        self.assertIn('node/sda/objects/0/', calls[3][2])  # data_file_path
+        self.assertTrue(calls[3][2].endswith('.data'))  # data_file_path
+        self.assertEqual({}, calls[3][3])
+
+        self.assertDictContainsSubset({'name': '/a/c_ec/o',
+                                       'X-Object-Meta-Flavor': 'peach'},
+                                      calls[4][1])
+        self.assertIn('node/sda/objects-2/0/', calls[4][2])  # data_file_path
+        self.assertTrue(calls[4][2].endswith('.data'))  # data_file_path
+        self.assertEqual({}, calls[4][3])
+
+        self.assertEqual(calls[5], ["end"])
+
+        log_lines = self.logger.get_lines_for_level('debug')
+        self.assertIn(
+            "[audit-watcher test_watcher1] getting started",
+            log_lines)
+
+    def test_builtin_watchers(self):
+
+        # Yep, back-channel signaling in tests.
+        sentinel = 'DARK'
+
+        timestamp = Timestamp(time.time())
+
+        disk_file = self.df_mgr.get_diskfile(
+            'sda', '0', 'a', sentinel, 'o2', policy=POLICIES.legacy)
+        data = b'2' * 1024
+        etag = md5()
+        with disk_file.create() as writer:
+            writer.write(data)
+            etag.update(data)
+            metadata = {
+                'ETag': etag.hexdigest(),
+                'X-Timestamp': timestamp.internal,
+                'Content-Length': str(len(data)),
+                'X-Object-Meta-Flavor': 'mango',
+            }
+            writer.put(metadata)
+            writer.commit(timestamp)
+
+        def fake_direct_get_container(node, part, account, container,
+                                      prefix=None, limit=None):
+            self.assertEqual(part, 1)
+            self.assertEqual(limit, 1)
+
+            if container == sentinel:
+                return {}, []
+
+            # The returned entry is not abbreviated, but is full of nonsese.
+            entry = {'bytes': 30968411,
+                     'hash': '60303f4122966fe5925f045eb52d1129',
+                     'name': '%s' % prefix,
+                     'content_type': 'video/mp4',
+                     'last_modified': '2017-08-15T03:30:57.693210'}
+            return {}, [entry]
+
+        conf = self.conf.copy()
+        conf['watchers'] = 'test_watcher1'
+        conf['__file__'] = '/etc/swift/swift.conf'
+
+        # with default watcher config the DARK object will not be older than
+        # grace_age so will not be logged
+        ret_config = {'test_watcher1': {'action': 'log'}}
+        with mock.patch('swift.obj.auditor.parse_prefixed_conf',
+                        return_value=ret_config), \
+                mock.patch('swift.obj.auditor.load_pkg_resource',
+                           side_effect=[DarkDataWatcher]):
+            my_auditor = auditor.ObjectAuditor(conf, logger=self.logger)
+
+        with mock.patch('swift.obj.watchers.dark_data.Ring', FakeRing1), \
+                mock.patch("swift.obj.watchers.dark_data.direct_get_container",
+                           fake_direct_get_container):
+            my_auditor.run_audit(mode='once')
+
+        log_lines = self.logger.get_lines_for_level('info')
+        self.assertIn(
+            '[audit-watcher test_watcher1] total unknown 0 ok 4 dark 0',
+            log_lines)
+
+        self.logger.clear()
+
+        # with grace_age=0 the DARK object will be older than
+        # grace_age so will be logged
+        ret_config = {'test_watcher1': {'action': 'log', 'grace_age': '0'}}
+        with mock.patch('swift.obj.auditor.parse_prefixed_conf',
+                        return_value=ret_config), \
+                mock.patch('swift.obj.auditor.load_pkg_resource',
+                           side_effect=[DarkDataWatcher]):
+            my_auditor = auditor.ObjectAuditor(conf, logger=self.logger)
+
+        with mock.patch('swift.obj.watchers.dark_data.Ring', FakeRing1), \
+                mock.patch("swift.obj.watchers.dark_data.direct_get_container",
+                           fake_direct_get_container):
+            my_auditor.run_audit(mode='once')
+
+        log_lines = self.logger.get_lines_for_level('info')
+        self.assertIn(
+            '[audit-watcher test_watcher1] total unknown 0 ok 3 dark 1',
+            log_lines)
+
+    def test_dark_data_watcher_init(self):
+        conf = {}
+        with mock.patch('swift.obj.watchers.dark_data.Ring', FakeRing1):
+            watcher = DarkDataWatcher(conf, self.logger)
+        self.assertEqual(self.logger, watcher.logger)
+        self.assertEqual(604800, watcher.grace_age)
+        self.assertEqual('log', watcher.dark_data_policy)
+
+        conf = {'grace_age': 360, 'action': 'delete'}
+        with mock.patch('swift.obj.watchers.dark_data.Ring', FakeRing1):
+            watcher = DarkDataWatcher(conf, self.logger)
+        self.assertEqual(self.logger, watcher.logger)
+        self.assertEqual(360, watcher.grace_age)
+        self.assertEqual('delete', watcher.dark_data_policy)
+
+        conf = {'grace_age': 0, 'action': 'invalid'}
+        with mock.patch('swift.obj.watchers.dark_data.Ring', FakeRing1):
+            watcher = DarkDataWatcher(conf, self.logger)
+        self.assertEqual(self.logger, watcher.logger)
+        self.assertEqual(0, watcher.grace_age)
+        self.assertEqual('log', watcher.dark_data_policy)
+
+    def test_dark_data_agreement(self):
+
+        # The dark data watcher only sees an object as dark if all container
+        # servers in the ring reply without an error and return an empty
+        # listing. So, we have the following permutations for an object:
+        #
+        #      Container Servers         Result
+        #      CS1         CS2
+        #      Listed      Listed        Good - the baseline result
+        #      Listed      Error         Good
+        #      Listed      Not listed    Good
+        #      Error       Error         Unknown - the baseline failure
+        #      Not listed  Error         Unknown
+        #      Not listed  Not listed    Dark - the only such result!
+        #
+        scenario = [
+            {'cr': ['L', 'L'], 'res': 'G'},
+            {'cr': ['L', 'E'], 'res': 'G'},
+            {'cr': ['L', 'N'], 'res': 'G'},
+            {'cr': ['E', 'E'], 'res': 'U'},
+            {'cr': ['N', 'E'], 'res': 'U'},
+            {'cr': ['N', 'N'], 'res': 'D'}]
+
+        conf = self.conf.copy()
+        conf['watchers'] = 'test_watcher1'
+        conf['__file__'] = '/etc/swift/swift.conf'
+        ret_config = {'test_watcher1': {'action': 'log', 'grace_age': '0'}}
+        with mock.patch('swift.obj.auditor.parse_prefixed_conf',
+                        return_value=ret_config), \
+                mock.patch('swift.obj.auditor.load_pkg_resource',
+                           side_effect=[DarkDataWatcher]):
+            my_auditor = auditor.ObjectAuditor(conf, logger=self.logger)
+
+        for cur in scenario:
+
+            def fake_direct_get_container(node, part, account, container,
+                                          prefix=None, limit=None):
+                self.assertEqual(part, 1)
+                self.assertEqual(limit, 1)
+
+                reply_type = cur['cr'][int(node['id']) - 1]
+
+                if reply_type == 'E':
+                    raise ClientException("Emulated container server error")
+
+                if reply_type == 'N':
+                    return {}, []
+
+                entry = {'bytes': 30968411,
+                         'hash': '60303f4122966fe5925f045eb52d1129',
+                         'name': '%s' % prefix,
+                         'content_type': 'video/mp4',
+                         'last_modified': '2017-08-15T03:30:57.693210'}
+                return {}, [entry]
+
+            self.logger.clear()
+
+            namespace = 'swift.obj.watchers.dark_data.'
+            with mock.patch(namespace + 'Ring', FakeRing2), \
+                    mock.patch(namespace + 'direct_get_container',
+                               fake_direct_get_container):
+                my_auditor.run_audit(mode='once')
+
+            # We inherit a common setUp with 3 objects, so 3 everywhere.
+            if cur['res'] == 'U':
+                unk_exp, ok_exp, dark_exp = 3, 0, 0
+            elif cur['res'] == 'G':
+                unk_exp, ok_exp, dark_exp = 0, 3, 0
+            else:
+                unk_exp, ok_exp, dark_exp = 0, 0, 3
+
+            log_lines = self.logger.get_lines_for_level('info')
+            for line in log_lines:
+
+                if not line.startswith('[audit-watcher test_watcher1] total'):
+                    continue
+                words = line.split()
+                if not (words[3] == 'unknown' and
+                        words[5] == 'ok' and
+                        words[7] == 'dark'):
+                    unittest.TestCase.fail('Syntax error in %r' % (line,))
+
+                try:
+                    unk_cnt = int(words[4])
+                    ok_cnt = int(words[6])
+                    dark_cnt = int(words[8])
+                except ValueError:
+                    unittest.TestCase.fail('Bad value in %r' % (line,))
+
+            if unk_cnt != unk_exp or ok_cnt != ok_exp or dark_cnt != dark_exp:
+                fmt = 'Expected unknown %d ok %d dark %d, got %r, for nodes %r'
+                msg = fmt % (unk_exp, ok_exp, dark_exp,
+                             ' '.join(words[3:]), cur['cr'])
+                self.fail(msg=msg)
+
+
 if __name__ == '__main__':
     unittest.main()
