@@ -12,7 +12,7 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+import os
 from argparse import Namespace
 import itertools
 import json
@@ -27,7 +27,7 @@ from swift.proxy.controllers.base import headers_to_container_info, \
     headers_to_account_info, headers_to_object_info, get_container_info, \
     get_cache_key, get_account_info, get_info, get_object_info, \
     Controller, GetOrHeadHandler, bytes_to_skip, clear_info_cache, \
-    set_info_cache, NodeIter
+    set_info_cache, NodeIter, headers_from_container_info
 from swift.common.swob import Request, HTTPException, RESPONSE_REASONS, \
     bytes_to_wsgi
 from swift.common import exceptions
@@ -36,9 +36,10 @@ from swift.common.utils import split_path, ShardRange, Timestamp, \
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.http import is_success
 from swift.common.storage_policy import StoragePolicy, StoragePolicyCollection
+from test.debug_logger import debug_logger
 from test.unit import (
     fake_http_connect, FakeRing, FakeMemcache, PatchPolicies,
-    make_timestamp_iter, mocked_http_conn, patch_policies, debug_logger)
+    make_timestamp_iter, mocked_http_conn, patch_policies)
 from swift.common.request_helpers import (
     get_sys_meta_prefix, get_object_transient_sysmeta
 )
@@ -470,6 +471,16 @@ class TestFuncs(BaseTest):
                                        u"\U0001F334".encode('utf8')),
                          expected)
 
+        self.assertEqual(get_cache_key("account", "cont", shard="listing"),
+                         'shard-listing/account/cont')
+        self.assertEqual(get_cache_key("account", "cont", shard="updating"),
+                         'shard-updating/account/cont')
+        self.assertRaises(ValueError,
+                          get_cache_key, "account", shard="listing")
+        self.assertRaises(ValueError,
+                          get_cache_key, "account", "cont", "obj",
+                          shard="listing")
+
     def test_get_container_info_env(self):
         cache_key = get_cache_key("account", "cont")
         req = Request.blank(
@@ -509,6 +520,16 @@ class TestFuncs(BaseTest):
         check_not_in_cache(req, acct_cache_key)
         check_not_in_cache(req, cont_cache_key)
 
+        # check shard cache-keys
+        shard_cache_key = get_cache_key('account', 'cont', shard='listing')
+        shard_data = [{'shard': 'ranges'}]
+        req.environ['swift.infocache'][shard_cache_key] = shard_data
+        req.environ['swift.cache'].set(shard_cache_key, shard_data, time=600)
+        check_in_cache(req, shard_cache_key)
+        clear_info_cache('app-is-unused', req.environ, 'account', 'cont',
+                         shard='listing')
+        check_not_in_cache(req, shard_cache_key)
+
     def test_get_account_info_swift_source(self):
         app = FakeApp()
         req = Request.blank("/v1/a", environ={'swift.cache': FakeCache()})
@@ -718,6 +739,101 @@ class TestFuncs(BaseTest):
             resp,
             headers_to_container_info(headers.items(), 200))
 
+    def test_headers_from_container_info(self):
+        self.assertIsNone(headers_from_container_info(None))
+        self.assertIsNone(headers_from_container_info({}))
+
+        meta = {'fruit': 'cake'}
+        sysmeta = {'green': 'land'}
+        info = {
+            'status': 200,
+            'read_acl': 'my-read-acl',
+            'write_acl': 'my-write-acl',
+            'sync_to': 'my-sync-to',
+            'sync_key': 'my-sync-key',
+            'object_count': 99,
+            'bytes': 999,
+            'versions': 'my-versions',
+            'storage_policy': '0',
+            'cors': {
+                'allow_origin': 'my-cors-origin',
+                'expose_headers': 'my-cors-hdrs',
+                'max_age': 'my-cors-age'},
+            'created_at': '123.456_12',
+            'put_timestamp': '234.567_34',
+            'delete_timestamp': '345_67',
+            'status_changed_at': '246.8_9',
+            'meta': meta,
+            'sysmeta': sysmeta,
+            'sharding_state': 'unsharded'
+        }
+
+        res = headers_from_container_info(info)
+
+        expected = {
+            'X-Backend-Delete-Timestamp': '345_67',
+            'X-Backend-Put-Timestamp': '234.567_34',
+            'X-Backend-Sharding-State': 'unsharded',
+            'X-Backend-Status-Changed-At': '246.8_9',
+            'X-Backend-Storage-Policy-Index': '0',
+            'X-Backend-Timestamp': '123.456_12',
+            'X-Container-Bytes-Used': '999',
+            'X-Container-Meta-Fruit': 'cake',
+            'X-Container-Object-Count': '99',
+            'X-Container-Read': 'my-read-acl',
+            'X-Container-Sync-Key': 'my-sync-key',
+            'X-Container-Sync-To': 'my-sync-to',
+            'X-Container-Sysmeta-Green': 'land',
+            'X-Container-Write': 'my-write-acl',
+            'X-Put-Timestamp': '0000000234.56700',
+            'X-Storage-Policy': 'zero',
+            'X-Timestamp': '0000000123.45600',
+            'X-Versions-Location': 'my-versions',
+            'X-Container-Meta-Access-Control-Allow-Origin': 'my-cors-origin',
+            'X-Container-Meta-Access-Control-Expose-Headers': 'my-cors-hdrs',
+            'X-Container-Meta-Access-Control-Max-Age': 'my-cors-age',
+        }
+
+        self.assertEqual(expected, res)
+
+        for required in (
+                'created_at', 'put_timestamp', 'delete_timestamp',
+                'status_changed_at', 'storage_policy', 'object_count', 'bytes',
+                'sharding_state'):
+            incomplete_info = dict(info)
+            incomplete_info.pop(required)
+            self.assertIsNone(headers_from_container_info(incomplete_info))
+
+        for hdr, optional in (
+                ('X-Container-Read', 'read_acl'),
+                ('X-Container-Write', 'write_acl'),
+                ('X-Container-Sync-Key', 'sync_key'),
+                ('X-Container-Sync-To', 'sync_to'),
+                ('X-Versions-Location', 'versions'),
+                ('X-Container-Meta-Fruit', 'meta'),
+                ('X-Container-Sysmeta-Green', 'sysmeta'),
+        ):
+            incomplete_info = dict(info)
+            incomplete_info.pop(optional)
+            incomplete_expected = dict(expected)
+            incomplete_expected.pop(hdr)
+            self.assertEqual(incomplete_expected,
+                             headers_from_container_info(incomplete_info))
+
+        for hdr, optional in (
+            ('Access-Control-Allow-Origin', 'allow_origin'),
+            ('Access-Control-Expose-Headers', 'expose_headers'),
+            ('Access-Control-Max-Age', 'max_age'),
+        ):
+            incomplete_info = dict(info)
+            incomplete_cors = dict(info['cors'])
+            incomplete_cors.pop(optional)
+            incomplete_info['cors'] = incomplete_cors
+            incomplete_expected = dict(expected)
+            incomplete_expected.pop('X-Container-Meta-' + hdr)
+            self.assertEqual(incomplete_expected,
+                             headers_from_container_info(incomplete_info))
+
     def test_container_info_needs_req(self):
         base = Controller(self.app)
         base.account_name = 'a'
@@ -966,7 +1082,25 @@ class TestFuncs(BaseTest):
         dst_headers = base.generate_request_headers(req, transfer=True)
         expected_headers = {'x-base-meta-owner': '',
                             'x-base-meta-size': '151M',
-                            'connection': 'close'}
+                            'connection': 'close',
+                            'user-agent': 'proxy-server %d' % os.getpid()}
+        for k, v in expected_headers.items():
+            self.assertIn(k, dst_headers)
+            self.assertEqual(v, dst_headers[k])
+        self.assertNotIn('new-owner', dst_headers)
+
+    def test_generate_request_headers_change_backend_user_agent(self):
+        base = Controller(self.app)
+        self.app.backend_user_agent = "swift-flux-capacitor"
+        src_headers = {'x-remove-base-meta-owner': 'x',
+                       'x-base-meta-size': '151M',
+                       'new-owner': 'Kun'}
+        req = Request.blank('/v1/a/c/o', headers=src_headers)
+        dst_headers = base.generate_request_headers(req, transfer=True)
+        expected_headers = {'x-base-meta-owner': '',
+                            'x-base-meta-size': '151M',
+                            'connection': 'close',
+                            'user-agent': 'swift-flux-capacitor'}
         for k, v in expected_headers.items():
             self.assertIn(k, dst_headers)
             self.assertEqual(v, dst_headers[k])
@@ -1128,7 +1262,7 @@ class TestFuncs(BaseTest):
             client_chunks = list(app_iter)
         self.assertEqual(client_chunks, [b'abcd1234', b'efgh5678'])
 
-    def test_disconnected_warning(self):
+    def test_disconnected_logging(self):
         self.app.logger = mock.Mock()
         req = Request.blank('/v1/a/c/o')
 
@@ -1155,7 +1289,7 @@ class TestFuncs(BaseTest):
             'some-path', {})
         app_iter = handler._make_app_iter(req, node, source)
         app_iter.close()
-        self.app.logger.warning.assert_called_once_with(
+        self.app.logger.info.assert_called_once_with(
             'Client disconnected on read of %r', 'some-path')
 
         self.app.logger = mock.Mock()
@@ -1212,7 +1346,8 @@ class TestFuncs(BaseTest):
             body_iter=iter([b'', json.dumps(shard_ranges).encode('ascii')]),
             headers=resp_headers
         ) as fake_conn:
-            actual = base._get_shard_ranges(req, 'a', 'c')
+            actual, resp = base._get_shard_ranges(req, 'a', 'c')
+        self.assertEqual(200, resp.status_int)
 
         # account info
         captured = fake_conn.requests
@@ -1243,7 +1378,8 @@ class TestFuncs(BaseTest):
                             json.dumps(shard_ranges[1:2]).encode('ascii')]),
             headers=resp_headers
         ) as fake_conn:
-            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+            actual, resp = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertEqual(200, resp.status_int)
 
         # account info
         captured = fake_conn.requests
@@ -1267,7 +1403,8 @@ class TestFuncs(BaseTest):
         headers = {'X-Backend-Record-Type': 'shard'}
         with mocked_http_conn(200, 200, body_iter=iter([b'', body]),
                               headers=headers):
-            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+            actual, resp = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertEqual(200, resp.status_int)
         self.assertIsNone(actual)
         lines = self.app.logger.get_lines_for_level('error')
         return lines
@@ -1311,7 +1448,8 @@ class TestFuncs(BaseTest):
         body = json.dumps([dict(sr)]).encode('ascii')
         with mocked_http_conn(
                 200, 200, body_iter=iter([b'', body])):
-            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+            actual, resp = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertEqual(200, resp.status_int)
         self.assertIsNone(actual)
         error_lines = self.app.logger.get_lines_for_level('error')
         self.assertIn('Failed to get shard ranges', error_lines[0])
@@ -1328,7 +1466,8 @@ class TestFuncs(BaseTest):
         with mocked_http_conn(
                 200, 200, body_iter=iter([b'', body]),
                 headers=headers):
-            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+            actual, resp = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertEqual(200, resp.status_int)
         self.assertIsNone(actual)
         error_lines = self.app.logger.get_lines_for_level('error')
         self.assertIn('Failed to get shard ranges', error_lines[0])
@@ -1340,7 +1479,8 @@ class TestFuncs(BaseTest):
         base = Controller(self.app)
         req = Request.blank('/v1/a/c/o', method='PUT')
         with mocked_http_conn(200, 404, 404, 404):
-            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+            actual, resp = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertEqual(404, resp.status_int)
         self.assertIsNone(actual)
         self.assertFalse(self.app.logger.get_lines_for_level('error'))
         warning_lines = self.app.logger.get_lines_for_level('warning')
@@ -1355,7 +1495,7 @@ class TestNodeIter(BaseTest):
     def test_iter_default_fake_ring(self):
         for ring in (self.account_ring, self.container_ring):
             self.assertEqual(ring.replica_count, 3.0)
-            node_iter = NodeIter(self.app, ring, 0)
+            node_iter = NodeIter(self.app, ring, 0, self.logger)
             self.assertEqual(6, node_iter.nodes_left)
             self.assertEqual(3, node_iter.primaries_left)
             count = 0
@@ -1369,7 +1509,8 @@ class TestNodeIter(BaseTest):
     def test_iter_with_handoffs(self):
         ring = FakeRing(replicas=3, max_more_nodes=20)  # handoffs available
         policy = StoragePolicy(0, 'zero', object_ring=ring)
-        node_iter = NodeIter(self.app, policy.object_ring, 0, policy=policy)
+        node_iter = NodeIter(self.app, policy.object_ring, 0, self.logger,
+                             policy=policy)
         self.assertEqual(6, node_iter.nodes_left)
         self.assertEqual(3, node_iter.primaries_left)
         primary_indexes = set()
@@ -1392,10 +1533,12 @@ class TestNodeIter(BaseTest):
         policy = StoragePolicy(0, 'ec', object_ring=ring)
 
         # sanity
-        node_iter = NodeIter(self.app, policy.object_ring, 0, policy=policy)
+        node_iter = NodeIter(self.app, policy.object_ring, 0, self.logger,
+                             policy=policy)
         self.assertEqual(16, len([n for n in node_iter]))
 
-        node_iter = NodeIter(self.app, policy.object_ring, 0, policy=policy)
+        node_iter = NodeIter(self.app, policy.object_ring, 0, self.logger,
+                             policy=policy)
         self.assertEqual(16, node_iter.nodes_left)
         self.assertEqual(8, node_iter.primaries_left)
         pile = GreenAsyncPile(5)
